/*
 * $Id: XmlRpcException.java 84 2007-11-22 14:10:57Z may $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 12, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmlrpc/XmlRpcException.java
 * $
 * 
 * $LastChangedDate: 2007-11-22 15:10:57 +0100 (Thu, 22 Nov 2007) $ $LastChangedBy: may $
 * $LastChangedRevision: 84 $
 */
/**
 *
 */
package de.starface.com.rpc;


/**
 * Exception which might occur during processing of an RPC request. Such a exception carries a error
 * code to the corresponding error message. Both are used in the RPC response which is specified at
 * <a href="http://www.xmlrpc.com/spec">http://www.xmlrpc.com/spec</a>
 *
 * The error codes that are known are defined in the class {@link Codes}
 *
 * @author may
 *
 */
public class RpcException extends Exception {
	/**
	 * This class holds constants for some common error codes used by {@link RpcException}.
	 *
	 * @author tilmann
	 */
	public static final class Codes {

		/**
		 * This class holds constants for errors used by {@link RpcException} that occur in the RPC
		 * client.
		 *
		 * @author tilmann
		 */
		public static final class Client {
			/** Java could not create a proxy implementation for a given interface */
			public static final int PROXY_CREATION_FAILED = -1;

			/** No auth token converter has been registered for the given server auth thoken type */
			public static final int AUTH_CONVERTER_MISSING = -3;

			/** The auth token converter threw an exception during execution */
			public static final int AUTH_CONVERTER_EXCEPTION = -4;

			/** The server auth token can not be converted using the registered auth token converter */
			public static final int AUTH_UNKNOWN = -5;

			/** No server token has been provided and no default server token has been set */
			public static final int DEFAULT_SERVER_TOKEN_MISSING = -6;

			/** The xml-rpc agent cannot be instantiated */
			public static final int AGENT_CREATION_FAILED = -7;

			/** An error has occurred in the underlying transport protocol */
			public static final int TRANSPORT_FAILURE = -8;

			/** The xml-rpc server did not sent a response for the call */
			public static final int NO_RESPONSE_FROM_SERVER = -9;

			/** The returned result from the server does not match the proxy method's return type */
			public static final int RETURN_TYPE_MISMATCH = -10;

			/** Could not parse a valid XML-RPC methodResponse from the transport payload */
			public static final int XML_PARSE_ERROR = -11;

			/** Could not serialize a valid XML-RPC methodCall as transport payload */
			public static final int XML_SERIALIZATION_ERROR = -12;

			/** The RPC interface method declares a return type that is invalid */
			public static final int RETURN_TYPE_INVALID = -13;

			//public static final int PARAMETER_TYPE_INVALID = -14;

			/** The value translation of the request parameters or return values has failed */
			public static final int VALUE_TRANSLATION_FAILED = -15;

			/** The server implementation has prepared a throwable to be rethrown on the client */
			public static final int PREPARED_CLIENT_THROWABLE = -16;

			/**
			 * The server implementation or transport infrastructure is processing asynchronous
			 * calls too slowly
			 */
			public static final int ASYNC_CALL_BUFFER_OVERFLOW = -17;

			/** The exception converter provided by the client implementation threw an exception */
			public static final int EXCEPTION_CONVERTER_ERROR = -17;

			/** The name of the xmpp service can not be determined due to missing xmpp connection */
			public static final int XMPP_SERVICE_NAME_MISSING = -101;

			/** A direct transport server with that name cannot be found */
			public static final int DIRECT_SERVER_NOT_REGISTERED = -201;
		}

		/**
		 * This class holds constants for errors used by {@link RpcException} that occour in the
		 * xml-rpc server.
		 *
		 * @author tilmann
		 */
		public static final class Server {
			/** The remote procedure threw an exception during execution */
			public static final int APPLICATION_EXCEPTION = 1;

			/** The RPC server does not provide a remote procedure with the given signature */
			public static final int PROCEDURE_NOT_AVAILABLE = 2;

			/** No auth token converter has been registered for the given client auth token type */
			public static final int AUTH_CONVERTER_MISSING = 3;

			/** The auth token converter threw an exception during execution */
			public static final int AUTH_CONVERTER_EXCEPTION = 4;

			/**
			 * The client transport token can not be converted using the registered auth token
			 * converter
			 */
			public static final int AUTH_UNKNOWN = 5;

			/** The RPC server could not execute the procedure */
			public static final int PROCEDURE_EXECUTION_FAILED = 6;

			/** An unexpected error has occurred in the code of the RPC server */
			public static final int SERVER_ERROR = 7;

			/** An error has occurred in the underlying transport protocol */
			public static final int TRANSPORT_FAILURE = 8;

			/**
			 * Could not parse a valid XML-RPC methodCall from the transport payload
			 */
			public static final int XML_PARSE_ERROR = 11;

			/**
			 * Could not serialize a valid XML-RPC methodResponse as transport payload
			 */
			public static final int XML_SERIALIZATION_ERROR = 12;

			//public static final int RETURN_TYPE_INVALID = 13;

			//public static final int PARAMETER_TYPE_INVALID = 14;

			/** The value translation of the request parameters or return values has failed */
			public static final int VALUE_TRANSLATION_FAILED = 15;

			/** The server implementation has prepared a throwable to be rethrown on the client */
			public static final int PREPARED_CLIENT_THROWABLE = 16;

			/** A direct transport server with that name cannot be found */
			public static final int DIRECT_SERVER_NOT_REGISTERED = 201;

			/** A direct transport server with that name has already been started */
			public static final int DIRECT_SERVER_ALREADY_REGISTERED = 202;
		}
	}

	/** Error code */
	private final int code;

	private static final long serialVersionUID = -5442770316775135828L;

	/**
	 * @param errorCode
	 *        Sets the error code
	 * @param errorMessage
	 *        Sets the error message
	 */
	public RpcException(int errorCode, String errorMessage) {
		super(errorMessage);
		code = errorCode;
	}

	/**
	 * @param errorCode
	 *        Sets the error code
	 * @param errorMessage
	 *        Sets the error message
	 * @param cause
	 *        The cause
	 */
	public RpcException(int errorCode, String errorMessage, Throwable cause) {
		super(errorMessage, cause);
		code = errorCode;
	}

	/**
	 * @return Returns the error code.
	 */
	public int getCode() {
		return code;
	}
}
