package de.starface.com.rpc;

/**
 * This is a {@link RuntimeException} subclass that is thrown in cases when a {@link RpcException}
 * occurs during a RPC and the called method of the RPC proxy does not declare {@link RpcException}
 * to be thrown. The originating {@link RpcException} can be obtained from this RpcRuntimeException.
 *
 * @author tilmann
 *
 */
public class RpcRuntimeException extends RuntimeException {

	private static final long serialVersionUID = -1886989427159206949L;

	/**
	 * Creates a new RpcRuntimeException with the given {@link RpcException} as cause using the
	 * message from the given cause.
	 *
	 * @param rpcException
	 *        the cause for this exception
	 */
	public RpcRuntimeException(RpcException rpcException) {
		super(rpcException.getMessage(), rpcException);
	}

	/**
	 * Retrieve the {@link RpcException} that caused this exception to be thrown.
	 *
	 * @see Throwable#getCause()
	 */
	@Override
	public RpcException getCause() {
		return (RpcException) super.getCause();
	}

	/**
	 * Convenience method to retrieve the error code from the underlying {@link RpcException}
	 *
	 * @return the error code from the underlying {@link RpcException}
	 * @see RpcException#getCode()
	 * @see RpcException.Codes
	 */
	public int getCode() {
		return getCause().getCode();
	}
}
