/*
 * $Id: RpcAsynchronous.java 383 2011-05-23 12:50:02Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 23, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmlrpc/annotation/Asynchronous
 * .java $
 * 
 * $LastChangedDate: 2011-05-23 14:50:02 +0200 (Mon, 23 May 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 383 $
 */
package de.starface.com.rpc.annotation;

import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * Annotation to mark methods as asynchronous calls for the RPC client.
 *
 * <p>
 * When a {@link de.starface.com.rpc.client.RpcClientBase} creates a proxy, this annoation is taken
 * into account. All methods labeled with this annotations will produce asynchronous calls to the
 * server. This means that such methods return immediately instead of waiting for the response from
 * the RPC server and implement an "At most once" call semantic. This means there is no guarantee
 * that a call actually arrives at the server! As a constraint methods annotated with this
 * annotation must have the return type void.
 * </p>
 *
 * @author may
 */
@Inherited
@Retention(RUNTIME)
@Target({METHOD, TYPE})
public @interface RpcAsynchronous {
	/**
	 * The default value for all RPC calls not specifying an RpcAsynchronous annotation. Change this
	 * value to change the behavior for all those calls.
	 */
	public static boolean DEFAULT_BEHAVIOUR = false;

	/**
	 * Specifies if the RPC call should be asynchronous with a default value of true if none is
	 * given.
	 */
	boolean value() default true;
}
