package de.starface.com.rpc.annotation;

import de.starface.com.rpc.common.authentication.RpcAuthToken;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * RPC interfaces and/or implementing classes or the contained RPC methods may be annotated with
 * this annotation to require authentication on incoming RPC calls on the Server. The value given
 * for the annotation specifies the required implementation of {@link RpcAuthToken}. If both RPC
 * interface and implementing class are annotated then the annotation of the class takes precedence.
 * Method annotation takes precedence over class/interface annotation.
 *
 * @author tilmann
 */
@Documented
@Inherited
@Retention(RUNTIME)
@Target({METHOD, TYPE})
public @interface RpcAuthRequired {
	/**
	 * Specifies the required auth token type which must be a subclass of {@link RpcAuthToken}. The
	 * type {@link RpcAuthToken} is used as a default if no value has been given.
	 */
	Class<? extends RpcAuthToken> value() default RpcAuthToken.class;
}
