/**
 *
 */
package de.starface.com.rpc.annotation;

import de.starface.com.rpc.client.RpcClient;
import de.starface.com.rpc.server.RpcObjectRegistry;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * Annotation to define custom identifiers for remotely callable methods via RPC.
 *
 * <p>
 * When the {@link RpcObjectRegistry} registers methods of an interface or an {@link RpcClient}
 * creates an RPC proxy for remote calling, this annotation is evaluated. This annotation can be
 * seen as extension of the {@link RpcProcedure} annotation and the provided value will be used as a
 * prefix for the identifier set by the RpcProcedure annotation.
 * </p>
 *
 * @author may
 */
@Documented
@Inherited
@Retention(RUNTIME)
@Target(TYPE)
public @interface RpcInterface {
	/** Identifiers for which the methods of the interface should be registered */
	String value();
}
