package de.starface.com.rpc.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * This annotation can be used to specify the name and {@link RpcValueType} to be used to translate
 * the annotated {@link RpcValueObject}s fields value during RPC value translation and to restrict
 * the translation of the value to specific value translation version.
 *
 * @author tilmann
 */
@Documented
@Inherited
@Retention(RUNTIME)
@Target(FIELD)
public @interface RpcValue {
	/**
	 * The name of the annotated fields value during RPC. If not specified the name of the field is
	 * used.
	 */
	String value() default "";

	/**
	 * The annotated fields value is not translated if the value translation version is lower than
	 * the specified minVersion. If not specified the value is translated for all versions.
	 */
	int minVersion() default 0;

	/**
	 * The annotated fields value is not translated if the value translation version is higher than
	 * the specified maxVersion. If not specified the value is translated for all versions.
	 */
	int maxVersion() default Integer.MAX_VALUE;

	/**
	 * The {@link RpcValueType} that should be used for the annotated fields value in the RPC call.
	 * If not specified a default of {@link RpcValueType#Automatic} is used.
	 */
	RpcValueType rpcType() default RpcValueType.Automatic;
}
