package de.starface.com.rpc.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * This annotation activates automatic value translation for RPC calls. The annotation can be
 * attached to RPC interface and/or methods. An annotation at the method takes precedence if both
 * are present.
 *
 * @author tilmann
 */
@Documented
@Inherited
@Retention(RUNTIME)
@Target({METHOD, TYPE})
public @interface RpcValueTranslation {
	/**
	 * This specifies the value translation behavior for all methods not annotated to be deactivated
	 * per default. Change this value to affect all these methods.
	 */
	public static boolean DEFAULT_BEHAVIOUR = true;

	/**
	 * This specifies the automatic file transfer for all methods not annotated to be deactivated
	 * per default. Change this value to affect all these methods.
	 */
	public static boolean DEFAULT_AUTOMATIC_FILE_TRANSFER = false;

	/**
	 * This specifies the value translation version for all methods not annotated to be 0 per
	 * default. Change this value to affect all these methods.
	 */
	public static int DEFAULT_VERSION = 0;

	/**
	 * Specify if value translation should be activated for the annotated methods(s). If no value is
	 * given the translation is activated.
	 */
	boolean value() default true;

	/**
	 * Specify the value translation version for the annotated methods(s). If no value is given the
	 * version 0 is used.
	 */
	int version() default 0;

	/**
	 * Specify if automatic file transfer should be activated for the annotated methods(s). If no
	 * value is given the file transfer is deactivated.
	 */
	boolean automaticFileTransfer() default false;
}
