package de.starface.com.rpc.client;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcThreadFactory;
import de.starface.com.rpc.common.RpcTransportToken;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class AsynchronousExecutor {
	private Map<RpcTransportToken, Executor> executors = new HashMap<>();

	private synchronized Executor getExecutor(RpcTransportToken transportToken) {
		Executor executor = executors.get(transportToken);
		if (executor == null) {
			executor = createExecutor(transportToken.getIdentifierString());
			executors.put(transportToken, executor);
		}
		return executor;
	}

	private static Executor createExecutor(String name) {
		// This means for each AsynchronousExecutorProvider all asynchronous calls will be executed
		// one after the other in a single thread and new calls exceeding 100 will be rejected
		return new ThreadPoolExecutor(0, 1, 30, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(100),
			new RpcThreadFactory("AsynchronousExecutor[" + name + "]"));
	}

	public void execute(Runnable runnable, RpcTransportToken transportToken) throws RpcException {
		try {
			getExecutor(transportToken).execute(runnable);
		} catch (RejectedExecutionException e) {
			throw new RpcException(RpcException.Codes.Client.ASYNC_CALL_BUFFER_OVERFLOW,
				"[" + Thread.currentThread().getName() + "] Too many asynchronous calls awaiting execution for this RPC client.", e);
		}
	}
}
