package de.starface.com.rpc.client;

import de.starface.com.rpc.common.RpcTransportToken;

/**
 * This interface represents a strategy that is used to prepare the exceptions thrown on the client
 * proxies. If an exception occurs during an RPC call it is handed over to a registered
 * RpcExceptionConverter that may alter/replace the exception to be thrown.
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        the transport token type of the request
 */
public interface RpcExceptionConverter<TransportToken extends RpcTransportToken> {
	/**
	 * Converts the given Throwable that occured during RPC to a Throwable that should be thrown in
	 * the proxy towards the client implementation using the RPC API
	 *
	 * @param t
	 *        the Throwable from the RPC call
	 * @param request
	 *        the request during which the Throwable was thrown
	 * @return the Throwable to be thrown on the proxy
	 */
	public Throwable convertExceptionToBeThrownOnProxy(Throwable t, OutgoingRequest<TransportToken> request);
}
