package de.starface.com.rpc.client.interceptor;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.annotation.RpcLoggingLevel;
import de.starface.com.rpc.client.OutgoingRequest;
import de.starface.com.rpc.common.LoggingUtils;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This request interceptor implements the aspect of logging RPC requests.
 *
 * @author tilmann
 */
public class LoggingRequestInterceptor implements OutgoingRequestInterceptor<RpcTransportToken> {
	private static final Log log = LogFactory.getLog(LoggingRequestInterceptor.class);

	/**
	 * @see de.starface.com.rpc.common.interceptor.RpcRequestInterceptor#interceptRequest(de.starface.com.rpc.common.RpcRequest,
	 *      de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink)
	 */
	public void interceptRequest(OutgoingRequest<RpcTransportToken> request,
								 RequestInterceptorChainLink interceptorChain) throws RpcException {
		RpcLoggingLevel logLevel = de.starface.com.rpc.server.interceptor.LoggingRequestInterceptor.getLogLevelForCall(
			request.getRpcInterface(), request.getRpcMethod());

		logOutgoingRequest(logLevel, request);

		interceptorChain.processRequest();

		logIncomingReturnValue(logLevel, request);
	}

	private static void logOutgoingRequest(RpcLoggingLevel level, OutgoingRequest<RpcTransportToken> request) {
		if (level != null) {
			switch (level) {
				case TRACE:
					if (log.isTraceEnabled()) {
						log.trace(LoggingUtils.outgoingRequestToString(request));
					}
					break;
				case DEBUG:
					if (log.isDebugEnabled()) {
						log.debug(LoggingUtils.outgoingRequestToString(request));
					}
					break;
				case INFO:
					if (log.isInfoEnabled()) {
						log.info(LoggingUtils.outgoingRequestToString(request));
					}
					break;
				case WARN:
					if (log.isWarnEnabled()) {
						log.warn(LoggingUtils.outgoingRequestToString(request));
					}
					break;
				case ERROR:
					if (log.isErrorEnabled()) {
						log.error(LoggingUtils.outgoingRequestToString(request));
					}
					break;
				case FATAL:
					if (log.isFatalEnabled()) {
						log.fatal(LoggingUtils.outgoingRequestToString(request));
					}
					break;
				default:
					break;
			}
		}
	}

	private static void logIncomingReturnValue(RpcLoggingLevel level, OutgoingRequest<RpcTransportToken> request) {
		if (level != null) {
			switch (level) {
				case TRACE:
					if (log.isTraceEnabled()) {
						log.trace(LoggingUtils.incomingReturnValueToString(request));
					}
					break;
				case DEBUG:
					if (log.isDebugEnabled()) {
						log.debug(LoggingUtils.incomingReturnValueToString(request));
					}
					break;
				case INFO:
					if (log.isInfoEnabled()) {
						log.info(LoggingUtils.incomingReturnValueToString(request));
					}
					break;
				case WARN:
					if (log.isWarnEnabled()) {
						log.warn(LoggingUtils.incomingReturnValueToString(request));
					}
					break;
				case ERROR:
					if (log.isErrorEnabled()) {
						log.error(LoggingUtils.incomingReturnValueToString(request));
					}
					break;
				case FATAL:
					if (log.isFatalEnabled()) {
						log.fatal(LoggingUtils.incomingReturnValueToString(request));
					}
					break;
				default:
					break;
			}
		}
	}
}
