package de.starface.com.rpc.common;

import de.starface.com.rpc.client.OutgoingRequest;
import de.starface.com.rpc.server.IncomingRequest;

import java.util.List;
import java.util.Map;

/**
 * This class contains some utility methods used for the logging of RPC requests
 *
 * @author tilmann
 */
public class LoggingUtils {
	private LoggingUtils() {
		// No instances will be made
	}

	private static void appendRpcMap(StringBuilder sb, Map<?, ?> rpcMap) {
		sb.append("{");
		int count = 0;
		for (Map.Entry<?, ?> entry : rpcMap.entrySet()) {
			sb.append(entry.getKey());
			sb.append("=");
			appendRpcValue(sb, entry.getValue());
			if (++count < rpcMap.size()) {
				sb.append(", ");
			}
		}
		sb.append("}");
	}

	private static void appendRpcList(StringBuilder sb, List<?> rpcList) {
		sb.append("[");
		int count = 0;
		for (Object entry : rpcList) {
			appendRpcValue(sb, entry);
			if (++count < rpcList.size()) {
				sb.append(", ");
			}
		}
		sb.append("]");
	}

	private static void appendRpcValue(StringBuilder sb, Object rpcValue) {
		if (rpcValue == null) {
			sb.append("<null>");
		} else if (rpcValue instanceof List) {
			appendRpcList(sb, (List<?>) rpcValue);
		} else if (rpcValue instanceof Map) {
			appendRpcMap(sb, (Map<?, ?>) rpcValue);
		} else if (rpcValue instanceof String) {
			sb.append("\"");
			sb.append(rpcValue.toString());
			sb.append("\"");
		} else if (rpcValue instanceof Byte[]) {
			sb.append("<binary data>");
		} else {
			sb.append(rpcValue.toString());
		}
	}

	public static String rpcParametersToString(Object[] parameters) {
		StringBuilder sb = new StringBuilder();
		appendRpcParameters(sb, parameters);
		return sb.toString();
	}

	private static void appendRpcParameters(StringBuilder sb, Object[] parameters) {
		sb.append("(");
		if (parameters != null) {
			int count = 0;
			for (Object parameter : parameters) {
				appendRpcValue(sb, parameter);
				if (++count < parameters.length) {
					sb.append(", ");
				}
			}
		}
		sb.append(")");
	}

	public static String outgoingRequestToString(OutgoingRequest<?> request) {
		StringBuilder sb = new StringBuilder();
		sb.append("Sending ");
		sb.append("RPC request: ");
		sb.append(request.getRpcMethodName());
		appendRpcParameters(sb, request.getParameters());
		sb.append(" to ");
		sb.append(request.getTargetTransportToken());
		return sb.toString();
	}

	public static String incomingRequestToString(IncomingRequest<?> request) {
		StringBuilder sb = new StringBuilder();
		sb.append("Incoming RPC request: ");
		sb.append(request.getRpcMethodName());
		appendRpcParameters(sb, request.getParameters());
		sb.append(" from ");
		sb.append(request.getSourceTransportToken());
		return sb.toString();
	}

	public static String incomingReturnValueToString(OutgoingRequest<?> request) {
		StringBuilder sb = new StringBuilder();
		sb.append("Incoming RPC return value: ");
		appendRpcValue(sb, request.getReturnValue());
		sb.append(" from ");
		sb.append(request.getTargetTransportToken());
		return sb.toString();
	}

	public static String outgoingReturnValueToString(IncomingRequest<?> request) {
		StringBuilder sb = new StringBuilder();
		sb.append("Sending RPC return value: ");
		appendRpcValue(sb, request.getReturnValue());
		sb.append(" to ");
		sb.append(request.getSourceTransportToken());
		return sb.toString();
	}
}
