/*
 * $Id: AuthTokenConverter.java 68 2007-11-14 15:04:24Z may $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 9, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmlrpc/server/AuthTokenConverter
 * .java $
 * 
 * $LastChangedDate: 2007-11-14 16:04:24 +0100 (Wed, 14 Nov 2007) $ $LastChangedBy: may $
 * $LastChangedRevision: 68 $
 */
/**
 *
 */
package de.starface.com.rpc.common.authentication;

import de.starface.com.rpc.annotation.RpcAuthRequired;
import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.server.RequestExecutorImpl;
import de.starface.com.rpc.server.RpcMethod;
import de.starface.com.rpc.server.RpcServer;

/**
 * This interface represents the core ingredient for authentication via transport channel. This
 * converter resolves a {@link RpcAuthToken} from the given {@link RpcTransportToken} and vice
 * versa.
 *
 * <p>
 * Setting an RpcAuthTokenConverter in {@link RpcServer#enableAuthentication(RpcAuthTokenConverter)}
 * is <i>necessary</i> in order make authentication available for {@link RpcRequest}s. A request can
 * only be routed to an {@link RpcMethod} which requires authentication using the
 * {@link RpcAuthRequired} annotation when a matching converter is found for the pair of
 * {@link RpcTransportToken} and {@link RpcAuthToken}. The converter is only called when a actual
 * RPC request is processed in the {@link RequestExecutorImpl}. Requests for an processor which
 * requires authentication are always discarded when no fitting converter was found or the converter
 * cannot resolve any token (i.e. returns null or throws exceptions). Such requests will always
 * result in an error response to the client.
 * </p>
 *
 * @author may
 *
 * @param <TransportToken>
 *        The type of TransportToken to identify a specific transport channel
 * @param <AuthToken>
 *        The type of AuthToken to identify the authentication method
 *
 * @see RpcAuthToken
 * @see RequestExecutorImpl
 * @see RpcAuthRequired
 */
public interface RpcAuthTokenConverter<TransportToken extends RpcTransportToken, AuthToken extends RpcAuthToken> {

	/**
	 * Converts a given {@link RpcAuthToken} to the corresponding {@link RpcTransportToken}
	 *
	 * @param auth
	 *        The AuthToken to convert
	 * @return The corresponding TransportToken
	 */
	public AuthToken convert(TransportToken auth);

	/**
	 * Converts a given {@link RpcTransportToken} to the corresponding {@link RpcAuthToken}
	 *
	 * @param token
	 *        The TransportToken to convert
	 * @return The corresponding AuthToken
	 */
	public TransportToken convert(AuthToken token);

	/**
	 * Returns the type of the {@link RpcTransportToken} for determination of the fitting processor.
	 *
	 * @return Class of the {@link RpcTransportToken}
	 */
	public Class<TransportToken> getTransportType();

	/**
	 * Returns the type of the {@link RpcAuthToken} for determination of the fitting processor.
	 *
	 * @return Class of the {@link RpcAuthToken}
	 */
	public Class<AuthToken> getAuthType();
}
