package de.starface.com.rpc.direct;

import de.starface.com.rpc.client.OutgoingRequestProcessor;
import de.starface.com.rpc.client.RpcClient;
import de.starface.com.rpc.client.RpcClientBase;

/**
 * This RPC client allows to calls RPC methods serviced inside a single Java class loader namespace
 * by a {@link DirectRpcServer}.
 *
 * @author tilmann
 *
 */
public class DirectRpcClient extends RpcClientBase<DirectTransportToken> {
	private DirectTransportToken callbackServerToken;

	public static RpcClient<DirectTransportToken> createWithDefaultServer(String defaultServerName) {
		DirectRpcClient client = new DirectRpcClient();
		client.setDefaultServerName(defaultServerName);
		return client;
	}

	public static RpcClient<DirectTransportToken> createWithDefaultServer(DirectTransportToken defaultServerToken) {
		DirectRpcClient client = new DirectRpcClient();
		client.setDefaultServerTransportToken(defaultServerToken);
		return client;
	}

	public static RpcClient<DirectTransportToken> createWithCallbackServer(String callbackServerName) {
		DirectRpcClient client = new DirectRpcClient();
		client.setCallbackServerName(callbackServerName);
		return client;
	}

	public static RpcClient<DirectTransportToken> createWithCallbackServer(DirectTransportToken callbackServerToken) {
		DirectRpcClient client = new DirectRpcClient();
		client.setCallbackServerToken(callbackServerToken);
		return client;
	}

	public static RpcClient<DirectTransportToken> createWithDefaultAndCallbackServer(String defaultServerName,
																					 String callbackServerName) {
		DirectRpcClient client = new DirectRpcClient();
		client.setDefaultServerName(defaultServerName);
		client.setCallbackServerName(callbackServerName);
		return client;
	}

	public static RpcClient<DirectTransportToken> createWithDefaultAndCallbackServer(DirectTransportToken defaultServerToken,
																					 DirectTransportToken callbackServerToken) {
		DirectRpcClient client = new DirectRpcClient();
		client.setDefaultServerTransportToken(defaultServerToken);
		client.setCallbackServerToken(callbackServerToken);
		return client;
	}

	private void setCallbackServerName(String callbackServerName) {
		this.callbackServerToken = new DirectTransportToken(callbackServerName);
	}

	private void setCallbackServerToken(DirectTransportToken callbackServerToken) {
		this.callbackServerToken = callbackServerToken;
	}

	/**
	 * Sets the default server transport token this {@link RpcClientBase} uses based on the given
	 * name of a {@link DirectRpcServer}.
	 *
	 * @param serverName
	 *        the name of a {@link DirectRpcServer} to be used for the default server transport
	 *        token
	 * @see RpcClientBase#setDefaultServerTransportToken(de.starface.com.rpc.RpcTransportToken)
	 */
	private void setDefaultServerName(String serverName) {
		DirectTransportToken defaultRpcServerTransportToken = new DirectTransportToken(serverName);
		setDefaultServerTransportToken(defaultRpcServerTransportToken);
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClientBase#createOutgoingRequestProcessor()
	 */
	@Override
	protected OutgoingRequestProcessor<DirectTransportToken> createOutgoingRequestProcessor() {
		return new DirectRpcOutgoingRequestProcessor(callbackServerToken);
	}
}
