package de.starface.com.rpc.direct;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.OutgoingRequestProcessor;
import de.starface.com.rpc.client.OutgoingRequestProcessorBase;
import de.starface.com.rpc.server.RequestExecutor;
import de.starface.com.rpc.server.RequestExecutorFactory;

/**
 * This is a {@link OutgoingRequestProcessor} implementation for the direct RPC calls.
 *
 * To perform a call it retrieves the fitting {@link RequestExecutor} from the
 * {@link DirectRpcRequestExecutorFactoryRegistry} and executes it.
 *
 * @author tilmann
 *
 */
public class DirectRpcOutgoingRequestProcessor extends OutgoingRequestProcessorBase<DirectTransportToken> {
	private final DirectTransportToken callbackServerToken;

	private RequestExecutor<DirectTransportToken> requestExecutor = null;

	/**
	 * Sets the {@link DirectTransportToken} used to identify the source of the calls made by this
	 * DirectRpcRequestCaller. If <code>null</code> is set, no source address will be set.
	 *
	 * @param callbackServerToken
	 *        the new call back address to be set on outgoing calls
	 */
	DirectRpcOutgoingRequestProcessor(DirectTransportToken callbackServerToken) {
		if (callbackServerToken == null) {
			this.callbackServerToken = new DirectTransportToken();
		} else {
			this.callbackServerToken = callbackServerToken;
		}
	}

	/**
	 * @see de.starface.com.rpc.client.OutgoingRequestProcessorBase#processRequest()
	 */
	@Override
	protected void processRequest() throws RpcException {
		prepareRequestExecutor();

		Object returnValue = requestExecutor.execute(request.getRpcMethodName(), request.getParameters(),
			callbackServerToken);

		request.setReturnValue(returnValue);
	}

	private void prepareRequestExecutor() throws RpcException {
		DirectTransportToken serverTransportToken = request.getTargetTransportToken();
		String serverName = serverTransportToken.getServerName();

		DirectRpcRequestExecutorFactoryRegistry requestExecutorFactoryRegistry = DirectRpcRequestExecutorFactoryRegistry.getInstance();
		RequestExecutorFactory<DirectTransportToken> requestExecutorFactory = requestExecutorFactoryRegistry.lookupExecutorFactoryWithName(serverName);

		requestExecutor = requestExecutorFactory.createRpcRequestExecutor();
	}
}
