/**
 * This package contains the common RPC classes that are independent from the chosen transport
 * layer.
 *
 * <h1>The RPC server</h1>
 * <p>
 * A RPC server is a piece of software which processes incoming
 * {@link de.starface.com.rpc.common.RpcRequest}s. This aspect is encapsulated in the
 * {@link de.starface.com.rpc.server.RequestExecutor} which determines a
 * {@link de.starface.com.rpc.server.RpcMethod} for the request and computes a result to the request
 * by invoking this method.
 * </p>
 *
 * <h2>RPC objects and registration</h2>
 * <p>
 * All known {@link de.starface.com.rpc.server.RpcMethod}s are stored in a
 * {@link de.starface.com.rpc.server.RpcObjectRegistry} and have a unique identifier for each
 * method. When a new method is registered in the registry, requests carrying the identifier as
 * <i>methodName</i> are passed to this processor.
 * </p>
 *
 * <p>
 * Furthermore it is possible to register all methods of a Java<sup>TM</sup> interfaces in the
 * {@link de.starface.com.rpc.server.RpcObjectRegistry}. In order to do so, annotations are offered
 * to have a more precise influence on the registration of the methods.
 * </p>
 *
 * <h2>Authentication via transport</h2>
 * <p>
 * As a special feature the {@link de.starface.com.rpc.server.RpcServer} allows to set a
 * {@link de.starface.com.rpc.common.authentication.RpcAuthTokenConverter}. This will enable the
 * usage of the per-request authentication via transport for all methods annotated to require
 * authentication.
 * </p>
 *
 * <h1>The RPC proxy factory</h1>
 * <p>
 * The client implementations available include implementations for XMPP (based on Jivesoft SMACK)
 * and HTTP (based on Apache Http Client) and direct Java call usable inside a single VM only.
 * </p>
 *
 * <h2>Using proxies</h2>
 * <p>
 * The {@link de.starface.com.rpc.client.RpcClientBase} is the part of the client and used to create
 * the proxies that are used to invoke remote procedures from user's code.
 * </p>
 */
package de.starface.com.rpc;
