package de.starface.com.rpc.services.filetransfer;

/**
 * A FileTransfer implementation provides information about a file transfer being executed
 *
 * @author tilmann
 */
public interface FileTransfer {

	/**
	 * Supported hash algorithms by the RPC file transfer
	 */
	public enum HashMethod {
		MD5,
		SHA;
	}

	/**
	 * Different states a file transfer can take
	 *
	 * @author tilmann
	 */
	public enum State {
		NEW,
		IN_PROGRESS,
		CANCELLED,
		TIMEOUT,
		FAILED,
		COMPLETED;
	}

	/**
	 * @return the id of this file transfer
	 */
	public String getId();

	/**
	 * @return the has method in use in this transfer
	 */
	public HashMethod getHashMethod();

	/**
	 * @return Returns the file.
	 */
	public RpcFile getRpcFile();

	/**
	 * @return Returns the send bytes.
	 */
	public long getTransferredBytes();

	/**
	 * @return {@code true} if all data has been transfered
	 */
	public boolean allBytesTransferred();

	/**
	 * @return Returns the progress.
	 */
	public double getProgress();

	/**
	 * @return the current state of this transfer
	 */
	public State getState();

	/**
	 * Cancels this transfer. This is not possible when the transfer is already completed.
	 */
	public void cancel();
}
