/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Oct 7, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer;

/**
 * This interface can be implemented to get progress, success and failure updates for an ongoing
 * file transfer.
 *
 * @author may
 *
 * @see RpcFileUploadAgent
 * @see RpcFileUploadService
 * @see RpcFileDownloadAgent
 * @see RpcFileDownloadService
 */
public interface FileTransferEventListener {
	/**
	 * This event is triggered when the file transfer is initialized.
	 *
	 * @param fileTransfer
	 *        an object containing detailed information about the transfer
	 */
	public void fileTransferInitialized(FileTransfer fileTransfer);

	/**
	 * This event is triggered repeatedly when the file transfer makes progress.
	 *
	 * @param fileTransfer
	 *        an object containing detailed information about the transfer
	 */
	public void fileTransferProgress(FileTransfer fileTransfer);

	/**
	 * This event is triggered when the file transfer lead to an error.
	 *
	 * @param event
	 *        an object containing detailed information about the transfer
	 */
	public void fileTransferFailed(FileTransferFailedEvent event);

	/**
	 * This event is triggered when the file transfer has completed.
	 *
	 * @param fileTransfer
	 *        an object containing detailed information about the transfer
	 */
	public void fileTransferCompleted(FileTransfer fileTransfer);
}
