/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Sep 29, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer;

/**
 * An exception that is thrown when a file transfer fails
 *
 * @author tilmann
 */
public class FileTransferFailedException extends Exception implements FileTransferFailedEvent {
	private static final long serialVersionUID = -2573740260041251353L;

	private final int _code;

	private final FileTransfer _fileTransfer;

	public FileTransferFailedException(int errorCode, FileTransfer file, String message) {
		super(message);
		_code = errorCode;
		_fileTransfer = file;
	}

	public FileTransferFailedException(int errorCode, FileTransfer file, String message, Throwable cause) {
		super(message, cause);
		_code = errorCode;
		_fileTransfer = file;
	}

	/**
	 * The file transfer information of the transfer that caused the exception.
	 *
	 * @return Returns the file.
	 */
	public FileTransfer getFileTransfer() {
		return _fileTransfer;
	}

	/**
	 * The code with one of the values of {@link FileTransferFailedEvent.Codes} specifies the reason
	 * for this exception.
	 *
	 * @return the code of the reason
	 */
	public int getCode() {
		return _code;
	}

	/**
	 * A throwable, probably an exception, that lead to this Exception
	 *
	 * @return the cause of this Exception or this exception itself, if the cause is not set
	 */
	public Throwable getReasonThrowable() {
		Throwable cause = getCause();
		if (cause == null) {
			return this;
		} else {
			return cause;
		}
	}

	@Override
	public String toString() {
		return "FileTransferFailedException [_code=" + _code + ", _fileTransfer=" + _fileTransfer + "]";
	}
}
