/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Sep 26, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer;

import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.server.RpcMethod;
import de.starface.com.rpc.server.RpcObject;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.services.RpcService;
import de.starface.com.rpc.services.filetransfer.common.RpcFileUpload;
import de.starface.com.rpc.services.filetransfer.common.RpcFiletransferLegacy;
import de.starface.com.rpc.services.filetransfer.service.RpcFileUploadImpl;
import de.starface.com.rpc.services.filetransfer.service.RpcFiletransferLegacyAdapter;

/**
 * An RPC service which provides logic to the RPC server to accept files uploads and enables
 * corresponding notifications via event listeners.
 *
 * @author tilmann
 */
public class RpcFileUploadService implements RpcService {
	private static final Long DEFAULT_RPC_CALL_TIMEOUT = 1000L * 30;

	/** The actual implementation */
	private final RpcFileUploadImpl fileUpload;

	private final RpcObject<?> fileUploadRpcObject;

	/**
	 * Creates a new RpcFileUploadService
	 *
	 * @param namespace
	 *        To use as prefix for this service's {@link RpcMethod}s in the
	 *        {@link RpcObjectRegistry}
	 * @param requiredAuthType
	 *        the type of authentication required for the callers trying to upload files
	 * @param timeout
	 *        Time in milliseconds which might pass between two calls on an ongoing file transfer
	 *        before the server assumes the transfer to be failed
	 */
	public RpcFileUploadService(String namespace, Class<? extends RpcAuthToken> requiredAuthType, long timeout) {
		this(namespace, requiredAuthType, timeout, false);
	}

	/**
	 * Creates a new RpcFileUploadService with a default timeout of 30 seconds
	 *
	 * @param namespace
	 *        To use as prefix for this service's {@link RpcMethod}s in the
	 *        {@link RpcObjectRegistry}
	 * @param requiredAuthType
	 *        the type of authentication required for the callers trying to upload files
	 */
	public RpcFileUploadService(String namespace, Class<? extends RpcAuthToken> requiredAuthType) {
		this(namespace, requiredAuthType, DEFAULT_RPC_CALL_TIMEOUT);
	}

	@Deprecated
	public static RpcFileUploadService createWithOldRpcInterface(String namespace,
																 Class<? extends RpcAuthToken> requiredAuthType,
																 long timeout) {
		return new RpcFileUploadService(namespace, requiredAuthType, timeout, true);
	}

	@Deprecated
	public static RpcFileUploadService createWithOldRpcInterface(String namespace,
																 Class<? extends RpcAuthToken> requiredAuthType) {
		return new RpcFileUploadService(namespace, requiredAuthType, DEFAULT_RPC_CALL_TIMEOUT, true);
	}

	@SuppressWarnings("deprecation")
	private RpcFileUploadService(String namespace, Class<? extends RpcAuthToken> requiredAuthType, long timeout,
								 boolean useOldInterface) {
		fileUpload = new RpcFileUploadImpl(timeout);
		if (useOldInterface) {
			fileUploadRpcObject = RpcObject.createRpcObjectWithInterfaceMethods(RpcFiletransferLegacy.class,
				new RpcFiletransferLegacyAdapter(fileUpload), namespace, requiredAuthType);
		} else {
			fileUploadRpcObject = RpcObject.createRpcObjectWithInterfaceMethods(RpcFileUpload.class, fileUpload,
				namespace, requiredAuthType);
		}
	}

	/**
	 * Adds a {@link FileTransferEventListener} to be informed about uploads using this service
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public void addEventListener(FileTransferEventListener listener) {
		fileUpload.addEventListener(listener);
	}

	/**
	 * Removes a {@link FileTransferEventListener} from this service's listeners
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public void removeEventListener(FileTransferEventListener listener) {
		fileUpload.removeEventListener(listener);
	}

	/**
	 * @see de.starface.com.rpc.services.RpcService#register(de.starface.com.rpc.server.RpcObjectRegistry)
	 */
	public void register(RpcObjectRegistry registry) {
		registry.registerRpcObject(fileUploadRpcObject);
	}

	/**
	 * @see de.starface.com.rpc.services.RpcService#unregister(de.starface.com.rpc.server.RpcObjectRegistry)
	 */
	public void unregister(RpcObjectRegistry registry) {
		registry.unregisterRpcObject(fileUploadRpcObject);
	}
}
