package de.starface.com.rpc.services.filetransfer.common;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.DigestOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * This class encapsulates file operations on files for incoming file transfers
 *
 * @author tilmann
 */
public class IncomingFile {

	private static final Log log = LogFactory.getLog(IncomingFile.class);

	private DigestOutputStream outputStream;

	private File file;

	public File getFile() {
		return file;
	}

	public void openWithDigestNameAndDesiredName(String digestName, String desiredName) throws IOException,
		NoSuchAlgorithmException {

		file = createTempFile(desiredName);

		MessageDigest streamDigest = MessageDigest.getInstance(digestName);

		outputStream = new DigestOutputStream(new BufferedOutputStream(new FileOutputStream(file)), streamDigest);
	}

	private File createTempFile(String desiredName) throws IOException {
		String prefix = getPrefix(desiredName);
		String suffix = getSuffix(desiredName);

		return File.createTempFile(prefix, suffix);
	}

	private static String getSuffix(String desiredName) {
		String suffix = null;
		if (desiredName != null && desiredName.contains(".")) {
			suffix = desiredName.substring(desiredName.lastIndexOf("."));
			if (suffix.isEmpty()) {
				return null;
			}
		}
		return suffix;
	}

	private static String getPrefix(String desiredName) {
		String prefix = null;
		if (desiredName != null && desiredName.contains(".")) {
			prefix = desiredName.substring(0, desiredName.lastIndexOf("."));
		}
		if (prefix == null || prefix.isEmpty()) {
			prefix = "filetransfer";
		}
		while (prefix.length() < 3) {
			prefix += "_";
		}
		return prefix;
	}

	public void write(byte[] data, int dataLength) throws IOException {
		outputStream.write(data, 0, dataLength);
		outputStream.flush();
	}

	public byte[] closeAndComputeDigest() throws IOException {
		outputStream.close();
		return outputStream.getMessageDigest().digest();
	}

	public void deleteSilently() {
		if (outputStream != null) {
			try {
				outputStream.close();
			} catch (IOException e) {
				log.warn("Exception while closing digest output stream.", e);
			}
			outputStream = null;
		}

		if (file != null) {
			if (!file.delete()) {
				log.warn("Could not delete temporary output file " + file.getAbsolutePath());
			}
			file = null;
		}
	}

	public long getSize() {
		return file.length();
	}

	public String getName() {
		return file.getAbsolutePath();
	}
}
