package de.starface.com.rpc.services.filetransfer.common;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.com.rpc.services.filetransfer.FileInfo;
import de.starface.com.rpc.services.filetransfer.FileTransfer.State;

import org.apache.commons.lang3.StringUtils;

/**
 * This is an RPC value object that is send between client and server during a file download. It
 * represents the result of the initialization of the upload and contains the file transfer id of
 * the download as well as the state of the download and the information of the transferring file
 *
 * @author tilmann
 */
@RpcValueObject
public class InitializeDownloadResult {
	@RpcValue
	private State fileTransferState;

	@RpcValue
	private FileInfo fileInfo;

	@RpcValue
	private String fileTransferId;

	public InitializeDownloadResult() {
		// provide default constructor
	}

	public InitializeDownloadResult(String fileTransferId, State fileTransferState, FileInfo fileInfo) {
		this.fileTransferId = StringUtils.trimToEmpty(fileTransferId);
		this.fileTransferState = fileTransferState;
		this.fileInfo = fileInfo;
	}

	public State getFileTransferState() {
		return fileTransferState;
	}

	public FileInfo getFileInfo() {
		return fileInfo;
	}

	public String getFileTransferId() {
		return fileTransferId;
	}
}
