package de.starface.com.rpc.services.filetransfer.common;

import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod;

/**
 * Interface providing the RPC signatures for the file download
 *
 * @author tilmann
 */
@RpcValueTranslation
public interface RpcFileDownload {
	/**
	 * Initialize the download of the file with the given id and hash method
	 *
	 * @param fileId
	 *        the id of the file to download
	 * @param hashMethod
	 *        the hash method to use during download
	 * @return the result of the initialization containing the id of the newly initialized file
	 *         transfer and the information about the transferring file
	 */
	public InitializeDownloadResult initializeDownload(String fileId, HashMethod hashMethod);

	/**
	 * Continue the download with the file transfer id
	 *
	 * @param fileTransferId
	 *        the id of the file transfer to continue
	 * @return the result of the continuation holding the transferred block data
	 */
	public ContinueDownloadResult continueDownload(String fileTransferId);

	/**
	 * Complete the download with the given file transfer id
	 *
	 * @param fileTransferId
	 *        the id of the transfer to complete
	 * @return the result of the completion containing the hashes for the file
	 */
	public CompleteDownloadResult completeDownload(String fileTransferId);

	/**
	 * Cancel the file download with the given file transfer id
	 *
	 * @param fileTransferId
	 *        the id of the file transfer to cancel
	 */
	public void cancelDownload(String fileTransferId);
}
