package de.starface.com.rpc.services.filetransfer.common;

import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.com.rpc.services.filetransfer.FileInfo;
import de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod;

/**
 * Interface providing the RPC signatures for the file upload
 *
 * @author tilmann
 */
@RpcValueTranslation
public interface RpcFileUpload {
	/**
	 * Initialize the upload of the file represented by the given file info using the given hash
	 * method
	 *
	 * @param fileInfo
	 *        information about the file to be uploaded
	 * @param hashMethod
	 *        the has method to use during upload
	 * @return the result of the initialization containing the id of the newly initialized file
	 *         transfer
	 */
	public InitializeUploadResult initializeUpload(FileInfo fileInfo, HashMethod hashMethod);

	/**
	 * Continue the upload with the given file transfer id
	 *
	 * @param fileTransferId
	 *        the id of the file transfer
	 * @param fileBlock
	 *        the block to be uploaded to the server
	 * @return the result of the continuation
	 */
	public ContinueUploadResult continueUpload(String fileTransferId, FileBlock fileBlock);

	/**
	 * Complelte the upload with the given file transfer id
	 *
	 * @param fileTransferId
	 *        the id of the file transfer
	 * @param fileEnd
	 *        the data required to end the file upload including the hashes for the file
	 * @return the result of the completion
	 */
	public CompleteUploadResult completeUpload(String fileTransferId, FileEnd fileEnd);

	/**
	 * Cancel the file upload with the given file transfer id
	 *
	 * @param fileTransferId
	 *        the id of the file transfer to cancel
	 */
	public void cancelUpload(String fileTransferId);
}
