/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Sep 29, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.services.filetransfer.common;

import de.starface.com.rpc.RpcException;

/**
 * Legacy Interface providing signatures for RPC during filetransfer.
 *
 * @author may
 */
@Deprecated()
public interface RpcFiletransferLegacy {
	/**
	 * Initialization of a filetransfer. Sent before file is actually transferred for requesting a
	 * new slot-id for the file to transfer from the server.
	 *
	 * @param description
	 *        Description for the filetransfer
	 * @param fileName
	 *        Name of the file to transfer
	 * @param mimeType
	 *        Mime type of the file to transfer
	 * @param fileSize
	 *        Size of the file to transfer (as string) because XML-RPC does not support the
	 *        {@code long} type
	 * @param digestName
	 *        Name of the hash algorithm used during filetransfer
	 * @return The Slot-ID for the new filetransfer or an empty string if filetransfer was rejected
	 * @throws RpcException
	 *         If a problem occurs during the RPC call
	 */
	public String initializeTransfer(String description, String fileName, String mimeType, String fileSize,
									 String digestName) throws RpcException;

	/**
	 * Sending data for an ongoing filetransfer
	 *
	 * @param fileId
	 *        Slot-ID of the file in transfer
	 * @param blockNumber
	 *        Number of the part sent
	 * @param block
	 *        The actual data as byte-array (hex encoded)
	 * @param blockDigest
	 *        Hash value for the data send in this block
	 * @return {@code true} when block was receied successfully, {@code false} otherwise
	 * @throws RpcException
	 *         If a problem occurs during the RPC call
	 */
	public boolean sendBlock(String fileId, int blockNumber, byte[] block, byte[] blockDigest) throws RpcException;

	/**
	 * Terminating filetransfer
	 *
	 * @param fileId
	 *        Slot-ID of the file in transfer
	 * @param fileDigest
	 *        digest of the whole file (ignored in fail-case)
	 * @return {@code true} when the file was received successfully, {@code false} otherwise
	 * @throws RpcException
	 *         If a problem occurs during the RPC call
	 */
	public boolean finishTransfer(String fileId, byte[] fileDigest) throws RpcException;
}
