package de.starface.com.rpc.services.filetransfer.service;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.services.filetransfer.FileInfo;
import de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod;
import de.starface.com.rpc.services.filetransfer.FileTransfer.State;
import de.starface.com.rpc.services.filetransfer.common.FileBlock;
import de.starface.com.rpc.services.filetransfer.common.FileEnd;
import de.starface.com.rpc.services.filetransfer.common.InitializeUploadResult;
import de.starface.com.rpc.services.filetransfer.common.RpcFileUpload;
import de.starface.com.rpc.services.filetransfer.common.RpcFiletransferLegacy;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * An adapter that allows the file upload service to be compatible with the legacy file transfer
 * interface used by a previous version of this API
 *
 * @author tilmann
 */
@SuppressWarnings("deprecation")
public class RpcFiletransferLegacyAdapter implements RpcFiletransferLegacy {
	private static final Log log = LogFactory.getLog(RpcFiletransferLegacyAdapter.class);

	private RpcFileUpload fileUpload;

	public RpcFiletransferLegacyAdapter(RpcFileUpload fileUpload) {
		this.fileUpload = fileUpload;
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFiletransferLegacy#initializeTransfer(String,
	 *      String, String, String, String)
	 */
	public String initializeTransfer(String description, String fileName, String mimeType, String fileSize,
									 String digestName) throws RpcException {
		long size = Long.parseLong(fileSize);

		try {
			HashMethod hashMethod = HashMethod.valueOf(digestName);

			InitializeUploadResult result = fileUpload.initializeUpload(new FileInfo(description, fileName,
				mimeType, size), hashMethod);

			if (State.IN_PROGRESS == result.getFileTransferState()) {
				return result.getFileTransferId();
			} else {
				return "";
			}
		} catch (IllegalArgumentException e) {
			log.info("Could not initialize file upload.", e);
			return "";
		}
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFiletransferLegacy#sendBlock(String,
	 *      int, byte[], byte[])
	 */
	public boolean sendBlock(String fileId, int blockNumber, byte[] block, byte[] blockDigest) throws RpcException {
		if (State.IN_PROGRESS == fileUpload.continueUpload(fileId, new FileBlock(block, blockDigest)).getFileTransferState()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @see de.starface.com.rpc.services.filetransfer.common.RpcFiletransferLegacy#finishTransfer(String,
	 *      byte[])
	 */
	public boolean finishTransfer(String fileId, byte[] fileDigest) {
		if (State.COMPLETED == fileUpload.completeUpload(fileId, new FileEnd(fileDigest)).getFileTransferState()) {
			return true;
		} else {
			return false;
		}
	}
}
