package de.starface.com.rpc.xmlrpc;


/**
 * Builder Pattern interface of the builder used to create incoming XML-RPC requests/responses
 *
 * @author tilmann
 *
 */
public interface XmlRpcObjectBuilder {
	/**
	 * Create a new incoming request object with the given method name and call parameters
	 *
	 * @param method
	 *        the name of the RPC method
	 * @param params
	 *        the parameters for the call
	 */
	void createIncomingRequest(String method, Object[] params);

	/**
	 * Create a new incoming response with the given return value
	 *
	 * @param returnValue
	 *        the return value of the response
	 */
	void createIncomingResponse(Object returnValue);

	/**
	 * Create a new incoming response with the given fault values
	 *
	 * @param faultCode
	 *        the code of the fault
	 * @param faultString
	 *        the message of the fault
	 */
	void createIncomingFaultResponse(int faultCode, String faultString);
}
