/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 21, 2007 by may
 * 
 * $HeadURL:http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmlrpc/client/
 * XmlRpcRequestCaller.java $
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.com.rpc.xmlrpc;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.OutgoingRequestProcessorBase;
import de.starface.com.rpc.common.RpcTransportToken;

import java.lang.reflect.Constructor;

/**
 * This abstract superclass represents the client side caller for a XML-RPC request. The caller is
 * heavily based on the transport channel and therefore requires a TransportToken to identify the
 * RPC-server.
 *
 * @author tilmann
 * @param <TransportToken>
 *        the transport token type of the processor
 */
public abstract class XmlRpcOutgoingRequestProcessor<TransportToken extends RpcTransportToken> extends
	OutgoingRequestProcessorBase<TransportToken> {

	/**
	 * Method tries to resolve a {@link Throwable} from a given fault string.
	 *
	 * @param faultMessage
	 *        The string which might be the fully qualified name of a java {@link Throwable} class
	 *        and its message, seperated by ":"
	 * @return The corresponding {@link Throwable} or {@code null} otherwise
	 */
	private static Throwable resolveThrowableFromXmlRpcFault(String faultMessage, int faultCode) {
		try {
			// may throw null pointer
			int index = faultMessage.indexOf(":");
			// may throw index out of bound
			String name = faultMessage.substring(0, index).trim();
			String message = faultMessage.substring(index + 1).trim();

			Class<?> clazz = null;

			if ("de.vertico.com.xmlrpc.XmlRpcException".equals(name)) {
				// Compatible to old STARFACE
				clazz = RpcException.class;
			} else {
				// may throw a lot of stuff
				clazz = Class.forName(name);
			}

			Throwable t;
			if (RpcException.class.equals(clazz)) {
				// may throw a bit of everything
				Constructor<?> c = clazz.getConstructor(int.class, String.class);
				// may throw class cast exception
				t = (Throwable) c.newInstance(faultCode, message);
			} else {
				// may throw a bit of everything
				Constructor<?> c = clazz.getConstructor(String.class);
				// may throw class cast exception
				t = (Throwable) c.newInstance(message);
			}
			t.setStackTrace(new StackTraceElement[]{});

			return t;
		} catch (Exception e) {
			return null;
		}
	}

	protected static void throwExceptionOnXmlRpcFault(XmlRpcResponse response) throws RpcException {
		if (response.isFault()) {
			// The received packet was a valid fault, so resolve the error
			// determine if this was an exception on the server and forward it
			String message = response.getFaultMessage();
			int code = response.getFaultCode();

			Throwable cause = resolveThrowableFromXmlRpcFault(message, code);
			if (cause != null) {
				// if server tunneled XmlRpcException rethrow it
				if (cause instanceof RpcException) {
					throw (RpcException) cause;
				} else {
					// if server tunneled application exception wrap it 
					if (code == RpcException.Codes.Server.APPLICATION_EXCEPTION) {
						throw new RpcException(code, "The remote procedure threw an exception during execution",
							cause);
					} else if (code == RpcException.Codes.Server.PREPARED_CLIENT_THROWABLE) {
						throw new RpcException(code,
							"The server code prepared a throwable to be thrown on the client", cause);
					}
					// this should never happen because only RpcExceptions or APPLICATION_EXCEPTIONs are tunneled
					else {
						throw new RpcException(code, "Unexpected Exception transmission from RPC server", cause);
					}
				}
			}
			// wrap all other XML-RPC faults
			else {
				throw new RpcException(code, message);
			}
		}
	}
}
