package de.starface.com.rpc.xmlrpc;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.server.RequestExecutorImpl;
import de.starface.com.rpc.server.interceptor.IncomingRequestInterceptor;

import java.util.List;

/**
 * This is a subclass of {@link RequestExecutorImpl} that adds the specific functionality required
 * when XML-RPC is used as method call mechanism.
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        The {@link RpcTransportToken} subclass this executer processes requests for.
 */
public class XmlRpcRequestExecutor<TransportToken extends RpcTransportToken> extends
	RequestExecutorImpl<TransportToken> {
	/**
	 * @see de.starface.com.rpc.server.RequestExecutorImpl#convertExceptionToRpcException(Exception)
	 */
	@Override
	public RpcException convertExceptionToRpcException(Exception e) {
		RpcException rpcException = super.convertExceptionToRpcException(e);
		return prepareRpcExceptionForXmlRpcTransport(rpcException);
	}

	/**
	 * @see de.starface.com.rpc.server.RequestExecutorImpl#prepareRequestInterceptors()
	 */
	@Override
	protected List<IncomingRequestInterceptor<?>> prepareRequestInterceptors() throws RpcException {
		List<IncomingRequestInterceptor<?>> interceptors = super.prepareRequestInterceptors();
		interceptors.add(new XmlRpcVoidMethodInterceptor());
		return interceptors;
	}

	private static RpcException prepareRpcExceptionForXmlRpcTransport(RpcException e) {
		Throwable cause = e.getCause();
		String faultException;
		String faultMessage;

		if (e.getCode() == RpcException.Codes.Server.APPLICATION_EXCEPTION
			|| e.getCode() == RpcException.Codes.Server.PREPARED_CLIENT_THROWABLE) {
			faultException = cause.getClass().getCanonicalName();
			faultMessage = cause.getMessage();
		} else {
			// To be compatible with previous versions use old Exception name
			faultException = "de.vertico.com.xmlrpc.XmlRpcException";
			faultMessage = e.getMessage();
			Throwable subCause = e.getCause();
			if (subCause != null) {
				faultMessage += " Reason: " + subCause.getMessage();
			}
		}
		return new RpcException(e.getCode(), faultException + " : " + faultMessage);
	}
}
