package de.starface.com.rpc.xmlrpc;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;
import de.starface.com.rpc.server.IncomingRequest;
import de.starface.com.rpc.server.interceptor.IncomingRequestInterceptor;

import java.lang.reflect.Method;

/**
 * This {@link IncomingRequestInterceptor} takes care of the correct return type for void methods
 * when invoked through XML-RPC. XML-RPC does not allow void or null returns for procedure calls so
 * this interceptor simulates a {@code boolean} return value of {@code true} for such calls.
 *
 * @author tilmann
 */
public class XmlRpcVoidMethodInterceptor implements IncomingRequestInterceptor<RpcTransportToken> {
	/**
	 * @see de.starface.com.rpc.common.interceptor.RpcRequestInterceptor#interceptRequest(de.starface.com.rpc.common.RpcRequest,
	 *      de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink)
	 */
	public void interceptRequest(IncomingRequest<RpcTransportToken> request,
								 RequestInterceptorChainLink interceptorChain) throws RpcException {
		interceptorChain.processRequest();

		// Emulate Boolean with value of true for void methods
		// TODO: (tilmann) Instead allow configuration of NULL extension http://ontosys.com/xml-rpc/extensions.php
		Method method = request.getRpcMethod();
		if (method != null && void.class.equals(method.getReturnType())) {
			request.setReturnValue(true);
		}
	}
}
