/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Jan 28, 2008 by may
 * 
 * $HeadURL$
 * 
 * $LastChangedDate$ $LastChangedBy$ $LastChangedRevision$
 */
package de.starface.com.rpc.xmlrpc.http;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectSerializer;

/**
 * A {@link XmlRpcObject} to represent a XML-RPC request via HTTP
 *
 * @author may
 *
 */
public class HttpXmlRpcRequest implements XmlRpcObject {
	private final String _methodName;
	private final Object[] _parameters;
	private String[] _serializedParameters;

	public static HttpXmlRpcRequest createOutgoingRequest(String method, Object[] params) throws RpcException {
		HttpXmlRpcRequest request = new HttpXmlRpcRequest(method, params);
		try {
			request.serializeParameters();
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Client.XML_SERIALIZATION_ERROR,
				"Could not serialize parameter values of RPC call to XML-RPC.", e);
		}
		return request;
	}

	public static HttpXmlRpcRequest createIncomingRequest(String method, Object[] params) {
		return new HttpXmlRpcRequest(method, params);
	}

	/**
	 * Note: As for code-safety the method name and the array of parameters cannot be modified in
	 * one packet. To modify a rpc request, a new packet has to be created!
	 *
	 * @param method
	 *        the name of the remote function to call
	 * @param params
	 *        Array of Object as parameters for the function call. See
	 *        {@link XmlRpcObjectSerializer} for a list of supported types
	 * @throws IllegalArgumentException
	 *         If the a given parameter is not recognized
	 */
	private HttpXmlRpcRequest(String method, Object[] params) {
		if (method == null) {
			throw new IllegalArgumentException("method name must not be null");
		}

		_methodName = method;
		_parameters = params;
	}

	private void serializeParameters() {
		if (_parameters != null) {
			_serializedParameters = new String[_parameters.length];
			for (int i = 0; i < _parameters.length; i++) {

				_serializedParameters[i] = XmlRpcObjectSerializer.serialize(_parameters[i]);
			}
		}
	}

	/**
	 * @return The request
	 */
	public String toXml() {
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\"?>");
		sb.append("<methodCall>");
		sb.append("<methodName>");
		sb.append(_methodName);
		sb.append("</methodName>");
		if (_parameters != null) {
			if (_serializedParameters == null) {
				throw new IllegalStateException("This is not an outgoing XML-RPC request.");
			}
			sb.append("<params>");
			for (String s : _serializedParameters) {
				sb.append("<param>");
				sb.append(s);
				sb.append("</param>");
			}
			sb.append("</params>");
		}
		sb.append("</methodCall>");

		return sb.toString();
	}

	/**
	 * @return Returns the methodName.
	 */
	public String getMethodName() {
		return _methodName;
	}

	/**
	 * @return Returns the parameters.
	 */
	public Object[] getParameters() {
		return _parameters;
	}
}
