/*
 * $Id: XmppXmlRpcClient.java 430 2011-11-07 19:27:54Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 13, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmlrpc/smack/XmppXmlRpcClient
 * .java $
 * 
 * $LastChangedDate: 2011-11-07 20:27:54 +0100 (Mon, 07 Nov 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 430 $
 */
/**
 *
 */
package de.starface.com.rpc.xmlrpc.smack;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.OutgoingRequestProcessor;
import de.starface.com.rpc.client.RpcClient;
import de.starface.com.rpc.client.RpcClientBase;
import de.starface.com.rpc.services.RpcAgent;
import de.starface.com.xmpp.smack.XmlRpcPacketProvider;
import de.starface.com.xmpp.smack.XmppConnectionException;
import de.starface.com.xmpp.smack.XmppConnector;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jivesoftware.smack.Connection;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.provider.ProviderManager;

/**
 * Implements a {@link RpcClient} using XMPP as transport
 *
 * @author may, tilmann
 *
 *
 */
public class XmppXmlRpcClient extends RpcClientBase<JID> {
	private static final Log log = LogFactory.getLog(XmppXmlRpcClient.class);

	static {
		ProviderManager.getInstance().addIQProvider("query", "jabber:iq:rpc", new XmlRpcPacketProvider());
	}

	private XmppConnector _connector;

	private Connection _connection;

	private boolean opened = false;

	private String defaultXmlRpcServerXmppUserName;

	private String defaultXmlRpcServerXmppResource;

	private XmppXmlRpcClient(String xmppUserName, String xmppResource, String xmppPassword,
							 String xmppServerHostName, int xmppServerPort, boolean xmppUseSSL,
							 boolean xmppAutoReconnect, boolean xmppSendPresence) {
		createConnection(xmppUserName, xmppResource, xmppPassword, xmppServerHostName, xmppServerPort, xmppUseSSL,
			xmppAutoReconnect, xmppSendPresence);
		log.trace("XmppXmlRpcClient created.");
	}

	private XmppXmlRpcClient(Connection xmppConnection) {
		Validate.notNull(xmppConnection, "xmppConnection=null");
		setConnection(xmppConnection);
		log.trace("XmppXmlRpcClient created.");
	}

	public static XmppXmlRpcClient createWithNewConnection(String xmppUserName, String xmppResource,
														   String xmppPassword, String xmppServerHostName,
														   int xmppServerPort, boolean xmppUseSSL,
														   boolean xmppAutoReconnect, boolean xmppSendPresence) {
		return new XmppXmlRpcClient(xmppUserName, xmppResource, xmppPassword, xmppServerHostName, xmppServerPort,
			xmppUseSSL, xmppAutoReconnect, xmppSendPresence);
	}

	public static XmppXmlRpcClient createWithNewConnectionAndDefaultServer(String xmppUserName,
																		   String xmppResource,
																		   String xmppPassword,
																		   String xmppServerHostName,
																		   int xmppServerPort, boolean xmppUseSSL,
																		   boolean xmppAutoReconnect,
																		   boolean xmppSendPresence,
																		   JID defaultServerToken) {
		Validate.notNull(defaultServerToken, "defaultServerToken=null");
		XmppXmlRpcClient client = new XmppXmlRpcClient(xmppUserName, xmppResource, xmppPassword,
			xmppServerHostName, xmppServerPort, xmppUseSSL, xmppAutoReconnect, xmppSendPresence);
		client.setDefaultServerTransportToken(defaultServerToken);
		return client;
	}

	public static XmppXmlRpcClient createWithExistingConnection(Connection xmppConnection) {
		return new XmppXmlRpcClient(xmppConnection);
	}

	public static XmppXmlRpcClient createWithExistingConnectionAndDefaultServer(XMPPConnection xmppConnection,
																				JID defaultServerToken) {
		Validate.notNull(defaultServerToken, "defaultServerToken=null");
		XmppXmlRpcClient client = new XmppXmlRpcClient(xmppConnection);
		client.setDefaultServerTransportToken(defaultServerToken);
		return client;
	}

	@Override
	protected JID createDefaultServerTransportToken() throws RpcException {
		if (StringUtils.isEmpty(defaultXmlRpcServerXmppUserName)) {
			return null;
		}
		return determineJidFromConnection(_connection, defaultXmlRpcServerXmppUserName,
			defaultXmlRpcServerXmppResource);
	}

	private JID determineJidFromConnection(Connection connection, String serverUserName, String serverResource)
		throws RpcException {
		JID serverJid = JID.createJidForTarget(_connection, serverUserName, serverResource);
		if (serverJid == null) {
			throw new RpcException(RpcException.Codes.Client.XMPP_SERVICE_NAME_MISSING,
				"Unable to determine server JID: Connection has never been established for this XmppXmlRpcClient.");
		}
		return serverJid;
	}

	/**
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.client.RpcClient#setDefaultServerTransportToken(de.starface.com.rpc.common.RpcTransportToken)
	 */
	@Override
	public void setDefaultServerTransportToken(JID defaultServerTransportToken) {
		super.setDefaultServerTransportToken(defaultServerTransportToken);

		if (defaultServerTransportToken == null) {
			defaultXmlRpcServerXmppResource = null;
			defaultXmlRpcServerXmppUserName = null;
		}
	}

	public void setDefaultXmlRpcServerJid(String xmlRpcServerXmppUserName, String xmlRpcServerXmppResource) {
		defaultXmlRpcServerXmppResource = xmlRpcServerXmppResource;
		defaultXmlRpcServerXmppUserName = xmlRpcServerXmppUserName;

		// Set default transport token to null
		// Proxy Factory will call createDefaultServerTransportToken()
		super.setDefaultServerTransportToken(null);
	}

	private void setConnection(Connection xmppConnection) {
		Validate.notNull(xmppConnection, "xmppConnection=null");
		_connection = xmppConnection;
	}

	private synchronized void createConnection(String xmppUserName, String xmppResource, String xmppPassword,
											   String xmppServerHostName, int xmppServerPort, boolean xmppUseSSL,
											   boolean xmppAutoReconnect, boolean xmppSendPresence) {
		_connector = new XmppConnector(xmppUserName, xmppResource, xmppPassword, xmppServerHostName,
			xmppServerPort, xmppUseSSL, xmppAutoReconnect, xmppSendPresence);
		_connection = _connector.getConnection();
	}

	/**
	 * Creates a new proxy instance of the given interface for the given server represented by the
	 * XMPP user name and resource. This method is supposed to throw exceptions if a valid proxy
	 * cannot be created instead of returning null!
	 *
	 * @param <ProxyInterface>
	 *        Type the proxy should implement
	 * @param interfaceClass
	 *        The class of the interface to use for RPC-Calls
	 * @param xmlRpcServerXmppUserName
	 *        The XMPP user name of the XML-RPC server to communicate with
	 * @param xmlRpcServerXmppResource
	 *        The XMPP resource of the XML-RPC server to communicate with
	 * @return The created proxy instance
	 *
	 * @throws IllegalArgumentException
	 *         If one of the arguments is {@code null} or if the given interfaceClass is not an
	 *         interface
	 * @throws RpcException
	 *         If the underlying XMPP connection has not yet been connected (-101), if the required
	 *         transport has not been registered for XML-RPC (code -2) or the RPC proxy cannot be
	 *         created for another reason (code -1) with more details available in this exceptions
	 *         cause.
	 */
	public <ProxyInterface> ProxyInterface createRpcProxy(Class<ProxyInterface> interfaceClass,
														  String xmlRpcServerXmppUserName,
														  String xmlRpcServerXmppResource) throws RpcException {
		JID serverJid = determineJidFromConnection(_connection, xmlRpcServerXmppUserName, xmlRpcServerXmppResource);

		return createRpcProxy(interfaceClass, serverJid);
	}

	/**
	 * Creates a new agent instance of the class provided and initializes it. This method is
	 * supposed to throw exceptions if a valid agent cannot be created instead of returning null!
	 *
	 * @param <ProxyInterface>
	 *        Type of the processor the agent uses
	 * @param <X>
	 *        Type of the agent to create
	 * @param serviceName
	 *        Name of the corresponding service on the server. Used as prefix for remote procedure
	 *        (or ignored when {@code null})
	 * @param agentClass
	 *        The Class of the agent to create and initialize (must not be {@code null})
	 * @param xmlRpcServerXmppUserName
	 *        The XMPP user name of the XML-RPC server to communicate with
	 * @param xmlRpcServerXmppResource
	 *        The XMPP resource of the XML-RPC server to communicate with
	 * @return The created agent
	 * @throws RpcException
	 *         If the underlying XMPP connection has not yet been connected (-101), if the agent can
	 *         not be instantiated (code -6) or if the required transport has not been registered
	 *         for XML-RPC (code -2) or the RPC agent cannot be created for another reason (code -1)
	 *         with more details available in this exceptions cause.
	 * @throws IllegalArgumentException
	 *         If agent class or server token are null
	 *
	 * @see RpcAgent
	 */
	public <ProxyInterface, X extends RpcAgent<ProxyInterface>> X createRpcAgent(String serviceName,
																				 Class<X> agentClass,
																				 String xmlRpcServerXmppUserName,
																				 String xmlRpcServerXmppResource)
		throws RpcException {
		JID serverJid = determineJidFromConnection(_connection, xmlRpcServerXmppUserName, xmlRpcServerXmppResource);

		return createRpcAgent(serviceName, agentClass, serverJid);
	}

	/**
	 * Starts the client using synchronous connects to the XMPP server if the connection has not
	 * already been established.
	 *
	 * @throws XmppConnectionException
	 *         if the connection cannot be established on the first try
	 *
	 */
	public void open() throws XmppConnectionException {
		open(false);
	}

	/**
	 * Starts the client using asynchronous connects to the XMPP server if the connection has not
	 * already been established.
	 */
	public void openAsync() {
		try {
			open(true);
		} catch (XmppConnectionException e) {
			// This cannot happen.
			log.warn("Asynchronous open caught exception.", e);
		}
	}

	/**
	 * Starts the client using either synchronous or asynchronous connects to the XMPP server if the
	 * connection has not already been established.
	 *
	 * @param async
	 *        asynchronous connection attempts are used if set to <code>true</code>
	 */
	private synchronized void open(boolean async) throws XmppConnectionException {
		if (!opened) {
			try {
				if (_connector != null) {
					if (async) {
						_connector.startAsyncConnects();
					} else {
						_connector.startConnects();
					}
				}
			} finally {
				opened = true;
				log.debug("XmppXmlRpcClient opened.");
			}
		} else {
			log.debug("XmppXmlRpcClient is allready open.");
		}
	}

	/**
	 * Stops the client
	 */
	public synchronized void close() {
		if (opened) {
			opened = false;

			if (_connector != null) {
				_connector.disconnect();
			}
			log.debug("XmppXmlRpcClient closed.");
		} else {
			log.debug("XmppXmlRpcClient is not open.");
		}
	}

	/**
	 * @return Returns the connection.
	 */
	public Connection getConnection() {
		return _connection;
	}

	/**
	 * Returns the JID belonging to the underlying xmpp connection. This method may return null if
	 * the underlying connection has not been established yet.
	 *
	 * @return the JID of the xmpp connection
	 */
	public JID getJid() {
		return JID.fetchFromConnection(_connection);
	}

	@Override
	protected OutgoingRequestProcessor<JID> createOutgoingRequestProcessor() {
		return new XmppXmlRpcOutgoingRequestProcessor(_connection);
	}
}
