/*
 * $Id: XmppXmlRpcOutgoingRequestProcessor.java 400 2011-07-21 13:41:36Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 21, 2007 by may
 * 
 * $HeadURL:http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src/de/vertico/com/xmlrpc/smack/
 * XmppXmlRpcRequestCaller.java $
 * 
 * $LastChangedDate: 2011-07-21 15:41:36 +0200 (Thu, 21 Jul 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 400 $
 */
package de.starface.com.rpc.xmlrpc.smack;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.OutgoingRequestProcessor;
import de.starface.com.rpc.xmlrpc.XmlRpcOutgoingRequestProcessor;
import de.starface.com.rpc.xmlrpc.XmlRpcResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jivesoftware.smack.Connection;
import org.jivesoftware.smack.PacketCollector;
import org.jivesoftware.smack.filter.AndFilter;
import org.jivesoftware.smack.filter.PacketFilter;
import org.jivesoftware.smack.filter.PacketIDFilter;
import org.jivesoftware.smack.filter.PacketTypeFilter;
import org.jivesoftware.smack.packet.IQ;

/**
 * An {@link OutgoingRequestProcessor} for the usage of a XMPP transport channel
 *
 * @author may
 *
 * @see OutgoingRequestProcessor
 */
public class XmppXmlRpcOutgoingRequestProcessor extends XmlRpcOutgoingRequestProcessor<JID> {
	private static final Log log = LogFactory.getLog(XmppXmlRpcOutgoingRequestProcessor.class);

	/** the xmpp connection to use */
	private final Connection connection;

	private XmlRpcRequestPacket packet;

	XmppXmlRpcOutgoingRequestProcessor(Connection connection) {
		this.connection = connection;
	}

	/**
	 * @see de.starface.com.rpc.client.OutgoingRequestProcessorBase#processRequest()
	 */
	@Override
	protected void processRequest() throws RpcException {
		createOutgoingPacket();

		PacketFilter filter = new AndFilter(new PacketIDFilter(packet.getPacketID()), new PacketTypeFilter(
			IQ.class));
		PacketCollector collector = null;

		if (log.isTraceEnabled()) {
			log.trace("Sending XML-RPC call: " + packet.toXML());
		}
		try {
			collector = connection.createPacketCollector(filter);
			connection.sendPacket(packet);
		} catch (Exception e) {
			log.debug("Could not send XML-RPC call over XMPP", e);
			throw new RpcException(RpcException.Codes.Client.TRANSPORT_FAILURE,
				"Could not send XML-RPC call over XMPP", e);
		}

		// we know its an IQ as we are listening just for IQs
		IQ result = (IQ) collector.nextResult(getTimeoutForCall());
		if (result != null && log.isTraceEnabled()) {
			log.trace("Received XML-RPC reslut: " + result.toXML());
		}

		// Stop queuing results
		collector.cancel();

		if (result instanceof XmlRpcResponse) {
			XmlRpcResponse response = (XmlRpcResponse) result;
			throwExceptionOnXmlRpcFault(response);
			request.setReturnValue(response.getResult());
		} else if (result == null) {
			throw new RpcException(RpcException.Codes.Client.NO_RESPONSE_FROM_SERVER,
				"No response from XML-RPC server.");
		} else if (result.getType() == IQ.Type.ERROR) {
			// Transport reported error
			throw new RpcException(RpcException.Codes.Client.TRANSPORT_FAILURE,
				"XMPP error while sending XMl-RPC call: " + result.getError());
		} else {
			// the result was complete gibberish
			throw new RpcException(RpcException.Codes.Client.TRANSPORT_FAILURE, "XMPP response type "
				+ result.getClass().getName() + " unexpected.");
		}
	}

	private void createOutgoingPacket() throws RpcException {
		JID serverJid = request.getTargetTransportToken();
		packet = XmlRpcRequestPacket.createOutgoingRequest(request.getRpcMethodName(), request.getParameters());
		packet.setFrom(connection.getUser());
		packet.setTo(serverJid.toString());
	}
}
