/*
 * $Id: XmlRpcInteger.java 373 2011-04-14 15:02:42Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 26, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/vertico-xml-rpc/trunk/src/de/starface/com/rpc/xmlrpc/types/XmlRpcInteger
 * .java $
 * 
 * $LastChangedDate: 2011-04-14 17:02:42 +0200 (Thu, 14 Apr 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 373 $
 */
package de.starface.com.rpc.xmlrpc.types;

/**
 * The integer type in XML-RPC.
 * <ul>
 * <li>Tag: {@code <i4>} or {@code <int>}</li>
 * <li>Type: four-byte signed integer</li>
 * <li>Example: -12</li>
 * </ul>
 *
 * @author may
 *
 */
public class XmlRpcInteger implements XmlRpcType {
	private final Integer _value;

	public XmlRpcInteger(Integer value) {
		_value = value;
	}

	public XmlRpcInteger(int value) {
		_value = Integer.valueOf(value);
	}

	public Integer toBoxedValue() {
		return _value;
	}

	public int toPrimitiveValue() {
		return _value.intValue();
	}

	/**
	 * @see Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_value == null) ? 0 : _value.hashCode());
		return result;
	}

	/**
	 * @see Object#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final XmlRpcInteger other = (XmlRpcInteger) obj;
		if (_value == null) {
			if (other._value != null) {
				return false;
			}
		} else if (!_value.equals(other._value)) {
			return false;
		}
		return true;
	}

	/**
	 * @see de.starface.com.rpc.xmlrpc.types.XmlRpcType#getJavaType()
	 */
	public Class<?> getJavaType() {
		return Integer.class;
	}
}
