/*
 * $Id: XmlRpcString.java 373 2011-04-14 15:02:42Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 26, 2007 by may
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/vertico-xml-rpc/trunk/src/de/starface/com/rpc/xmlrpc/types/XmlRpcString
 * .java $
 * 
 * $LastChangedDate: 2011-04-14 17:02:42 +0200 (Thu, 14 Apr 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 373 $
 */
package de.starface.com.rpc.xmlrpc.types;

/**
 * The string type in XML-RPC.
 * <ul>
 * <li>Tag: {@code <string>}</li>
 * <li>Type: string</li>
 * <li>Example: hello world</li>
 * </ul>
 *
 * @author may
 *
 */
public class XmlRpcString implements XmlRpcType {
	private final String _value;

	public XmlRpcString(String value) {
		_value = value;
	}

	public String toStringValue() {
		return _value;
	}

	/**
	 * @see Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_value == null) ? 0 : _value.hashCode());
		return result;
	}

	/**
	 * @see Object#equals(Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final XmlRpcString other = (XmlRpcString) obj;
		if (_value == null) {
			if (other._value != null) {
				return false;
			}
		} else if (!_value.equals(other._value)) {
			return false;
		}
		return true;
	}

	/**
	 * @see de.starface.com.rpc.xmlrpc.types.XmlRpcType#getJavaType()
	 */
	public Class<?> getJavaType() {
		return String.class;
	}
}
