package de.starface.com.xmpp.smack;

/**
 * This exception is thrown if a problem occurs while opening an XMPP connection
 *
 * @author tilmann
 */
public class XmppConnectionException extends Exception {
	/**
	 * This static class contains the constants for the exception codes.
	 *
	 * @author tilmann
	 */
	public static class Codes {
		/** An error in system IO has occurred */
		public static final int IO_ERROR = 1;

		/** The XMPP host cannot be found */
		public static final int UNKNOWN_HOST = 2;

		/** Authentication at the XMPP host has failed */
		public static final int AUTHENTICATION_FAILED = 3;
	}

	private static final long serialVersionUID = -7682171923657469286L;

	private int _code;

	XmppConnectionException(int code) {
		super();
		_code = code;
	}

	XmppConnectionException(int code, String message) {
		super(message);
		_code = code;
	}

	XmppConnectionException(int code, String message, Throwable cause) {
		super(message, cause);
		_code = code;
	}

	/**
	 * Retrieve the code identifying this problem.
	 *
	 * @return one of the constants defined in {@link Codes}
	 */
	public int getCode() {
		return _code;
	}
}
