/*
 * Copyright (C) by STARFACE Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Nov 3, 2014 by pburlov
 */
package de.starface.security.auth;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;

public class StarfaceAuthReporter {
	private final String scope;
	private final int port;
	private final InetAddress server;

	public StarfaceAuthReporter(String scope, int port, InetAddress server) {
		this.scope = scope;
		this.port = port;
		this.server = server;
	}

	private String createReport(String clientIP, String clientUserName, boolean accessGranted) {
		StringBuilder report = new StringBuilder();
		report.append(clientIP);
		report.append(";");
		if (clientUserName != null && clientUserName.length() > 0) {
			report.append(clientUserName);
		}
		report.append(";");
		report.append(this.scope);
		report.append(";");
		report.append(accessGranted);
		return report.toString();
	}

	/**
	 * Sends the notification to the phone.
	 *
	 * @param address
	 *        The address.
	 * @param port
	 *        The port.
	 * @param notification
	 *        The notification message.
	 */
	public synchronized void sendAuthReport(String clientIP, String clientUserName, boolean accessGranted) {
		String notification = createReport(clientIP, clientUserName, accessGranted);

		DatagramSocket socket = null;
		try {
			byte[] buffer = notification.getBytes();
			socket = new DatagramSocket();
			DatagramPacket packet = new DatagramPacket(buffer, buffer.length, server, port);
			socket.send(packet);
		} catch (SocketException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (socket != null) {
				socket.close();
			}
		}
	}
}
