/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE SOFTWARE. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */

package de.starface.com.rpc.xmlrpc.smack

import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter
import de.starface.com.rpc.server.RpcObjectRegistry

import org.jivesoftware.smack.Connection

import spock.lang.Specification
import spock.lang.Subject

/**
 * Created by paul on 29.03.17.
 */
class XmppXmlRpcServerTest extends Specification {
	@Subject
	XmppXmlRpcServer sut

	def setup() {
		sut = Spy(XmppXmlRpcServer, constructorArgs: [Mock(Connection)]) {
			getRegistry() >> Mock(RpcObjectRegistry)
			getAuthTokenConverter() >> Mock(RpcAuthTokenConverter)
		}
	}
	//	def 'test queue limit'(){
	//		setup:
	//		System.setProperty(XmppXmlRpcServer.XMPP_XML_RPC_SERVER_MAX_QUEUE_LENGTH, 1)
	//		def p1 = XmlRpcRequestPacket.createIncomingRequest("SET", new Object[0])
	//		when:
	//		sut.processPacket(p1)
	//		then:
	//		1 *
	//	}

	def 'test for each source own XmlRpcRequestHandler'() {
		setup:
		def h1 = Mock(XmppXmlRpcServer.XmlRpcRequestHandler)
		def h2 = Mock(XmppXmlRpcServer.XmlRpcRequestHandler)
		def p1 = XmlRpcRequestPacket.createIncomingRequest("SET", new Object[0])
		def p2 = XmlRpcRequestPacket.createIncomingRequest("SET", new Object[0])

		when:
		p1.setFrom('p1')
		sut.processPacket(p1)

		then:
		1 * sut.createXmlRpcRequestHandler('p1') >> h1
		0 * sut.createXmlRpcRequestHandler('p2') >> h2
		1 * h1.handle(p1)
		0 * h2.handle(_)

		when:
		p2.setFrom('p2')
		sut.processPacket(p2)

		then:
		0 * sut.createXmlRpcRequestHandler('p1') >> h1
		1 * sut.createXmlRpcRequestHandler('p2') >> h2
		0 * h1.handle(_)
		1 * h2.handle(p2)
	}
}
