package de.starface.com.rpc.tutorial.basics;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.RpcClient;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.server.RpcServer;
import de.starface.com.rpc.xmlrpc.http.HttpXmlRpcClient;
import de.starface.com.rpc.xmlrpc.http.URL;
import de.starface.com.rpc.xmlrpc.jetty.HttpXmlRpcServer;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class RpcTest {
	private RpcServer<URL> rpcServer;

	@BeforeEach
	public void before() throws RpcException {
		createServer();
	}

	@AfterEach
	public void after() throws RpcException {
		destroyServer();
	}

	private void createServer() throws RpcException {
		rpcServer = HttpXmlRpcServer.createHttpServerWithAutomaticHostName(38080, "/xml-rpc");
		rpcServer.startup();
	}

	private void destroyServer() throws RpcException {
		RpcObjectRegistry.DEFAULT_REGISTRY.clear();

		rpcServer.shutdown();
		rpcServer = null;
	}

	@Test
	public void basicClientAndCall() throws IllegalArgumentException, RpcException {
		RpcObjectRegistry.DEFAULT_REGISTRY.registerRpcObject(new ToUpperCaseImpl(), ToUpperCaseService.class);

		RpcClient<URL> rpcClient = new HttpXmlRpcClient();
		rpcClient.setDefaultServerTransportToken(new URL("http://localhost:38080/xml-rpc"));

		ToUpperCaseService toUpperCaseService = rpcClient.createRpcProxy(ToUpperCaseService.class);

		String upperCaseStr = toUpperCaseService.toUpperCase("lowerCaseStr");

		assertEquals("LOWERCASESTR", upperCaseStr);
	}
}
