package de.starface.com.rpc.unittests.common.valuetranslation;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;

import java.util.HashMap;
import java.util.Map;

@RpcValueObject
public class VTVersionTestBean {
	@RpcValue
	String defaultVersionValue;

	@RpcValue(minVersion = 1)
	String minVersion1Value;

	@RpcValue(maxVersion = 1)
	String maxVersion1Value;

	@RpcValue(minVersion = 1, maxVersion = 1)
	String onlyVersion1Value;

	@RpcValue(minVersion = 2, maxVersion = 1)
	String noVersionValue;

	public static VTVersionTestBean createVTVersionTestBean() {
		VTVersionTestBean bean = new VTVersionTestBean();
		bean.initValues();
		return bean;
	}

	private void initValues() {
		defaultVersionValue = "defaultVersionValue";
		minVersion1Value = "minVersion1Value";
		maxVersion1Value = "maxVersion1Value";
		onlyVersion1Value = "onlyVersion1Value";
		noVersionValue = "noVersionValue";
	}

	Map<String, Object> convertToMap() {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("defaultVersionValue", defaultVersionValue);
		map.put("minVersion1Value", minVersion1Value);
		map.put("maxVersion1Value", maxVersion1Value);
		map.put("onlyVersion1Value", onlyVersion1Value);
		map.put("noVersionValue", noVersionValue);
		return map;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((defaultVersionValue == null) ? 0 : defaultVersionValue.hashCode());
		result = prime * result + ((maxVersion1Value == null) ? 0 : maxVersion1Value.hashCode());
		result = prime * result + ((minVersion1Value == null) ? 0 : minVersion1Value.hashCode());
		result = prime * result + ((noVersionValue == null) ? 0 : noVersionValue.hashCode());
		result = prime * result + ((onlyVersion1Value == null) ? 0 : onlyVersion1Value.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		VTVersionTestBean other = (VTVersionTestBean) obj;
		if (defaultVersionValue == null) {
			if (other.defaultVersionValue != null) {
				return false;
			}
		} else if (!defaultVersionValue.equals(other.defaultVersionValue)) {
			return false;
		}
		if (maxVersion1Value == null) {
			if (other.maxVersion1Value != null) {
				return false;
			}
		} else if (!maxVersion1Value.equals(other.maxVersion1Value)) {
			return false;
		}
		if (minVersion1Value == null) {
			if (other.minVersion1Value != null) {
				return false;
			}
		} else if (!minVersion1Value.equals(other.minVersion1Value)) {
			return false;
		}
		if (noVersionValue == null) {
			if (other.noVersionValue != null) {
				return false;
			}
		} else if (!noVersionValue.equals(other.noVersionValue)) {
			return false;
		}
		if (onlyVersion1Value == null) {
			if (other.onlyVersion1Value != null) {
				return false;
			}
		} else if (!onlyVersion1Value.equals(other.onlyVersion1Value)) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "VTVersionTestBean [defaultVersionValue=" + defaultVersionValue + ", minVersion1Value="
			+ minVersion1Value + ", maxVersion1Value=" + maxVersion1Value + ", onlyVersion1Value="
			+ onlyVersion1Value + ", noVersionValue=" + noVersionValue + "]";
	}
}
