/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Mar 26, 2008 by may
 * 
 * $HeadURL:
 * http://192.168.2.2/repos/verticomisc/xmpp-xml-rpc/src-test/de/vertico/com/xmlrpc/SerializationTest
 * .java $
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.com.rpc.unittests.xmlrpc;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectParser;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectSerializer;
import de.starface.com.rpc.xmlrpc.http.HttpXmlRpcObjectParser;
import de.starface.com.rpc.xmlrpc.http.HttpXmlRpcRequest;
import de.starface.com.rpc.xmlrpc.http.HttpXmlRpcResponse;
import de.starface.com.rpc.xmlrpc.http.XmlRpcObject;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.xmlpull.mxp1.MXParser;
import org.xmlpull.v1.XmlPullParser;

import java.io.ByteArrayInputStream;
import java.io.StringReader;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import static java.time.Duration.ofMillis;
import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTimeout;
import static org.junit.jupiter.api.Assertions.fail;


/**
 * TestCase for the serialization and deserialization implementation.
 *
 * @author may
 *
 */
public class XmlRpcSerializationTest {
	private String specInteger1 = "<i4>-12</i4>";
	private String specInteger2 = "<int>-12</int>";
	private String specBoolean = "<boolean>1</boolean>";
	private String specString = "<string>hello world</string>";
	private String specDouble = "<double>-12.214</double>";
	private String specDate = "<dateTime.iso8601>19980717T14:08:55</dateTime.iso8601>";
	private String specBase64 = "<base64>eW91IGNhbid0IHJlYWQgdGhpcyE=</base64>";
	private String specStruct = "<struct><member><name>lowerBound</name><value><i4>18</i4></value> </member><member><name>upperBound</name><value><i4>139</i4></value></member></struct>";
	private String specArray = "<array><data><value><i4>12</i4></value><value><string>Egypt</string></value><value><boolean>0</boolean></value><value><i4>-31</i4></value></data></array>";

	private String specRequest = "<methodCall><methodName>examples.getStateName</methodName><params><param><value><i4>41</i4></value></param></params></methodCall>";
	private String specResponse = "<methodResponse><params><param><value><string>South Dakota</string></value></param></params></methodResponse>";
	private String specFault = "<methodResponse><fault><value><struct><member><name>faultCode</name><value><int>4</int></value></member><member><name>faultString</name><value><string>Too many parameters.</string></value></member></struct></value></fault></methodResponse>";

	/**
	 * Tests serialization of integers
	 */
	@Test
	public void testInteger1() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue(specInteger2);
			String unexpected = toValue(specInteger1);

			int i1 = -12;
			String actual1 = XmlRpcObjectSerializer.serialize(i1);
			assertEquals(expected, actual1);
			assertFalse(unexpected.equals(actual1));

			Integer i2 = Integer.valueOf(-12);
			String actual2 = XmlRpcObjectSerializer.serialize(i2);
			assertEquals(expected, actual2);
			assertFalse(unexpected.equals(actual2));
		});
	}

	/**
	 * Tests deserialization of integers
	 */
	@Test
	public void testInteger2() {
		assertTimeout(ofMillis(1000), () -> {
			int expected1 = -12;
			int unexpected1 = 42;
			Integer expected2 = Integer.valueOf(-12);
			Integer unexpected2 = Integer.valueOf(42);

			Object actual1 = "";
			Object actual2 = "";

			try {
				actual1 = this.parse(toValue(specInteger1));
				actual2 = this.parse(toValue(specInteger2));
			} catch (Exception e) {
				fail(e.getMessage());
			}
			assertEquals(expected1, actual1);
			assertEquals(expected2, actual2);

			assertFalse(Integer.valueOf(unexpected1).equals(actual1));
			assertFalse(unexpected2.equals(actual2));
		});
	}

	/**
	 * Tests serialization of booleans
	 */
	@Test
	public void testBoolean1() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue(specBoolean);
			String unexpected1 = toValue("<boolean>true</boolean>");
			String unexpected2 = toValue("<boolean>0</boolean>");

			boolean b = true;
			String actual = XmlRpcObjectSerializer.serialize(b);
			assertEquals(expected, actual);
			assertFalse(unexpected1.equals(actual));
			assertFalse(unexpected2.equals(actual));
		});
	}

	/**
	 * Tests deserialization of booleans
	 */
	@Test
	public void testBoolean2() {
		assertTimeout(ofMillis(1000), () -> {
			boolean expected = true;
			boolean unexpected = false;

			Object actual1 = "";
			Object actual2 = "";

			try {
				actual1 = this.parse(toValue(specBoolean));
				actual2 = this.parse(toValue("<boolean>true</boolean>"));
				// just to know this does not throw errors
				this.parse(toValue("<boolean>false</boolean>"));
			} catch (Exception e) {
				fail(e.getMessage());
			}
			assertEquals(expected, actual1);
			assertEquals(expected, actual2);

			assertFalse(Boolean.valueOf(unexpected).equals(actual1));
			assertFalse(Boolean.valueOf(unexpected).equals(actual2));
		});
	}

	/**
	 * Tests serialization of strings
	 */
	@Test
	public void testString1() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue(specString);
			String unexpected = toValue("<string>evil</string>");

			String s = "hello world";
			String actual = XmlRpcObjectSerializer.serialize(s);
			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests deserialization of strings
	 */
	@Test
	public void testString2() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = "hello world";
			String unexpected = "something different";

			Object actual = "";

			try {
				actual = this.parse(toValue(specString));
			} catch (Exception e) {
				fail(e.getMessage());
			}
			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests serialization of strings
	 */
	@Test
	public void testString3() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue("<string>&quot;54: Tilmann Kuhn &quot; &lt;&gt;</string>");
			String unexpected = toValue("<string>evil</string>");

			String s = "\"54: Tilmann Kuhn \" <>";
			String actual = XmlRpcObjectSerializer.serialize(s);
			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests serialization of doubles
	 */
	@Test
	public void testDouble1() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue(specDouble);
			String unexpected = toValue("<double>23.32</double>");

			double d = -12.214d;
			String actual = XmlRpcObjectSerializer.serialize(d);
			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests deserialization of doubles
	 */
	@Test
	public void testDouble2() {
		assertTimeout(ofMillis(1000), () -> {
			double expected = -12.214d;
			double unexpected = 23.32d;

			Object actual = "";

			try {
				actual = this.parse(toValue(specDouble));
			} catch (Exception e) {
				fail(e.getMessage());
			}
			assertEquals(expected, actual);
			assertFalse(new Double(unexpected).equals(actual));
		});
	}

	/**
	 * Tests serialization of dates
	 */
	@Disabled("Will always fail due to incomplete iso8601 implementation")
	@Test
	public void testDate1() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue(specDate);
			String unexpected = toValue("<dateTime.iso8601>19891109T18:57:30</dateTime.iso8601>");

			Calendar c = Calendar.getInstance();
			c.set(1998, 6, 17, 14, 8, 55);
			Date d = c.getTime();
			String actual = XmlRpcObjectSerializer.serialize(d);

			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests deserialization of dates
	 */
	@Test
	public void testDate2() {
		assertTimeout(ofMillis(1000), () -> {
			Calendar c = Calendar.getInstance();
			c.set(1998, 6, 17, 14, 8, 55);
			c.set(Calendar.MILLISECOND, 0);
			Date expected = c.getTime();
			c.set(1989, 10, 9, 18, 57, 30);
			Date unexpected = c.getTime();

			Object actual = "";
			Object actual1 = "";

			try {
				actual = this.parse(toValue(specDate));
				actual1 = this.parse(toValue("<dateTime.iso8601>1998-07-17T14:08:55</dateTime.iso8601>"));
			} catch (Exception e) {
				fail(e.getMessage());
			}
			assertEquals(expected, actual);
			assertEquals(expected, actual1);

			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests serialization of byte arrays
	 */
	@Test
	public void testBase64_1() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue(specBase64);
			String unexpected = toValue(
				"<base64>SMOkdHRlbiBIw7x0ZSBlaW4gw58gaW0gTmFtZW4sIHfDpHJlbiBzaWUgbcO2Z2xpY2hlcndlaXNlIGtlaW5lIEjDvHRlIG1laHIsDQpzb25kZXJuIEjDvMOfZS4NCg==</base64>");

			byte[] b = "you can't read this!".getBytes();
			String actual = XmlRpcObjectSerializer.serialize(b);
			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests deserialization of byte arrays
	 */
	@Test
	public void testBase64_2() {
		assertTimeout(ofMillis(1000), () -> {
			byte[] expected = "you can't read this!".getBytes();
			byte[] unexpected = "Hello world".getBytes();

			Object actual = "";

			try {
				actual = this.parse(toValue(specBase64));
			} catch (Exception e) {
				fail(e.getMessage());
			}
			assertEquals(new String(expected), new String((byte[]) actual));

			assertFalse(new String(unexpected).equals(new String((byte[]) actual)));
		});
	}

	/**
	 * Tests serialization of byte maps
	 */
	@Disabled
	@Test
	public void testStruct1() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue(
				"<struct><member><name>lowerBound</name><value><int>18</int></value></member><member><name>upperBound</name><value><int>139</int></value></member></struct>");
			String unexpected = toValue(
				"<struct><member><name>one</name><value><string>Hello World</string></value></member><member><name>two</name><value><i4>42</i4></value></member></struct>");

			Map<String, Object> m = new HashMap<String, Object>();
			m.put("lowerBound", 18);
			m.put("upperBound", 139);
			String actual = XmlRpcObjectSerializer.serialize(m);

			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests deserialization of byte maps
	 */
	@Test
	public void testStruct2() {
		
		assertTimeout(ofMillis(1000), () -> {
			Map<String, Object> expected = new HashMap<String, Object>();
			expected.put("lowerBound", 18);
			expected.put("upperBound", 139);
			Map<String, Object> unexpected = new HashMap<String, Object>();
			unexpected.put("one", "hallo welt");
			unexpected.put("two", 42);

			Object actual = "";

			try {
				actual = this.parse(toValue(specStruct));
			} catch (Exception e) {
				fail(e.getMessage());
			}

			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	} 

	/**
	 * Tests serialization of lists
	 */
	@Test
	public void testArray1() {
		assertTimeout(ofMillis(1000), () -> {
			String expected = toValue(
				"<array><data><value><int>12</int></value><value><string>Egypt</string></value><value><boolean>0</boolean></value><value><int>-31</int></value></data></array>");
			String unexpected = toValue(
				"<array><data><value><boolean>true</boolean></value><string>Hello World</string></value><value><i4>666</i4></value></data></array>");

			List<Object> l = new LinkedList<Object>();
			l.add(12);
			l.add("Egypt");
			l.add(false);
			l.add(-31);
			String actual = XmlRpcObjectSerializer.serialize(l);

			assertEquals(expected, actual);
			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests serialization of lists
	 */
	@Test
	public void testArray2() {
		assertTimeout(ofMillis(1000), () -> {
			List<Object> expected = new LinkedList<Object>();
			expected.add(12);
			expected.add("Egypt");
			expected.add(false);
			expected.add(-31);
			List<Object> unexpected = new LinkedList<Object>();
			unexpected.add(true);
			unexpected.add("Hello World");
			unexpected.add(666);

			Object actual = "";

			try {
				actual = this.parse(toValue(specArray));
			} catch (Exception e) {
				fail(e.getMessage());
			}
			assertEquals(expected, actual);

			assertFalse(unexpected.equals(actual));
		});
	}

	/**
	 * Tests serialization of a request
	 *
	 * @throws RpcException
	 */
	@Test
	public void testRequest1() throws RpcException {
		
		assertTimeout(ofMillis(1000), () -> {
			String expected = "<?xml version=\"1.0\"?><methodCall><methodName>examples.getStateName</methodName><params><param><value><int>41</int></value></param></params></methodCall>";
			String unexpected = "<?xml version=\"1.0\"?><methodCall><methodName>examples.getDemo</methodName><params><param><value><string>Hello World</string></value></param></params></methodCall>";

			String actual1 = HttpXmlRpcRequest.createOutgoingRequest("examples.getStateName",
				new Object[] { 41 }).toXml();
			String actual2 = HttpXmlRpcRequest.createOutgoingRequest("examples.forceFail", new Object[] { 42 }).toXml();

			assertEquals(expected, actual1);

			assertFalse(unexpected.equals(actual1));
			assertFalse(expected.equals(actual2));
		});
	}

	/**
	 * Tests deserialization of a request
	 *
	 * @throws RpcException
	 */
	@Test
	public void testRequest2() throws RpcException {
		
		assertTimeout(ofMillis(1000), () -> {
			HttpXmlRpcRequest expected = HttpXmlRpcRequest.createIncomingRequest("examples.getStateName", new Object[] { 41 });
			HttpXmlRpcRequest unexpected = HttpXmlRpcRequest.createIncomingRequest("examples.forceFail", new Object[] { 42 });

			HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();
			XmlRpcObject o = parser.parseStreamAsServer(new ByteArrayInputStream(this.specRequest.getBytes()));

			assertEquals(HttpXmlRpcRequest.class, o.getClass());

			HttpXmlRpcRequest actual = (HttpXmlRpcRequest) o;
			assertEquals(expected.getMethodName(), actual.getMethodName());
			assertArrayEquals(expected.getParameters(), actual.getParameters());

			assertFalse(unexpected.getMethodName().equals(actual.getMethodName()));
			assertFalse(unexpected.getParameters().equals(actual.getParameters()));
		});
	}

	/**
	 * Tests serialization of a response
	 *
	 * @throws RpcException
	 */
	@Test
	public void testResponse1() throws RpcException {
		assertTimeout(ofMillis(1000), () -> {
			String expected = "<?xml version=\"1.0\"?>" + this.specResponse;
			String unexpected = "<?xml version=\"1.0\"?><methodResponse><params><param><value><string>Hello World</string></value></param></params></methodResponse>";

			String actual1 = HttpXmlRpcResponse.createOutgoingResponse("South Dakota").toXml();
			String actual2 = HttpXmlRpcResponse.createOutgoingResponse("Hello World").toXml();

			assertEquals(expected, actual1);

			assertFalse(unexpected.equals(actual1));
			assertFalse(expected.equals(actual2));
		});
	}

	/**
	 * Tests deserialization of a response
	 *
	 * @throws RpcException
	 */
	@Test
	public void testResponse2() throws RpcException {
		assertTimeout(ofMillis(1000), () -> {
			HttpXmlRpcResponse expected = HttpXmlRpcResponse.createIncomingResponse("South Dakota");
			HttpXmlRpcResponse unexpected = HttpXmlRpcResponse.createIncomingResponse("Hello World");
			HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();
			XmlRpcObject o = parser.parseStreamAsServer(new ByteArrayInputStream(this.specResponse.getBytes()));

			assertEquals(HttpXmlRpcResponse.class, o.getClass());

			HttpXmlRpcResponse actual = (HttpXmlRpcResponse) o;
			assertEquals(expected.getResult(), actual.getResult());

			assertFalse(unexpected.getResult().equals(actual.getResult()));
		});
	}

	/**
	 * Tests serialization of a fault
	 */
	@Test
	public void testFault1() {

		assertTimeout(ofMillis(1000), () -> {
			String expected = "<?xml version=\"1.0\"?>" + this.specFault;
			String unexpected = "<?xml version=\"1.0\"?><methodResponse><fault><value><struct><member><name>faultCode</name><value><int>666</int></value></member><member><name>faultString</name><value><string>Fault from hell.</string></value></member></struct></value></fault></methodResponse>";

			String actual1 = HttpXmlRpcResponse.createOutgoingFaultResponse(4, "Too many parameters.").toXml();
			String actual2 = HttpXmlRpcResponse.createOutgoingFaultResponse(666, "Fault from hell.").toXml();

			assertEquals(expected, actual1);

			assertFalse(unexpected.equals(actual1));
			assertFalse(expected.equals(actual2));
		});
	}

	/**
	 * Tests deserialization of a fault
	 *
	 * @throws RpcException
	 */
	@Test
	public void testFault2() throws RpcException {
		
		assertTimeout(ofMillis(1000), () -> {
			HttpXmlRpcResponse expected = HttpXmlRpcResponse.createIncomingFaultResponse(4, "Too many parameters.");
			HttpXmlRpcResponse unexpected = HttpXmlRpcResponse.createIncomingFaultResponse(666, "Fault from hell.");

			HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();
			XmlRpcObject o = parser.parseStreamAsServer(new ByteArrayInputStream(this.specFault.getBytes()));
			assertEquals(HttpXmlRpcResponse.class, o.getClass());

			HttpXmlRpcResponse actual = (HttpXmlRpcResponse) o;
			assertEquals(expected.getFaultCode(), actual.getFaultCode());
			assertEquals(expected.getFaultMessage(), actual.getFaultMessage());

			assertFalse(unexpected.getFaultCode() == actual.getFaultCode());
			assertFalse(unexpected.getFaultMessage().equals(actual.getFaultMessage()));
	    });
	}

	@Test
	public void testRequestWithGremlins() throws RpcException {
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\"?>");
		sb.append("\t\n<methodCall>\t\n");
		sb.append("\t\n<methodName>ucp.v20.server.connection.setProvidedServices</methodName>\t\n");
		sb.append("\t\n<params>\t\n");
		sb.append("\t\n<param>\t\n");
		sb.append("\t\n<value>\t\n<array>\t\n");
		sb.append("\t\n<data>\t\n");
		sb.append("\t\n<value>\t\n<string>ucp.v20.client.communication.call</string>\t\n</value>\t\n");
		sb.append("\t\n<value>\t\n<string>ucp.v20.client.connection</string>\t\n</value>\t\n");
		sb.append("\t\n</data>\t\n");
		sb.append("\t\n</array>\t\n</value>\t\n");
		sb.append("\t\n</param>\t\n");
		sb.append("\t\n</params>\t\n");
		sb.append("\t\n</methodCall>\t\n");

		HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();
		HttpXmlRpcRequest request = (HttpXmlRpcRequest) parser.parseStreamAsServer(new ByteArrayInputStream(
			sb.toString().getBytes()));

		assertEquals("ucp.v20.server.connection.setProvidedServices", request.getMethodName());
		assertEquals(1, request.getParameters().length);
		assertEquals("[ucp.v20.client.communication.call, ucp.v20.client.connection]",
			request.getParameters()[0].toString());
	}

	@Test
	public void testRequestWithMissingTypes() throws RpcException {
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\"?>");
		sb.append("\t\n<methodCall>\t\n");
		sb.append("\t\n<methodName>ucp.v20.server.connection.setProvidedServices</methodName>\t\n");
		sb.append("\t\n<params>\t\n");
		sb.append("\t\n<param>\t\n");
		sb.append("\t\n<value>\t\n<array>\t\n");
		sb.append("\t\n<data>\t\n");
		sb.append("\t\n<value>ucp.v20.client.communication.call</value>\t\n");
		sb.append("\t\n<value>ucp.v20.client.connection</value>\t\n");
		sb.append("\t\n</data>\t\n");
		sb.append("\t\n</array>\t\n</value>\t\n");
		sb.append("\t\n</param>\t\n");
		sb.append("\t\n</params>\t\n");
		sb.append("\t\n</methodCall>\t\n");

		HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();
		HttpXmlRpcRequest request = (HttpXmlRpcRequest) parser.parseStreamAsServer(new ByteArrayInputStream(
			sb.toString().getBytes()));

		assertEquals("ucp.v20.server.connection.setProvidedServices", request.getMethodName());
		assertEquals(1, request.getParameters().length);
		assertEquals("[ucp.v20.client.communication.call, ucp.v20.client.connection]",
			request.getParameters()[0].toString());
	}

	@Test
	public void testRequestWithMissingTypes2() throws RpcException {
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		sb.append("<methodCall>\n");
		sb.append("<methodName>ucp.v20.server.communication.call.placeCall</methodName>\n");
		sb.append("<params>\n");
		sb.append("<param>\n");
		sb.append("<value>00492642308288</value>\n");
		sb.append("</param>\n");
		sb.append("<param>\n");
		sb.append("<value></value>\n");
		sb.append("</param>\n");
		sb.append("<param>\n");
		sb.append("<value> </value>\n");
		sb.append("</param>\n");
		sb.append("</params>");
		sb.append("</methodCall>");

		HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();
		HttpXmlRpcRequest request = (HttpXmlRpcRequest) parser.parseStreamAsServer(new ByteArrayInputStream(
			sb.toString().getBytes()));

		assertEquals("ucp.v20.server.communication.call.placeCall", request.getMethodName());
		assertEquals(3, request.getParameters().length);
		assertEquals("00492642308288", request.getParameters()[0].toString());
		assertEquals("", request.getParameters()[1].toString());
		assertEquals(" ", request.getParameters()[2].toString());
	}

	@Test
	public void testRequestWithMissingTypes3() throws RpcException {
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		sb.append("<methodCall>\n");
		sb.append("<methodName>ucp.v20.server.communication.call.placeCall</methodName>\n");
		sb.append("<params>\n");
		sb.append("<param>\n");
		sb.append("<value>02642308288</value>\n");
		sb.append("</param>\n");
		sb.append("<param>\n");
		sb.append("<value></value>\n");
		sb.append("</param>\n");
		sb.append("<param>\n");
		sb.append("<value> </value>\n");
		sb.append("</param>\n");
		sb.append("</params>\n");
		sb.append("</methodCall>");

		HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();
		HttpXmlRpcRequest request = (HttpXmlRpcRequest) parser.parseStreamAsServer(new ByteArrayInputStream(
			sb.toString().getBytes()));

		assertEquals("ucp.v20.server.communication.call.placeCall", request.getMethodName());
		assertEquals(3, request.getParameters().length);
		assertEquals("02642308288", request.getParameters()[0].toString());
		assertEquals("", request.getParameters()[1].toString());
		assertEquals(" ", request.getParameters()[2].toString());
	}

	private String toValue(String value) {
		StringBuilder sb = new StringBuilder("<value>");
		sb.append(value);
		sb.append("</value>");

		return sb.toString();
	}

	private Object parse(String value) throws Exception {
		XmlPullParser xpp = new MXParser();
		StringReader reader = new StringReader(value);
		xpp.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);
		xpp.setInput(reader);

		xpp.next();

		if (xpp.getName().equals("value")) {
			return XmlRpcObjectParser.parseValue(xpp);
		} else {
			throw new IllegalArgumentException("Given stanza not a value");
		}
	}

	@Test
	public void testParseValue() {
		final String stringValue = "<value>string</value>";
		final String intValue = "<value><i4>5</i4></value>";

		Object actual = "";
		try {
			actual = this.parse(stringValue);
		} catch (Exception e) {
			fail(e.getMessage());
		}
		assertEquals("string", actual);

		try {
			actual = this.parse(intValue);
		} catch (Exception e) {
			fail(e.getMessage());
		}
		assertEquals(5, actual);
	}
}
