/*
 * $Id$
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.examples.uci30.swing.view;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.adapter.ComboBoxAdapter;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.beans.PropertyConnector;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.model.Config;
import de.starface.examples.uci30.swing.view.binding.MultiListSelectionAdapter;
import de.starface.examples.uci30.swing.viewmodel.CallViewModel;
import de.starface.integration.uci.java.v30.types.Mailbox;
import de.starface.integration.uci.java.v30.values.CallProperties;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.LinkedList;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

/**
 * This panel provides a GUI for the UCI communication call functionality
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class CallPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JButton transferToVoicemailBoxButton;
	private JButton grabCallFromVoicemailBoxButton;
	private JButton parkAndOrbitCallButton;
	private JButton monitorCallButton;
	private JButton placeConsultationCallButteon;
	private JButton transferConsultationCallButton;
	private JButton transferToConferenceConsultationCallCallButton;

	private JButton switchPhoneButton;
	private JButton transferCallButton;
	private JButton forwardCallButton;
	private JButton conferenceCallButton;
	private JButton resumeCallButton;
	private JButton acceptCallButton;
	private JButton holdCallButton;
	private JButton hangupCallButton;
	private JButton placeCallWithPhoneButton;
	private JButton placeCallWithNumberButton;
	private JButton getCallIdsButton;
	private JButton getCallStateButton;
	private JButton dialVoicemailBoxButton;
	private JButton playDtmfButton;
	private JButton imageForHashButton;
	private JButton imageFromDoorlineButton;
	private JButton openDoorlineButton;
	private JButton getMwiStateButton;

	private String selectSingleCall = "Select a single call";
	private String selectTwoCalls = "Select two calls";

	private JScrollPane callTableScrollPane;
	private JScrollPane mwiListScrollPane;
	private JScrollPane callDetailsTableScrollPane;

	private JList mwiList;
	private JTable callTable;
	private JTable callDetailsTable;

	private JLabel newCallIdLabel;
	private JLabel destinationNumberLabel;
	private JLabel originatingNumberLabel;
	private JLabel dtmfCodeLabel;
	private JLabel avatarLabel;
	private JLabel doorlineLabel;

	private JTextField newCallIdText;
	private JTextField destinationNumberText;
	private JTextField originatingNumberText;
	private JTextField playDtmfText;

	private JComboBox parkAndOrbitIdCombo;
	private JComboBox mailboxCombo;

	@SuppressWarnings("unused")
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public CallPanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(850, 500));
		setPreferredSize(new Dimension(850, 500));
		setLayout(new GroupLayout());
		add(getDestinationNumberLabel(), new Constraints(new Leading(9, 100, 6), new Leading(10, 6, 6)));
		add(getNewCallIdLabel(), new Constraints(new Leading(9, 100, 6), new Leading(40, 6, 6)));
		add(getOriginatingNumberLabel(), new Constraints(new Leading(9, 100, 6), new Leading(70, 6, 6)));
		add(getDtmfCodeLabel(), new Constraints(new Leading(9, 100, 6), new Leading(100, 6, 6)));
		add(getMwiListScrollPane(), new Constraints(new Leading(9, 230, 6, 6), new Leading(130, 90, 6, 6)));

		add(getDestinationNumberText(), new Constraints(new Leading(130, 110, 6, 6), new Leading(10, 6, 6)));
		add(getNewCallIdText(), new Constraints(new Leading(130, 110, 6, 6), new Leading(40, 6, 6)));
		add(getOriginatingNumberText(), new Constraints(new Leading(130, 110, 6, 6), new Leading(70, 6, 6)));
		add(getPlayDtmfText(), new Constraints(new Leading(130, 110, 6, 6), new Leading(100, 6, 6)));

		add(getPlaceCallWithPhoneButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(10, 6, 6)));
		add(getPlaceCallWithNumberButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(40, 6, 6)));
		add(getForwardCallButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(70, 6, 6)));
		add(getPlayDtmfButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(100, 6, 6)));
		add(getImageForHashButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(130, 6, 6)));
		add(getImageFromDoorlineButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(160, 6, 6)));
		add(getOpenDoorlineButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(190, 6, 6)));

		add(getHoldCallButton(), new Constraints(new Leading(410, 100, 6, 6), new Leading(10, 6, 6)));
		add(getResumeCallButton(), new Constraints(new Leading(410, 100, 6, 6), new Leading(40, 6, 6)));
		add(getAcceptCallButton(), new Constraints(new Leading(410, 100, 6, 6), new Leading(70, 6, 6)));
		add(getHangupCallButton(), new Constraints(new Leading(410, 100, 6, 6), new Leading(100, 6, 6)));
		add(getTransferCallButton(), new Constraints(new Leading(410, 100, 6, 6), new Leading(130, 6, 6)));
		add(getConferenceCallButton(), new Constraints(new Leading(410, 100, 6, 6), new Leading(170, 6, 6)));

		add(getPlaceConsultationCallButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(230, 6, 6)));
		add(getTransferConsultationCallButton(), new Constraints(new Leading(250, 150, 6, 6), new Leading(260, 6,
			6)));

		add(getTransferConsultationCallToConferenceCallButton(), new Constraints(new Leading(250, 150, 6, 6),
			new Leading(290, 6, 6)));

		add(getMonitorCallButton(), new Constraints(new Leading(410, 100, 6, 6), new Leading(200, 6, 6)));
		add(getSwitchPhoneButton(), new Constraints(new Leading(410, 100, 6, 6), new Leading(230, 6, 6)));

		add(getGetCallStateButton(), new Constraints(new Leading(520, 100, 6, 6), new Leading(10, 6, 6)));
		add(getGetCallIdsButton(), new Constraints(new Leading(520, 100, 6, 6), new Leading(40, 6, 6)));
		add(getGetMwiStateButton(), new Constraints(new Leading(520, 100, 6, 6), new Leading(70, 6, 6)));
		add(getAvatarLabel(), new Constraints(new Leading(520, 58, 6, 6), new Leading(100, 58, 6, 6)));

		add(getParkAndOrbitCallButton(), new Constraints(new Leading(630, 100, 6, 6), new Leading(10, 6, 6)));
		add(getTransferToVoicemailBoxButton(), new Constraints(new Leading(630, 100, 6, 6), new Leading(40, 6, 6)));
		add(getDialVoicemailBoxButton(), new Constraints(new Leading(630, 100, 6, 6), new Leading(70, 6, 6)));
		add(getDoorlineLabel(), new Constraints(new Leading(630, 130, 10, 10), new Leading(100, 130, 6, 6)));

		// P&O Combo
		add(getMailboxCombo(), new Constraints(new Leading(740, 100, 6, 6), new Leading(40, 6, 6)));
		add(getGrabCallFromVoicemailBoxButton(), new Constraints(new Leading(740, 100, 10, 10), new Leading(70, 6,
			6)));

		add(getCallTableScrollPane(), new Constraints(new Bilateral(250, 10, 500), new Leading(320, 140, 6, 6)));

		add(getCallDetailsTableScrollPane(), new Constraints(new Leading(9, 230, 6, 6),
			new Leading(230, 230, 6, 6)));
	}

	private JLabel getDtmfCodeLabel() {
		if (dtmfCodeLabel == null) {
			dtmfCodeLabel = new JLabel();
			dtmfCodeLabel.setText("Dtmf Code");
		}
		return dtmfCodeLabel;
	}

	private JScrollPane getMwiListScrollPane() {
		if (mwiListScrollPane == null) {
			mwiListScrollPane = new JScrollPane();
			mwiListScrollPane.setViewportView(getMwiList());
		}
		return mwiListScrollPane;
	}

	private JList getMwiList() {
		if (mwiList == null) {
			mwiList = new JList();
		}
		return mwiList;
	}

	private JLabel getDoorlineLabel() {
		if (doorlineLabel == null) {
			doorlineLabel = new JLabel("Doorline Image", SwingConstants.CENTER);
			doorlineLabel.setBorder(BorderFactory.createLineBorder(Color.black));
		}
		return doorlineLabel;
	}

	private JLabel getAvatarLabel() {
		if (avatarLabel == null) {
			avatarLabel = new JLabel("Avatar Image", SwingConstants.CENTER);
			avatarLabel.setBorder(BorderFactory.createLineBorder(Color.black));
		}
		return avatarLabel;
	}

	private JComboBox getMailboxCombo() {
		if (mailboxCombo == null) {
			mailboxCombo = new JComboBox();
			mailboxCombo.setModel(new DefaultComboBoxModel(new Object[0]));
			mailboxCombo.setRenderer(new MailboxesListCellRenderer());
		}
		return mailboxCombo;
	}

	private JButton getTransferToVoicemailBoxButton() {
		if (transferToVoicemailBoxButton == null) {
			transferToVoicemailBoxButton = new JButton();
			transferToVoicemailBoxButton.setText("transferToVM");
			transferToVoicemailBoxButton.setEnabled(false);
			transferToVoicemailBoxButton.setToolTipText(selectSingleCall);
		}
		return transferToVoicemailBoxButton;
	}

	private JLabel getOriginatingNumberLabel() {
		if (originatingNumberLabel == null) {
			originatingNumberLabel = new JLabel();
			originatingNumberLabel.setText("originatingNumber");
		}
		return originatingNumberLabel;
	}

	private JTextField getOriginatingNumberText() {
		if (originatingNumberText == null) {
			originatingNumberText = new JTextField();
			originatingNumberText.setText(Config.getValue("originatingNumber"));
			originatingNumberText.addFocusListener(new FocusAdapter() {

				public void focusGained(FocusEvent event) {
					originatingNumberTextFocusFocusGained(event);
				}
			});
		}
		return originatingNumberText;
	}

	private JTextField getPlayDtmfText() {
		if (playDtmfText == null) {
			playDtmfText = new JTextField();
			playDtmfText.setText(Config.getValue("playDtmfText"));
		}
		return playDtmfText;
	}

	private JButton getPlaceCallWithNumberButton() {
		if (placeCallWithNumberButton == null) {
			placeCallWithNumberButton = new JButton();
			placeCallWithNumberButton.setText("placeCallWithNumber");
			//placeCallWithNumberButton.setEnabled(false);
		}
		return placeCallWithNumberButton;
	}

	private JButton getDialVoicemailBoxButton() {
		if (dialVoicemailBoxButton == null) {
			dialVoicemailBoxButton = new JButton();
			dialVoicemailBoxButton.setText("Dial VM");
		}
		return dialVoicemailBoxButton;
	}

	private JButton getParkAndOrbitCallButton() {
		if (parkAndOrbitCallButton == null) {
			parkAndOrbitCallButton = new JButton();
			parkAndOrbitCallButton.setText("P&O");
			parkAndOrbitCallButton.setEnabled(true);
		}
		return parkAndOrbitCallButton;
	}

	private JTextField getDestinationNumberText() {
		if (destinationNumberText == null) {
			destinationNumberText = new JTextField();
			destinationNumberText.setText(Config.getValue("destinationNumber"));
			destinationNumberText.addFocusListener(new FocusAdapter() {

				public void focusGained(FocusEvent event) {
					destinationNumberTextFocusFocusGained(event);
				}
			});
		}
		return destinationNumberText;
	}

	private JLabel getDestinationNumberLabel() {
		if (destinationNumberLabel == null) {
			destinationNumberLabel = new JLabel();
			destinationNumberLabel.setText("destinationNumber");
		}
		return destinationNumberLabel;
	}

	private JLabel getNewCallIdLabel() {
		if (newCallIdLabel == null) {
			newCallIdLabel = new JLabel();
			newCallIdLabel.setText("callId");
		}
		return newCallIdLabel;
	}

	private JTextField getNewCallIdText() {
		if (newCallIdText == null) {
			newCallIdText = new JTextField();
			newCallIdText.setText(Config.getValue("callId"));
			newCallIdText.addFocusListener(new FocusAdapter() {

				public void focusGained(FocusEvent event) {
					newCallIdTextFocusFocusGained(event);
				}
			});
		}
		return newCallIdText;
	}

	private JButton getGetCallStateButton() {
		if (getCallStateButton == null) {
			getCallStateButton = new JButton();
			getCallStateButton.setText("getCallState");
			getCallStateButton.setEnabled(false);
			getCallStateButton.setToolTipText(selectSingleCall);
		}
		return getCallStateButton;
	}

	private JButton getGrabCallFromVoicemailBox() {
		if (grabCallFromVoicemailBoxButton == null) {
			grabCallFromVoicemailBoxButton = new JButton();
			grabCallFromVoicemailBoxButton.setText("grabFromVM");
			grabCallFromVoicemailBoxButton.setEnabled(false);
			grabCallFromVoicemailBoxButton.setToolTipText(selectSingleCall);
		}
		return grabCallFromVoicemailBoxButton;
	}

	private JButton getGetCallIdsButton() {
		if (getCallIdsButton == null) {
			getCallIdsButton = new JButton();
			getCallIdsButton.setText("getCallIds");
		}
		return getCallIdsButton;
	}

	private JButton getPlaceCallWithPhoneButton() {
		if (placeCallWithPhoneButton == null) {
			placeCallWithPhoneButton = new JButton();
			placeCallWithPhoneButton.setText("placeCallWithPhone");
			//placeCallWithPhoneButton.setEnabled(false);
		}
		return placeCallWithPhoneButton;
	}

	private JButton getImageFromDoorlineButton() {
		if (imageFromDoorlineButton == null) {
			imageFromDoorlineButton = new JButton();
			imageFromDoorlineButton.setText("get Doorline Image");
			imageFromDoorlineButton.setEnabled(false);
			imageFromDoorlineButton.setToolTipText(selectSingleCall);
		}
		return imageFromDoorlineButton;
	}

	private JButton getGetMwiStateButton() {
		if (getMwiStateButton == null) {
			getMwiStateButton = new JButton();
			getMwiStateButton.setText("getMwiState");
		}
		return getMwiStateButton;
	}

	private JButton getGrabCallFromVoicemailBoxButton() {
		if (grabCallFromVoicemailBoxButton == null) {
			grabCallFromVoicemailBoxButton = new JButton();
			grabCallFromVoicemailBoxButton.setText("grabFromVM");
			grabCallFromVoicemailBoxButton.setEnabled(false);
			grabCallFromVoicemailBoxButton.setToolTipText(selectSingleCall);
		}
		return grabCallFromVoicemailBoxButton;
	}

	private JButton getOpenDoorlineButton() {
		if (openDoorlineButton == null) {
			openDoorlineButton = new JButton();
			openDoorlineButton.setText("Open Door");
			openDoorlineButton.setEnabled(false);
			openDoorlineButton.setToolTipText(selectSingleCall);
		}
		return openDoorlineButton;
	}

	private JButton getImageForHashButton() {
		if (imageForHashButton == null) {
			imageForHashButton = new JButton();
			imageForHashButton.setText("getAvatar");
			imageForHashButton.setEnabled(false);
			imageForHashButton.setToolTipText(selectSingleCall);
		}
		return imageForHashButton;
	}

	private JButton getPlayDtmfButton() {
		if (playDtmfButton == null) {
			playDtmfButton = new JButton();
			playDtmfButton.setText("play this Dtmf");
			playDtmfButton.setEnabled(false);
			playDtmfButton.setToolTipText(selectSingleCall);
		}
		return playDtmfButton;
	}

	private JButton getHangupCallButton() {
		if (hangupCallButton == null) {
			hangupCallButton = new JButton();
			hangupCallButton.setText("hangup");
			hangupCallButton.setEnabled(false);
			hangupCallButton.setToolTipText(selectSingleCall);
		}
		return hangupCallButton;
	}

	private JButton getHoldCallButton() {
		if (holdCallButton == null) {
			holdCallButton = new JButton();
			holdCallButton.setText("hold");
			holdCallButton.setEnabled(false);
			holdCallButton.setToolTipText(selectSingleCall);
		}
		return holdCallButton;
	}

	private JButton getTransferConsultationCallButton() {
		if (transferConsultationCallButton == null) {
			transferConsultationCallButton = new JButton();
			transferConsultationCallButton.setText("transferConsult");
			transferConsultationCallButton.setEnabled(false);
			transferConsultationCallButton.setToolTipText(selectSingleCall);
		}
		return transferConsultationCallButton;
	}

	private JButton getTransferConsultationCallToConferenceCallButton() {
		if (transferToConferenceConsultationCallCallButton == null) {
			transferToConferenceConsultationCallCallButton = new JButton();
			transferToConferenceConsultationCallCallButton.setText("transferConference");
			transferToConferenceConsultationCallCallButton.setEnabled(false);
			transferToConferenceConsultationCallCallButton.setToolTipText(selectSingleCall);
		}
		return transferToConferenceConsultationCallCallButton;
	}

	private JButton getPlaceConsultationCallButton() {
		if (placeConsultationCallButteon == null) {
			placeConsultationCallButteon = new JButton();
			placeConsultationCallButteon.setText("consult");
			placeConsultationCallButteon.setEnabled(false);
			placeConsultationCallButteon.setToolTipText(selectSingleCall);
		}
		return placeConsultationCallButteon;
	}

	private JButton getMonitorCallButton() {
		if (monitorCallButton == null) {
			monitorCallButton = new JButton();
			monitorCallButton.setText("monitor");
			monitorCallButton.setEnabled(false);
			monitorCallButton.setToolTipText(selectSingleCall);
		}
		return monitorCallButton;
	}

	private JButton getResumeCallButton() {
		if (resumeCallButton == null) {
			resumeCallButton = new JButton();
			resumeCallButton.setText("resume");
			resumeCallButton.setEnabled(false);
			resumeCallButton.setToolTipText(selectSingleCall);
		}
		return resumeCallButton;
	}

	private JButton getAcceptCallButton() {
		if (acceptCallButton == null) {
			acceptCallButton = new JButton();
			acceptCallButton.setText("accept");
			acceptCallButton.setEnabled(false);
			acceptCallButton.setToolTipText(selectSingleCall);
		}
		return acceptCallButton;
	}

	private JButton getConferenceCallButton() {
		if (conferenceCallButton == null) {
			conferenceCallButton = new JButton();
			conferenceCallButton.setText("conference");
		}
		return conferenceCallButton;
	}

	private JButton getForwardCallButton() {
		if (forwardCallButton == null) {
			forwardCallButton = new JButton();
			forwardCallButton.setText("forward/blindTransfer");
			forwardCallButton.setEnabled(false);
			forwardCallButton.setToolTipText(selectSingleCall);
		}
		return forwardCallButton;
	}

	private JButton getTransferCallButton() {
		if (transferCallButton == null) {
			transferCallButton = new JButton();
			transferCallButton.setText("transfer");
			transferCallButton.setEnabled(false);
			transferCallButton.setToolTipText(selectTwoCalls);
		}
		return transferCallButton;
	}

	private JButton getSwitchPhoneButton() {
		if (switchPhoneButton == null) {
			switchPhoneButton = new JButton();
			switchPhoneButton.setText("call2go");
			switchPhoneButton.setEnabled(false);
			switchPhoneButton.setToolTipText(selectSingleCall);
		}
		return switchPhoneButton;
	}

	private JScrollPane getCallTableScrollPane() {
		if (callTableScrollPane == null) {
			callTableScrollPane = new JScrollPane();
			callTableScrollPane.setViewportView(getCallTable());
		}
		return callTableScrollPane;
	}

	private JTable getCallTable() {
		if (callTable == null) {
			callTable = new JTable();
			callTable.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			callTable.setRowSelectionAllowed(true);
			callTable.setColumnSelectionAllowed(false);
			callTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return callTable;
	}

	private JScrollPane getCallDetailsTableScrollPane() {
		if (callDetailsTableScrollPane == null) {
			callDetailsTableScrollPane = new JScrollPane(getCallDetailsTable(),
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			getCallDetailsTable().setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		}
		return callDetailsTableScrollPane;
	}

	private JTable getCallDetailsTable() {
		if (callDetailsTable == null) {
			callDetailsTable = new JTable();
			callDetailsTable.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			callDetailsTable.setRowSelectionAllowed(false);
			callDetailsTable.setColumnSelectionAllowed(false);
			callDetailsTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		}
		return callDetailsTable;
	}

	private void destinationNumberTextFocusFocusGained(FocusEvent event) {
		destinationNumberText.selectAll();
	}

	private void newCallIdTextFocusFocusGained(FocusEvent event) {
		newCallIdText.selectAll();
	}

	private PropertyChangeListener enableGetCallStateButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_GET_CALL_STATE_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getGetCallStateButton().setEnabled(enableButton);
				getGetCallStateButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enablePlaceCallWithPhoneButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_PLACE_CALL_WITH_PHONE_IS_ALLOWED) {
				getPlaceCallWithPhoneButton().setEnabled((Boolean) e.getNewValue());
			}
		}
	};

	private PropertyChangeListener enablePlaceCallWithNumberButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_PLACE_CALL_WITH_NUMBER_IS_ALLOWED) {
				getPlaceCallWithNumberButton().setEnabled((Boolean) e.getNewValue());
			}
		}
	};

	private PropertyChangeListener enableHangupButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_HANGUP_CALL_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getHangupCallButton().setEnabled(enableButton);
				getHangupCallButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableHoldButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_HOLD_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getHoldCallButton().setEnabled(enableButton);
				getHoldCallButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableResumeCallButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_RESUME_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getResumeCallButton().setEnabled(enableButton);
				getResumeCallButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableAcceptCallButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_ACCEPT_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getAcceptCallButton().setEnabled(enableButton);
				getAcceptCallButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableForwardButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_FORWARD_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getForwardCallButton().setEnabled(enableButton);
				getForwardCallButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableTransferButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_TRANSFER_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getTransferCallButton().setEnabled(enableButton);
				getTransferCallButton().setToolTipText(enableButton ? null : selectTwoCalls);
			}
		}
	};

	private PropertyChangeListener enableSwitchPhoneButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_SWITCH_PHONE_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getSwitchPhoneButton().setEnabled(enableButton);
				getSwitchPhoneButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableMonitorCallButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_MONITOR_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getMonitorCallButton().setEnabled(enableButton);
				getMonitorCallButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableParkAndOrbitCallButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_PARK_AND_ORBIT_IS_ALLOWED) {
				getParkAndOrbitCallButton().setEnabled((Boolean) e.getNewValue());
			}
		}
	};

	private PropertyChangeListener enablePlaceConsultCallButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.CONSULT_CALL_IS_ALLOWED) {
				getPlaceConsultationCallButton().setEnabled((Boolean) e.getNewValue());
			}
		}
	};

	private PropertyChangeListener enableTransferConsultCallButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.TRANSFER_CONSULT_CALL_IS_ALLOWED) {
				getTransferConsultationCallButton().setEnabled((Boolean) e.getNewValue());
			}
		}
	};

	private PropertyChangeListener enableTransferToConferenceConsultationCallCallButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_TRANSFER_TO_CONFERENCE_CONSULTATION_CALL_IS_ALLOWED) {
				getTransferConsultationCallToConferenceCallButton().setEnabled((Boolean) e.getNewValue());
			}
		}
	};

	private PropertyChangeListener enableDialVoicemailBoxButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.CALL_VOICEMAIL_BOX_IS_ALLOWED) {
				getDialVoicemailBoxButton().setEnabled((Boolean) e.getNewValue());
			}
		}
	};

	private PropertyChangeListener enableGrabCallFromVoicemailBoxButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.GRAB_VOICEMAIL_BOX_CALLER_WITH_PHONE_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getGrabCallFromVoicemailBoxButton().setEnabled(enableButton);
				getGrabCallFromVoicemailBoxButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableTransferToVoicemailBoxButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_TRANSFER_TO_VOICEMAIL_BOX_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getTransferToVoicemailBoxButton().setEnabled(enableButton);
				getTransferToVoicemailBoxButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enablePlayDtmfButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_PLAY_DTMF_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getPlayDtmfButton().setEnabled(enableButton);
				getPlayDtmfButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableOpenDoorlineButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_OPEN_DOOR_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getOpenDoorlineButton().setEnabled(enableButton);
				getOpenDoorlineButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableGetAvatarButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_GET_AVATAR_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getImageForHashButton().setEnabled(enableButton);
				getImageForHashButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private PropertyChangeListener enableGetDoorlineImageButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == CallViewModel.PROPERTY_GET_DOORLINE_IMAGE_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getImageFromDoorlineButton().setEnabled(enableButton);
				getImageFromDoorlineButton().setToolTipText(enableButton ? null : selectSingleCall);
			}
		}
	};

	private void originatingNumberTextFocusFocusGained(FocusEvent event) {
		originatingNumberText.selectAll();
	}

	public void setViewModel(CallViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final CallViewModel viewModel) {

		getGetCallIdsButton().addActionListener(viewModel.getGetCallIdsActionListener());
		getPlaceCallWithPhoneButton().addActionListener(viewModel.getPlaceCallWithPhoneActionListener());
		getPlaceCallWithNumberButton().addActionListener(viewModel.getPlaceCallWithNumberActionListener());
		getHangupCallButton().addActionListener(viewModel.getHangupCallActionListener());
		getHoldCallButton().addActionListener(viewModel.getHoldCallActionListener());
		getResumeCallButton().addActionListener(viewModel.getResumeCallActionListener());
		getAcceptCallButton().addActionListener(viewModel.getAcceptCallActionListener());
		getConferenceCallButton().addActionListener(viewModel.getConferenceCallActionListener());
		getTransferCallButton().addActionListener(viewModel.getTransferCallActionListener());
		getSwitchPhoneButton().addActionListener(viewModel.getSwitchPhoneActionListener());
		getMonitorCallButton().addActionListener(viewModel.getMonitorCallActionListener());
		getTransferConsultationCallButton().addActionListener(
			viewModel.getTransferConsultationCallActionListenerActionListener());
		getTransferConsultationCallToConferenceCallButton().addActionListener(
			viewModel.getTransferToConferenceConsultationCallActionListenerActionListener());
		getPlaceConsultationCallButton().addActionListener(
			viewModel.getPlaceConsultationCallWithPhoneActionListener());
		getForwardCallButton().addActionListener(viewModel.getForwardCallActionListener());
		getGetCallStateButton().addActionListener(viewModel.getGetCallStateActionListener());
		getParkAndOrbitCallButton().addActionListener(viewModel.getParkAndOrbitActionListener());
		getDialVoicemailBoxButton().addActionListener(viewModel.getDialVoicemailBoxActionListener());
		getGrabCallFromVoicemailBox().addActionListener(viewModel.getGrabCallFromVoicemailBoxActionListener());
		getTransferToVoicemailBoxButton().addActionListener(viewModel.getTransferToVoicemailBoxActionListener());
		getPlayDtmfButton().addActionListener(viewModel.getPlayDtmfActionListener());
		getImageForHashButton().addActionListener(viewModel.getImageForHashActionListener());
		getOpenDoorlineButton().addActionListener(viewModel.getOpenDoorLineActionListener());
		getImageFromDoorlineButton().addActionListener(viewModel.getImageFromDoorlineActionListener());
		getGetMwiStateButton().addActionListener(viewModel.getGetMessagWaitingIndicationActionListener());

		BeanAdapter<CallViewModel> beanAdapter = new BeanAdapter<CallViewModel>(viewModel, true);
		viewModel.addPropertyChangeListener(enablePlaceCallWithPhoneButton);
		viewModel.addPropertyChangeListener(enableForwardButton);
		viewModel.addPropertyChangeListener(enablePlaceCallWithNumberButton);
		viewModel.addPropertyChangeListener(enablePlayDtmfButton);
		viewModel.addPropertyChangeListener(enableHangupButton);
		viewModel.addPropertyChangeListener(enableSwitchPhoneButton);
		viewModel.addPropertyChangeListener(enableMonitorCallButton);
		viewModel.addPropertyChangeListener(enableDialVoicemailBoxButton);
		viewModel.addPropertyChangeListener(enablePlaceConsultCallButton);
		viewModel.addPropertyChangeListener(enableTransferConsultCallButton);
		viewModel.addPropertyChangeListener(enableTransferToConferenceConsultationCallCallButton);
		viewModel.addPropertyChangeListener(enableGrabCallFromVoicemailBoxButton);
		viewModel.addPropertyChangeListener(enableTransferToVoicemailBoxButton);
		viewModel.addPropertyChangeListener(enableGetCallStateButton);
		viewModel.addPropertyChangeListener(enableParkAndOrbitCallButton);
		viewModel.addPropertyChangeListener(enableResumeCallButton);
		viewModel.addPropertyChangeListener(enableAcceptCallButton);
		viewModel.addPropertyChangeListener(enableTransferButton);
		viewModel.addPropertyChangeListener(enableHoldButton);
		viewModel.addPropertyChangeListener(enableOpenDoorlineButton);
		viewModel.addPropertyChangeListener(enableGetDoorlineImageButton);
		viewModel.addPropertyChangeListener(enableGetAvatarButton);

		if (parkAndOrbitIdCombo == null) {
			LinkedList<String> values = new java.util.LinkedList<String>();
			for (int i = 0; i < 100; i++) {
				String value = "";
				if (i < 10) {
					value += "0";
				}
				value += String.valueOf(i);
				values.add(value);
			}

			parkAndOrbitIdCombo = new JComboBox(new ComboBoxAdapter(values.toArray(),
				beanAdapter.getValueModel(CallViewModel.PROPERTY_PARK_AND_ORBIT_NUMBER)));
			parkAndOrbitIdCombo.setSelectedIndex(0);

		}
		add(parkAndOrbitIdCombo, new Constraints(new Leading(740, 100, 6, 6), new Leading(10, 6, 6)));

		Bindings.bind(destinationNumberText, beanAdapter.getValueModel(CallViewModel.PROPERTY_DESTINATION_NUMBER));
		Bindings.bind(originatingNumberText, beanAdapter.getValueModel(CallViewModel.PROPERTY_ORIGINATING_NUMBER));
		Bindings.bind(newCallIdText, beanAdapter.getValueModel(CallViewModel.PROPERTY_NEW_CALL_ID));

		Bindings.bind(playDtmfText, beanAdapter.getValueModel(CallViewModel.PROPERTY_DTMF_DIGITS));

		Bindings.bind(
			mwiList,
			new SelectionInList<String>(beanAdapter.getValueModel(CallViewModel.PROPERTY_MWI_LIST),
				beanAdapter.getValueModel(CallViewModel.PROPERTY_SELECTED_MWI)));

		Bindings.bind(mailboxCombo,
			new SelectionInList<Mailbox>(beanAdapter.getValueModel(CallViewModel.PROPERTY_MAILBOX_VALUES),
				beanAdapter.getValueModel(CallViewModel.PROPERTY_SELECTED_MAILBOX)));

		callTable.setModel(viewModel.getCallTableModel());
		callTable.setSelectionModel(new MultiListSelectionAdapter(viewModel.getSelectedRows()));
		callTable.getSelectionModel().addListSelectionListener(viewModel.getSelectedCallsListener());
		callTable.removeColumn(callTable.getColumn(CallProperties.timestamp.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.groupId.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.peerNames.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.channelNames.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.sipCallIds.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.caller.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.connectedTimestamp.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.recorder.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.recorded.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.recordOwner.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.avatarHash.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.doorlineCamUrl.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.doorlineDtmfCode.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.doorlineImageProviderId.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.forwardType.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.forwarderCallerIdName.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.forwarderCallerIdNumber.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.conferenceRoomId.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.conferenceAdmin.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.referenceOfConsultation.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.consultationCall.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.duration.toString()));
		callTable.removeColumn(callTable.getColumn(CallProperties.internalCall.toString()));

		final HashSet<String> callDetails = new HashSet<String>();
		callDetails.add(CallProperties.timestamp.toString());
		callDetails.add(CallProperties.groupId.toString());
		//		callDetails.add(CallProperties.peerNames.toString());
		//		callDetails.add(CallProperties.channelNames.toString());
		//		callDetails.add(CallProperties.sipCallIds.toString());
		callDetails.add(CallProperties.caller.toString());
		callDetails.add(CallProperties.connectedTimestamp.toString());
		callDetails.add(CallProperties.recorder.toString());
		callDetails.add(CallProperties.recorded.toString());
		callDetails.add(CallProperties.recordOwner.toString());
		callDetails.add(CallProperties.avatarHash.toString());
		callDetails.add(CallProperties.doorlineCamUrl.toString());
		callDetails.add(CallProperties.doorlineDtmfCode.toString());
		callDetails.add(CallProperties.doorlineImageProviderId.toString());
		callDetails.add(CallProperties.forwardType.toString());
		callDetails.add(CallProperties.forwarderCallerIdName.toString());
		callDetails.add(CallProperties.forwarderCallerIdNumber.toString());
		callDetails.add(CallProperties.conferenceRoomId.toString());
		callDetails.add(CallProperties.conferenceAdmin.toString());
		callDetails.add(CallProperties.referenceOfConsultation.toString());
		callDetails.add(CallProperties.consultationCall.toString());
		callDetails.add(CallProperties.duration.toString());
		callDetails.add(CallProperties.internalCall.toString());

		RowFilter<TableModel, Integer> callDetailsRowFilter = new RowFilter<TableModel, Integer>() {
			public boolean include(Entry<? extends TableModel, ? extends Integer> entry) {
				return callDetails.contains(entry.getValue(0).toString());
			}
		};

		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(viewModel.getCallDetailsTableModel());
		sorter.setRowFilter(callDetailsRowFilter);
		sorter.setSortable(0, false);
		sorter.setSortable(1, false);

		callDetailsTable.setModel(viewModel.getCallDetailsTableModel());
		callDetailsTable.setRowSorter(sorter);
		callDetailsTable.getColumnModel().getColumn(1).setMinWidth(200);

		PropertyConnector.connectAndUpdate(beanAdapter.getValueModel(CallViewModel.PROPERTY_AVATAR), avatarLabel,
			"icon");

		PropertyConnector.connectAndUpdate(beanAdapter.getValueModel(CallViewModel.PROPERTY_DOORLINE),
			doorlineLabel, "icon");

	}
}
