/*
 * $Id$
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.examples.uci30.swing.view;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.view.binding.MultiListSelectionAdapter;
import de.starface.examples.uci30.swing.viewmodel.FaxListViewModel;
import de.starface.integration.uci.java.v30.values.FaxListEntryFolder;
import de.starface.integration.uci.java.v30.values.FaxListEntryProperties;
import de.starface.integration.uci.java.v30.values.OrderDirection;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Date;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;

/**
 * This panel provides a GUI for the UCI communication fax list functionality
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class FaxListPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JTable table;
	private JScrollPane tableScrollPane;
	private JComboBox orderPropertyCombo;
	private JComboBox orderDirectionCombo;
	private JFormattedTextField receivedAfterText;
	private JFormattedTextField countOffsetText;
	private JFormattedTextField countLimitText;
	private JLabel receivedBeforeLabel;
	private JLabel receivedAfterLabel;
	private JFormattedTextField receivedBeforeText;
	private JLabel folderRestrictionLabel;
	private JComboBox folderRestrictionCombo;
	private JLabel orderPropertyLabel;
	private JLabel orderDirectionLabel;
	private JLabel countOffsetLabel;
	private JLabel countLimitLabel;
	private JButton getFaxListButton;
	private JButton deleteFaxListEntryButton;
	private JButton getFaxFileUrlButton;
	private JButton moveFaxListEntryButton;
	private JLabel destinationFolderLabel;
	private JComboBox destinationFolderCombo;
	private JTextField faxFileUrlText;
	private JSeparator jSeparator0;

	private JButton deleteFaxListEntriesButton;

	private JButton moveFaxListEntriesButton;

	private JLabel groupRestrictionLabel;

	private JComboBox groupRestrictionCombo;

	private JButton getFaxFileButton;

	private JTextField faxFileText;

	private JButton browseUrlButton;

	private JButton openFileButton;

	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public FaxListPanel() {
		initComponents();
	}

	private void initComponents() {
		setLayout(new GroupLayout());
		add(getCountOffsetText(), new Constraints(new Leading(524, 40, 10, 10), new Leading(6, 6, 6)));
		add(getOrderPropertyLabel(), new Constraints(new Leading(238, 6, 6), new Leading(12, 6, 6)));
		add(getOrderDirectionLabel(), new Constraints(new Leading(238, 6, 6), new Leading(46, 6, 6)));
		add(getCountOffsetLabel(), new Constraints(new Leading(451, 6, 6), new Leading(12, 6, 6)));
		add(getCountLimitLabel(), new Constraints(new Leading(450, 6, 6), new Leading(45, 6, 6)));
		add(getCountLimitText(), new Constraints(new Leading(524, 40, 6, 6), new Leading(40, 6, 6)));
		add(getOrderDirectionCombo(), new Constraints(new Leading(320, 122, 10, 10), new Leading(42, 6, 6)));
		add(getOrderPropertyCombo(), new Constraints(new Leading(320, 122, 6, 6), new Leading(7, 6, 6)));
		add(getFolderRestrictionLabel(), new Constraints(new Leading(6, 6, 6), new Leading(79, 6, 6)));
		add(getJSeparator0(), new Constraints(new Leading(6, 555, 6, 6), new Leading(107, 5, 10, 10)));
		add(getDestinationFolderLabel(), new Constraints(new Leading(6, 6, 6), new Leading(118, 6, 6)));
		add(getDestinationFolderCombo(), new Constraints(new Leading(106, 120, 6, 6), new Leading(113, 6, 6)));
		add(getReceivedAfterText(), new Constraints(new Leading(94, 136, 6, 6), new Leading(6, 6, 6)));
		add(getReceivedBeforeText(), new Constraints(new Leading(94, 136, 6, 6), new Leading(41, 6, 6)));
		add(getReceivedAfterLabel(), new Constraints(new Leading(6, 6, 6), new Leading(12, 6, 6)));
		add(getReceivedBeforeLabel(), new Constraints(new Leading(6, 6, 6), new Leading(47, 6, 6)));
		add(getGetFaxFileUrlButton(), new Constraints(new Leading(6, 101, 10, 10), new Leading(142, 6, 6)));
		add(getMoveFaxListEntryButton(), new Constraints(new Leading(238, 152, 10, 10), new Leading(112, 6, 6)));
		add(getMoveFaxListEntriesButton(), new Constraints(new Leading(395, 166, 6, 6), new Leading(112, 6, 6)));
		add(getGroupRestrictionLabel(), new Constraints(new Leading(238, 6, 6), new Leading(80, 6, 6)));
		add(getFolderRestrictionCombo(), new Constraints(new Leading(105, 125, 6, 6), new Leading(74, 6, 6)));
		add(getGroupRestrictionCombo(), new Constraints(new Leading(335, 107, 6, 6), new Leading(75, 6, 6)));
		add(getGetFaxListButton(), new Constraints(new Leading(451, 113, 6, 6), new Leading(74, 26, 6, 6)));
		add(getGetFaxFileButton(), new Constraints(new Leading(6, 101, 6, 6), new Leading(170, 6, 6)));
		add(getBrowseUrlButton(), new Constraints(new Leading(492, 6, 6), new Leading(142, 6, 6)));
		add(getFaxFileUrlText(), new Constraints(new Leading(105, 381, 6, 6), new Leading(142, 6, 6)));
		add(getOpenFileButton(), new Constraints(new Leading(491, 70, 6, 6), new Leading(170, 6, 6)));
		add(getFaxFileText(), new Constraints(new Leading(105, 381, 6, 6), new Leading(170, 6, 6)));
		add(getDeleteFaxListEntriesButton(), new Constraints(new Leading(290, 271, 6, 6), new Leading(204, 6, 6)));
		add(getDeleteFaxListEntryButton(), new Constraints(new Leading(6, 278, 6, 6), new Leading(204, 6, 6)));
		add(getTableScrollPane(), new Constraints(new Bilateral(6, 10, 558), new Leading(238, 205, 6, 6)));
		setSize(574, 450);
	}

	private JButton getOpenFileButton() {
		if (openFileButton == null) {
			openFileButton = new JButton();
			openFileButton.setText("open");
		}
		return openFileButton;
	}

	private JButton getBrowseUrlButton() {
		if (browseUrlButton == null) {
			browseUrlButton = new JButton();
			browseUrlButton.setText("browse");
			browseUrlButton.setEnabled(false);
			browseUrlButton.setToolTipText("do a getFaxFileUrl");

		}
		return browseUrlButton;
	}

	private JTextField getFaxFileText() {
		if (faxFileText == null) {
			faxFileText = new JTextField();
			faxFileText.setEditable(false);
			faxFileText.addFocusListener(new FocusAdapter() {

				public void focusGained(FocusEvent event) {
					faxFileTextFocusFocusGained(event);
				}
			});
		}
		return faxFileText;
	}

	private JButton getGetFaxFileButton() {
		if (getFaxFileButton == null) {
			getFaxFileButton = new JButton();
			getFaxFileButton.setText("getFaxFile");
		}
		return getFaxFileButton;
	}

	private JComboBox getGroupRestrictionCombo() {
		if (groupRestrictionCombo == null) {
			groupRestrictionCombo = new JComboBox();
			groupRestrictionCombo.setRenderer(new GroupSettingsListCellRenderer());
		}
		return groupRestrictionCombo;
	}

	private JLabel getGroupRestrictionLabel() {
		if (groupRestrictionLabel == null) {
			groupRestrictionLabel = new JLabel();
			groupRestrictionLabel.setText("groupRestriction");
		}
		return groupRestrictionLabel;
	}

	private JButton getMoveFaxListEntriesButton() {
		if (moveFaxListEntriesButton == null) {
			moveFaxListEntriesButton = new JButton();
			moveFaxListEntriesButton.setText("moveFaxListEntries");
		}
		return moveFaxListEntriesButton;
	}

	private JButton getDeleteFaxListEntriesButton() {
		if (deleteFaxListEntriesButton == null) {
			deleteFaxListEntriesButton = new JButton();
			deleteFaxListEntriesButton.setText("deleteFaxListEntries");
		}
		return deleteFaxListEntriesButton;
	}

	private JSeparator getJSeparator0() {
		if (jSeparator0 == null) {
			jSeparator0 = new JSeparator();
		}
		return jSeparator0;
	}

	private JTextField getFaxFileUrlText() {
		if (faxFileUrlText == null) {
			faxFileUrlText = new JTextField();
			faxFileUrlText.setEditable(false);
			faxFileUrlText.addFocusListener(new FocusAdapter() {

				public void focusGained(FocusEvent event) {
					faxFileUrlTextFocusFocusGained(event);
				}
			});
		}
		return faxFileUrlText;
	}

	private JComboBox getDestinationFolderCombo() {
		if (destinationFolderCombo == null) {
			destinationFolderCombo = new JComboBox();
			destinationFolderCombo.setModel(new DefaultComboBoxModel(FaxListEntryFolder.values()));
		}
		return destinationFolderCombo;
	}

	private JLabel getDestinationFolderLabel() {
		if (destinationFolderLabel == null) {
			destinationFolderLabel = new JLabel();
			destinationFolderLabel.setText("destinationFolder");
		}
		return destinationFolderLabel;
	}

	private JButton getMoveFaxListEntryButton() {
		if (moveFaxListEntryButton == null) {
			moveFaxListEntryButton = new JButton();
			moveFaxListEntryButton.setText("moveFaxListEntry");
		}
		return moveFaxListEntryButton;
	}

	private JButton getGetFaxFileUrlButton() {
		if (getFaxFileUrlButton == null) {
			getFaxFileUrlButton = new JButton();
			getFaxFileUrlButton.setText("getFaxFileUrl");
		}
		return getFaxFileUrlButton;
	}

	private JButton getDeleteFaxListEntryButton() {
		if (deleteFaxListEntryButton == null) {
			deleteFaxListEntryButton = new JButton();
			deleteFaxListEntryButton.setText("deleteFaxListEntry");
		}
		return deleteFaxListEntryButton;
	}

	private JButton getGetFaxListButton() {
		if (getFaxListButton == null) {
			getFaxListButton = new JButton();
			getFaxListButton.setText("getFaxList");
		}
		return getFaxListButton;
	}

	private JLabel getCountLimitLabel() {
		if (countLimitLabel == null) {
			countLimitLabel = new JLabel();
			countLimitLabel.setText("countLimit");
		}
		return countLimitLabel;
	}

	private JLabel getCountOffsetLabel() {
		if (countOffsetLabel == null) {
			countOffsetLabel = new JLabel();
			countOffsetLabel.setText("countOffset");
		}
		return countOffsetLabel;
	}

	private JLabel getOrderDirectionLabel() {
		if (orderDirectionLabel == null) {
			orderDirectionLabel = new JLabel();
			orderDirectionLabel.setText("orderDirection");
		}
		return orderDirectionLabel;
	}

	private JLabel getOrderPropertyLabel() {
		if (orderPropertyLabel == null) {
			orderPropertyLabel = new JLabel();
			orderPropertyLabel.setText("orderProperty");
		}
		return orderPropertyLabel;
	}

	private JComboBox getFolderRestrictionCombo() {
		if (folderRestrictionCombo == null) {
			folderRestrictionCombo = new JComboBox();
		}
		return folderRestrictionCombo;
	}

	private JLabel getFolderRestrictionLabel() {
		if (folderRestrictionLabel == null) {
			folderRestrictionLabel = new JLabel();
			folderRestrictionLabel.setText("folderRestriction");
		}
		return folderRestrictionLabel;
	}

	private JTextField getReceivedBeforeText() {
		if (receivedBeforeText == null) {
			receivedBeforeText = new JFormattedTextField(new EmptyDateFormatter());
		}
		return receivedBeforeText;
	}

	private JLabel getReceivedAfterLabel() {
		if (receivedAfterLabel == null) {
			receivedAfterLabel = new JLabel();
			receivedAfterLabel.setText("receivedAfter");
		}
		return receivedAfterLabel;
	}

	private JLabel getReceivedBeforeLabel() {
		if (receivedBeforeLabel == null) {
			receivedBeforeLabel = new JLabel();
			receivedBeforeLabel.setText("receivedBefore");
		}
		return receivedBeforeLabel;
	}

	private JTextField getCountLimitText() {
		if (countLimitText == null) {
			countLimitText = new JFormattedTextField();
		}
		return countLimitText;
	}

	private JTextField getCountOffsetText() {
		if (countOffsetText == null) {
			countOffsetText = new JFormattedTextField();
		}
		return countOffsetText;
	}

	private JTextField getReceivedAfterText() {
		if (receivedAfterText == null) {
			receivedAfterText = new JFormattedTextField(new EmptyDateFormatter());
		}
		return receivedAfterText;
	}

	private JComboBox getOrderDirectionCombo() {
		if (orderDirectionCombo == null) {
			orderDirectionCombo = new JComboBox();
		}
		return orderDirectionCombo;
	}

	private JComboBox getOrderPropertyCombo() {
		if (orderPropertyCombo == null) {
			orderPropertyCombo = new JComboBox();
		}
		return orderPropertyCombo;
	}

	private JScrollPane getTableScrollPane() {
		if (tableScrollPane == null) {
			tableScrollPane = new JScrollPane();
			tableScrollPane.setViewportView(getTable());
		}
		return tableScrollPane;
	}

	private JTable getTable() {
		if (table == null) {
			table = new JTable();
			table.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			table.setRowSelectionAllowed(true);
			table.setColumnSelectionAllowed(false);
			table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			table.setDefaultRenderer(Date.class, new DateCellRenderer());
		}
		return table;
	}

	private PropertyChangeListener enableBrowseButton = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			if (e.getPropertyName() == FaxListViewModel.PROPERTY_BROWSE_URL_IS_ALLOWED) {
				boolean enableButton = (Boolean) e.getNewValue();
				getBrowseUrlButton().setEnabled(enableButton);
				getBrowseUrlButton().setToolTipText(enableButton ? null : "do a getFaxFileUrl");
			}
		}
	};

	private void faxFileUrlTextFocusFocusGained(FocusEvent event) {
		faxFileUrlText.selectAll();
	}

	private void faxFileTextFocusFocusGained(FocusEvent event) {
		faxFileText.selectAll();
	}

	public void setViewModel(FaxListViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final FaxListViewModel viewModel) {
		getFaxListButton.addActionListener(viewModel.getGetFaxListActionListener());
		deleteFaxListEntryButton.addActionListener(viewModel.getDeleteFaxListEntryActionListener());
		deleteFaxListEntriesButton.addActionListener(viewModel.getDeleteFaxListEntriesActionListener());
		moveFaxListEntryButton.addActionListener(viewModel.getMoveFaxListEntryActionListener());
		moveFaxListEntriesButton.addActionListener(viewModel.getMoveFaxListEntriesActionListener());
		getFaxFileUrlButton.addActionListener(viewModel.getGetFaxFileUrlActionListener());
		getFaxFileButton.addActionListener(viewModel.getGetFaxFileActionListener());
		openFileButton.addActionListener(viewModel.getOpenFaxFileActionListener());
		browseUrlButton.addActionListener(viewModel.getBrowseFaxFileUrlActionListener());

		viewModel.addPropertyChangeListener(enableBrowseButton);

		BeanAdapter<FaxListViewModel> beanAdapter = new BeanAdapter<FaxListViewModel>(viewModel, true);

		Bindings.bind(
			orderDirectionCombo,
			new SelectionInList<OrderDirection>(viewModel.getOrderDirectionValues(),
				beanAdapter.getValueModel(FaxListViewModel.PROPERTY_ORDER_DIRECTION)));
		Bindings.bind(
			orderPropertyCombo,
			new SelectionInList<FaxListEntryProperties>(viewModel.getOrderPropertyValues(),
				beanAdapter.getValueModel(FaxListViewModel.PROPERTY_ORDER_PROPERTY)));
		Bindings.bind(
			folderRestrictionCombo,
			new SelectionInList<FaxListEntryFolder>(viewModel.getFolderRestrictionValues(),
				beanAdapter.getValueModel(FaxListViewModel.PROPERTY_FOLDER_RESTRICTION)));
		Bindings.bind(
			destinationFolderCombo,
			new SelectionInList<FaxListEntryFolder>(viewModel.getDestinationFolderValues(),
				beanAdapter.getValueModel(FaxListViewModel.PROPERTY_DESTINATION_FOLDER)));
		Bindings.bind(
			groupRestrictionCombo,
			new SelectionInList<Object>(
				beanAdapter.getValueModel(FaxListViewModel.PROPERTY_GROUP_RESTRICTION_VALUES),
				beanAdapter.getValueModel(FaxListViewModel.PROPERTY_GROUP_RESTRICTION)));

		Bindings.bind(receivedAfterText, beanAdapter.getValueModel(FaxListViewModel.PROPERTY_RECEIVED_AFTER));
		Bindings.bind(receivedBeforeText, beanAdapter.getValueModel(FaxListViewModel.PROPERTY_RECEIVED_BEFORE));
		Bindings.bind(countLimitText, beanAdapter.getValueModel(FaxListViewModel.PROPERTY_COUNT_LIMIT));
		Bindings.bind(countOffsetText, beanAdapter.getValueModel(FaxListViewModel.PROPERTY_COUNT_OFFSET));
		Bindings.bind(faxFileUrlText, beanAdapter.getValueModel(FaxListViewModel.PROPERTY_FAX_FILE_URL));
		Bindings.bind(faxFileText, beanAdapter.getValueModel(FaxListViewModel.PROPERTY_FAX_FILE));

		table.setModel(viewModel.getFaxListTableModel());
		table.setSelectionModel(new MultiListSelectionAdapter(viewModel.getSelectedRows()));
	}

}
