/*
 * $Id: CallListPanel.java 499 2011-02-22 16:57:28Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2011-02-22 17:57:28 +0100 (Di, 22 Feb 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 499 $
 */
package de.starface.examples.uci30.swing.view;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.viewmodel.PhoneViewModel;
import de.starface.examples.uci30.swing.viewmodel.PhoneViewModel.PhoneIdViewModel;
import de.starface.examples.uci30.swing.viewmodel.PhoneViewModel.PhoneNumberViewModel;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.*;

/**
 * This panel provides a GUI for the UCI communication call list functionality
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class PhonePanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private int eventId = 0;

	private JList phoneIdsList;
	private JScrollPane phoneIdsScrollPane;
	private JButton getPhoneIdsButton;

	private JButton setPrimaryPhoneIdButton;

	private JButton getPrimaryPhoneIdButton;

	private JTextField primaryPhoneText;

	private JLabel primaryPhoneLabel;

	private JButton getAvailableDisplayNumbersButton;

	private JList availableDisplayNumbersList;

	private JScrollPane availableDisplayNumbersScrollPane;

	private JButton setDisplayNumberIdButton;

	private JButton registerWinSoftPhoneButton;
	private JButton registerMacSoftPhoneButton;

	private JButton isWinSoftPhoneInUseButton;
	private JButton isMacSoftPhoneInUseButton;

	private JButton getDisplayNumberIdButton;

	private JFormattedTextField displayNumberIdText;

	private JLabel displayNumberIdLabel;

	private JCheckBox callWaitingIndicationCheckBox;

	private JButton getCallWaitingIndicationButton;

	private JButton setCallWaitingIndicationButton;

	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public PhonePanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(570, 450));
		setPreferredSize(new Dimension(570, 450));
		setLayout(new GroupLayout());
		add(getCallWaitingIndicationCheckBox(), new Constraints(new Leading(12, 12, 12), new Leading(413, 10, 10)));
		add(getGetCallWaitingIndicationButton(), new Constraints(new Leading(172, 10, 10), new Leading(408, 12, 12)));
		add(getSetCallWaitingIndicationButton(), new Constraints(new Leading(354, 10, 10), new Leading(408, 12, 12)));
		add(getAvailableDisplayNumbersScrollPane(), new Constraints(new Leading(10, 276, 165, 165), new Leading(266, 137, 10, 10)));
		add(getDisplayNumberIdText(), new Constraints(new Bilateral(292, -10, 12), new Leading(295, 10, 10)));
		add(getGetDisplayNumberIdButton(), new Constraints(new Leading(292, 278, 12, 12), new Leading(332, 10, 10)));
		add(getSetDisplayNumberIdButton(), new Constraints(new Leading(292, 272, 12, 12), new Leading(368, 12, 12)));
		add(getGetAvailableDisplayNumbersButton(), new Constraints(new Leading(10, 12, 12), new Leading(231, 10, 10)));
		add(getDisplayNumberIdLabel(), new Constraints(new Leading(292, 12, 12), new Leading(272, 10, 10)));
		add(getPhoneIdsScrollPane(), new Constraints(new Leading(11, 279, 10, 10), new Leading(81, 134, 10, 10)));
		add(getSetPrimaryPhoneIdButton(), new Constraints(new Leading(289, 284, 12, 12), new Leading(186, 12, 12)));
		add(getGetPrimaryPhoneIdButton(), new Constraints(new Leading(289, 287, 12, 12), new Leading(147, 12, 12)));
		add(getPrimaryPhoneText(), new Constraints(new Leading(292, 272, 12, 12), new Leading(108, 12, 12)));
		add(getPrimaryPhoneLabel(), new Constraints(new Leading(292, 12, 12), new Leading(87, 12, 12)));
		add(getPhoneIdsButton(), new Constraints(new Leading(12, 12, 12), new Leading(52, 10, 10)));
		add(getIsWinSoftPhoneInUseButton(), new Constraints(new Leading(389, 12, 12), new Leading(45, 10, 10)));
		add(getIsMacSoftPhoneInUseButton(), new Constraints(new Leading(388, 12, 12), new Leading(15, 10, 10)));
		add(getRegisterWinSoftPhoneButton(), new Constraints(new Leading(180, 12, 12), new Leading(45, 12, 12)));
		add(getRegisterMacSoftPhoneButton(), new Constraints(new Leading(180, 206, 12, 12), new Leading(15, 12, 12)));
		setSize(570, 450);
	}

	private JButton getSetCallWaitingIndicationButton() {
		if (setCallWaitingIndicationButton == null) {
			setCallWaitingIndicationButton = new JButton();
			setCallWaitingIndicationButton.setText("setCallWaitingIndication");
		}
		return setCallWaitingIndicationButton;
	}

	private JButton getGetCallWaitingIndicationButton() {
		if (getCallWaitingIndicationButton == null) {
			getCallWaitingIndicationButton = new JButton();
			getCallWaitingIndicationButton.setText("getCallWaitingIndication");
		}
		return getCallWaitingIndicationButton;
	}

	private JCheckBox getCallWaitingIndicationCheckBox() {
		if (callWaitingIndicationCheckBox == null) {
			callWaitingIndicationCheckBox = new JCheckBox();
			callWaitingIndicationCheckBox.setSelected(true);
			callWaitingIndicationCheckBox.setText("callWaitingIndication");
		}
		return callWaitingIndicationCheckBox;
	}

	private JLabel getDisplayNumberIdLabel() {
		if (displayNumberIdLabel == null) {
			displayNumberIdLabel = new JLabel();
			displayNumberIdLabel.setText("displayNumberId");
		}
		return displayNumberIdLabel;
	}

	private JFormattedTextField getDisplayNumberIdText() {
		if (displayNumberIdText == null) {
			displayNumberIdText = new JFormattedTextField();
			displayNumberIdText.setEditable(false);
		}
		return displayNumberIdText;
	}

	private JButton getGetDisplayNumberIdButton() {
		if (getDisplayNumberIdButton == null) {
			getDisplayNumberIdButton = new JButton();
			getDisplayNumberIdButton.setText("getDisplayNumberId");
		}
		return getDisplayNumberIdButton;
	}

	private JButton getSetDisplayNumberIdButton() {
		if (setDisplayNumberIdButton == null) {
			setDisplayNumberIdButton = new JButton();
			setDisplayNumberIdButton.setText("setDisplayNumberId");
		}
		return setDisplayNumberIdButton;
	}

	private JButton getRegisterWinSoftPhoneButton() {
		if (registerWinSoftPhoneButton == null) {
			registerWinSoftPhoneButton = new JButton();
			registerWinSoftPhoneButton.setText("registerWinSoftPhone");
		}
		return registerWinSoftPhoneButton;
	}

	private JButton getRegisterMacSoftPhoneButton() {
		if (registerMacSoftPhoneButton == null) {
			registerMacSoftPhoneButton = new JButton();
			registerMacSoftPhoneButton.setText("registerMacSoftPhone");
		}
		return registerMacSoftPhoneButton;
	}

	private JButton getIsMacSoftPhoneInUseButton() {
		if (isMacSoftPhoneInUseButton == null) {
			isMacSoftPhoneInUseButton = new JButton();
			isMacSoftPhoneInUseButton.setText("isMacSoftPhoneInUse");
		}
		return isMacSoftPhoneInUseButton;
	}

	private JButton getIsWinSoftPhoneInUseButton() {
		if (isWinSoftPhoneInUseButton == null) {
			isWinSoftPhoneInUseButton = new JButton();
			isWinSoftPhoneInUseButton.setText("isWinSoftPhoneInUse");
		}
		return isWinSoftPhoneInUseButton;
	}

	private JScrollPane getAvailableDisplayNumbersScrollPane() {
		if (availableDisplayNumbersScrollPane == null) {
			availableDisplayNumbersScrollPane = new JScrollPane();
			availableDisplayNumbersScrollPane.setViewportView(getAvailableDisplayNumbersList());
		}
		return availableDisplayNumbersScrollPane;
	}

	private JList getAvailableDisplayNumbersList() {
		if (availableDisplayNumbersList == null) {
			availableDisplayNumbersList = new JList();
		}
		return availableDisplayNumbersList;
	}

	private JButton getGetAvailableDisplayNumbersButton() {
		if (getAvailableDisplayNumbersButton == null) {
			getAvailableDisplayNumbersButton = new JButton();
			getAvailableDisplayNumbersButton.setText("getAvailableDisplayNumbers");
		}
		return getAvailableDisplayNumbersButton;
	}

	private JLabel getPrimaryPhoneLabel() {
		if (primaryPhoneLabel == null) {
			primaryPhoneLabel = new JLabel();
			primaryPhoneLabel.setText("primaryPhoneId");
		}
		return primaryPhoneLabel;
	}

	private JTextField getPrimaryPhoneText() {
		if (primaryPhoneText == null) {
			primaryPhoneText = new JTextField();
			primaryPhoneText.setEditable(false);
		}
		return primaryPhoneText;
	}

	private JButton getGetPrimaryPhoneIdButton() {
		if (getPrimaryPhoneIdButton == null) {
			getPrimaryPhoneIdButton = new JButton();
			getPrimaryPhoneIdButton.setText("getPrimaryPhoneId");
		}
		return getPrimaryPhoneIdButton;
	}

	private JButton getSetPrimaryPhoneIdButton() {
		if (setPrimaryPhoneIdButton == null) {
			setPrimaryPhoneIdButton = new JButton();
			setPrimaryPhoneIdButton.setText("setPrimaryPhoneId");
		}
		return setPrimaryPhoneIdButton;
	}

	private JButton getPhoneIdsButton() {
		if (getPhoneIdsButton == null) {
			getPhoneIdsButton = new JButton();
			getPhoneIdsButton.setText("getPhoneIds");
		}
		return getPhoneIdsButton;
	}

	private JScrollPane getPhoneIdsScrollPane() {
		if (phoneIdsScrollPane == null) {
			phoneIdsScrollPane = new JScrollPane();
			phoneIdsScrollPane.setPreferredSize(new Dimension(25, 25));
			phoneIdsScrollPane.setOpaque(true);
			phoneIdsScrollPane.setViewportView(getPhoneIdsList());
		}
		return phoneIdsScrollPane;
	}

	private JList getPhoneIdsList() {
		if (phoneIdsList == null) {
			phoneIdsList = new JList();
			phoneIdsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			DefaultListModel listModel = new DefaultListModel();
			phoneIdsList.setModel(listModel);
		}
		return phoneIdsList;
	}

	public void setViewModel(PhoneViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final PhoneViewModel viewModel) {
		getPhoneIdsButton.addActionListener(viewModel.getGetPhoneIdsActionListener());
		getPrimaryPhoneIdButton.addActionListener(viewModel.getGetPrimaryPhoneIdActionListener());
		setPrimaryPhoneIdButton.addActionListener(viewModel.getSetPrimaryPhoneIdPhoneIdActionListener());
		getCallWaitingIndicationButton.addActionListener(viewModel.getGetCallWaitingIndicationActionListener());
		setCallWaitingIndicationButton.addActionListener(viewModel.getSetCallWaitingIndicationActionListener());
		getAvailableDisplayNumbersButton.addActionListener(viewModel.getGetGetAvailableDisplayNumbersActionListener());
		getDisplayNumberIdButton.addActionListener(viewModel.getGetDisplayNumberIdActionListener());
		setDisplayNumberIdButton.addActionListener(viewModel.getSetDisplayNumberIdActionListener());

		registerWinSoftPhoneButton.addActionListener(viewModel.getRegisterWinSoftPhoneActionListener());
		registerMacSoftPhoneButton.addActionListener(viewModel.getRegisterMacSoftPhoneActionListener());
		isWinSoftPhoneInUseButton.addActionListener(viewModel.getIsWinSoftPhoneInUse());
		isMacSoftPhoneInUseButton.addActionListener(viewModel.getIsWacSoftPhoneInUse());

		BeanAdapter<PhoneViewModel> beanAdapter = new BeanAdapter<PhoneViewModel>(viewModel, true);

		Bindings.bind(primaryPhoneText, beanAdapter.getValueModel(PhoneViewModel.PROPERTY_PRIMARY_PHONE_ID));
		Bindings.bind(displayNumberIdText, beanAdapter.getValueModel(PhoneViewModel.PROPERTY_DISPLAY_NUMBER));

		Bindings.bind(callWaitingIndicationCheckBox,
			beanAdapter.getValueModel(PhoneViewModel.PROPERTY_CALL_WAITING_INDICATION));

		Bindings.bind(phoneIdsList,
			new SelectionInList<PhoneIdViewModel>(beanAdapter.getValueModel(PhoneViewModel.PROPERTY_PHONE_IDS),
				beanAdapter.getValueModel(PhoneViewModel.PROPERTY_SELECTED_PHONE_ID)));

		Bindings.bind(
			availableDisplayNumbersList,
			new SelectionInList<PhoneNumberViewModel>(
				beanAdapter.getValueModel(PhoneViewModel.PROPERTY_DISPLAY_NUMBERS),
				beanAdapter.getValueModel(PhoneViewModel.PROPERTY_SELECTED_DISPLAY_NUMBER)));

		phoneIdsList.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					viewModel.getTransferSelectedPhoneIdActionListener().actionPerformed(
						new ActionEvent(this, eventId++, null));
				}
			}
		});

		availableDisplayNumbersList.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					viewModel.getTransferSelectedDisplayNumberActionListener().actionPerformed(
						new ActionEvent(this, eventId++, null));
				}
			}
		});

	}

}
