/*
 * $Id$
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.examples.uci30.swing.view;

import de.starface.examples.uci30.swing.viewmodel.RedirectSettingsTableModel;
import de.starface.integration.uci.java.v30.types.Mailbox;
import de.starface.integration.uci.java.v30.types.RedirectSetting;
import de.starface.integration.uci.java.v30.values.RedirectSettingDestinationType;

import org.apache.commons.lang3.StringUtils;

import java.awt.*;
import java.util.List;

import javax.swing.*;
import javax.swing.table.TableCellEditor;

/**
 * A table cell editor for the the destination property of redirect settings
 *
 * @author tilmann
 */
public class RedirectSettingsDestinationCellEditor extends AbstractCellEditor implements TableCellEditor {
	private static final long serialVersionUID = 1L;

	private JTextField textField = new JTextField();

	private JComboBox comboBox = new JComboBox();

	private JLabel label = new JLabel();

	private RedirectSettingDestinationType destinationType;

	private String result;

	public RedirectSettingsDestinationCellEditor() {
		comboBox.putClientProperty("JComboBox.isTableCellEditor", Boolean.TRUE);
		comboBox.setRenderer(new MailboxesListCellRenderer());
		textField.setName("Table.editor");
	}

	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row,
												 int column) {
		result = (String) value;
		final RedirectSetting redirectSetting = ((RedirectSettingsTableModel) table.getModel()).getRedirectSetting(row);
		final String destinationTypeName = (String) redirectSetting.getDestinationType().name();
		if (!StringUtils.isEmpty(destinationTypeName)) {
			destinationType = RedirectSettingDestinationType.valueOf(destinationTypeName);

			if (destinationType == RedirectSettingDestinationType.VOICEMAIL) {
				List<Mailbox> mailboxes = (List<Mailbox>) redirectSetting.getMailboxes();
				comboBox.setModel(new DefaultComboBoxModel(mailboxes.toArray()));
				comboBox.setSelectedItem(computeSelectedItem(result, mailboxes));
				return comboBox;
			} else {
				textField.setText(result);
				return textField;
			}
		} else {
			label.setText(result);
			return label;
		}
	}

	public Object getCellEditorValue() {
		return result;
	}

	@Override
	public boolean stopCellEditing() {
		if (destinationType == null) {
			return super.stopCellEditing();
		}
		if (destinationType == RedirectSettingDestinationType.VOICEMAIL) {
			Mailbox mailbox = (Mailbox) comboBox.getSelectedItem();
			if (mailbox != null) {
				result = mailbox.getId();
			} else {
				result = "";
			}
		} else {
			result = textField.getText();
		}
		destinationType = null;
		return super.stopCellEditing();
	}

	private static Mailbox computeSelectedItem(String value, List<Mailbox> mailboxes) {
		if (value == null) {
			return null;
		}
		for (Mailbox mailbox : mailboxes) {
			if (value.equals(mailbox.getId())) {
				return mailbox;
			}
		}
		return null;
	}
}
