/*
 * $Id$
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.examples.uci30.swing.view;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.model.Config;
import de.starface.examples.uci30.swing.view.binding.MultiListSelectionAdapter;
import de.starface.examples.uci30.swing.viewmodel.VoicemailListViewModel;
import de.starface.integration.uci.java.v30.types.Mailbox;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryFolder;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryProperties;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import java.awt.*;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.Date;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;

/**
 * This panel provides a GUI for the UCI communication voicemail list functionality
 *
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class VoicemailListPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JTable table;
	private JScrollPane tableScrollPane;
	private JComboBox orderPropertyCombo;
	private JComboBox orderDirectionCombo;
	private JFormattedTextField startedAfterText;
	private JFormattedTextField countOffsetText;
	private JFormattedTextField countLimitText;
	private JLabel startedBeforeLabel;
	private JLabel startedAfterLabel;
	private JFormattedTextField startedBeforeText;
	private JLabel folderRestrictionLabel;
	private JComboBox folderRestrictionCombo;
	private JLabel orderPropertyLabel;
	private JLabel orderDirectionLabel;
	private JLabel countOffsetLabel;
	private JLabel countLimitLabel;
	private JButton getVoicemailListButton;
	private JButton deleteVoicemailListEntryButton;
	private JButton getVoicemailFileUrlButton;
	private JButton moveVoicemailListEntryButton;
	private JLabel destinationFolderLabel;
	private JComboBox destinationFolderCombo;
	private JTextField voicemailFileUrlText;
	private JLabel mailboxRestrictionLabel;
	private JComboBox mailboxRestrictionCombo;
	private JButton getMailboxesButton;
	private JComboBox mailboxCombo;
	private JLabel mailbox;
	private JLabel phoneNumberLabel;
	private JTextField phoneNumberText;
	private JButton callMailboxWithNumberButton;
	private JButton callVoicemailWithPhoneButton;
	private JButton callVoicemailWithNumberButton;
	private JButton callMailboxWithPhoneButton;
	private JSeparator jSeparator0;
	private JButton deleteVoicemailListEntriesButton;
	private JButton moveVoicemailListEntriesButton;

	private JButton getVoicemailFileButton;

	private JTextField voicemailFileText;

	private JButton browseVoicemailFileUrlButton;

	private JButton openVoicemailFileButton;

	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";

	public VoicemailListPanel() {
		initComponents();
	}

	private void initComponents() {
		setMinimumSize(new Dimension(570, 450));
		setPreferredSize(new Dimension(570, 450));
		setLayout(new GroupLayout());
		add(getOrderPropertyLabel(), new Constraints(new Leading(239, 6, 6), new Leading(12, 6, 6)));
		add(getOrderDirectionLabel(), new Constraints(new Leading(238, 6, 6), new Leading(46, 6, 6)));
		add(getOrderPropertyCombo(), new Constraints(new Leading(322, 123, 10, 10), new Leading(7, 6, 6)));
		add(getOrderDirectionCombo(), new Constraints(new Leading(322, 122, 6, 6), new Leading(40, 6, 6)));
		add(getCountOffsetLabel(), new Constraints(new Leading(457, 6, 6), new Leading(12, 6, 6)));
		add(getCountLimitLabel(), new Constraints(new Leading(456, 6, 6), new Leading(46, 6, 6)));
		add(getCountOffsetText(), new Constraints(new Leading(525, 43, 10, 10), new Leading(6, 6, 6)));
		add(getCountLimitText(), new Constraints(new Leading(525, 42, 6, 6), new Leading(40, 6, 6)));
		add(getFolderRestrictionLabel(), new Constraints(new Leading(6, 6, 6), new Leading(80, 6, 6)));
		add(getMailboxRestrictionLabel(), new Constraints(new Leading(238, 6, 6), new Leading(80, 6, 6)));
		add(getMailboxRestrictionCombo(), new Constraints(new Leading(346, 220, 6, 6), new Leading(74, 6, 6)));
		add(getDestinationFolderLabel(), new Constraints(new Leading(6, 6, 6), new Leading(153, 6, 6)));
		add(getFolderRestrictionCombo(), new Constraints(new Leading(108, 118, 6, 6), new Leading(74, 6, 6)));
		add(getStartedAfterText(), new Constraints(new Leading(72, 154, 6, 6), new Leading(6, 6, 6)));
		add(getStartedBeforeText(), new Constraints(new Leading(72, 155, 6, 6), new Leading(39, 6, 6)));
		add(getStartedBeforeLabel(), new Constraints(new Leading(6, 6, 6), new Leading(46, 6, 6)));
		add(getStartedAfterLabel(), new Constraints(new Leading(6, 6, 6), new Leading(12, 6, 6)));
		add(getJSeparator0(), new Constraints(new Leading(6, 561, 6, 6), new Leading(140, 10, 6, 6)));
		add(getMoveVoicemailListEntriesButton(), new Constraints(new Leading(395, 6, 6), new Leading(147, 6, 6)));
		add(getMoveVoicemailListEntryButton(),
			new Constraints(new Leading(208, 181, 6, 6), new Leading(147, 6, 6)));
		add(getDestinationFolderCombo(), new Constraints(new Leading(108, 94, 6, 6), new Leading(148, 10, 10)));
		add(getGetVoicemailListButton(), new Constraints(new Leading(294, 272, 10, 10), new Leading(106, 6, 6)));
		add(getGetMailboxesButton(), new Constraints(new Leading(6, 284, 6, 6), new Leading(106, 6, 6)));
		add(getDeleteVoicemailListEntriesButton(), new Constraints(new Leading(303, 264, 6, 6), new Leading(337,
			6, 6)));
		add(getTableScrollPane(), new Constraints(new Bilateral(6, 10, 558), new Leading(371, 125, 6, 6)));
		add(getDeleteVoicemailListEntryButton(),
			new Constraints(new Leading(6, 284, 6, 6), new Leading(337, 6, 6)));
		add(getCallVoicemailWithPhoneButton(), new Constraints(new Leading(6, 284, 6, 6), new Leading(303, 6, 6)));
		add(getCallMailboxWithPhoneButton(), new Constraints(new Leading(303, 264, 6, 6), new Leading(303, 6, 6)));
		add(getCallVoicemailWithNumberButton(), new Constraints(new Leading(6, 284, 6, 6), new Leading(269, 6, 6)));
		add(getCallMailboxWithNumberButton(), new Constraints(new Leading(303, 264, 6, 6), new Leading(269, 6, 6)));
		add(getGetVoicemailFileUrlButton(), new Constraints(new Leading(6, 6, 6), new Leading(180, 6, 6)));
		add(getGetVoicemailFileButton(), new Constraints(new Leading(6, 135, 6, 6), new Leading(209, 10, 10)));
		add(getMailboxCombo(), new Constraints(new Leading(356, 212, 6, 6), new Leading(241, 6, 6)));
		add(getMailbox(), new Constraints(new Leading(306, 6, 6), new Leading(245, 6, 6)));
		add(getPhoneNumberLabel(), new Constraints(new Leading(6, 6, 6), new Leading(246, 6, 6)));
		add(getPhoneNumberText(), new Constraints(new Leading(88, 202, 6, 6), new Leading(240, 6, 6)));
		add(getBrowseVoicemailFileUrlButton(), new Constraints(new Leading(497, 6, 6), new Leading(180, 6, 6)));
		add(getVoicemailFileUrlText(), new Constraints(new Leading(147, 344, 6, 6), new Leading(180, 6, 6)));
		add(getVoicemailFileText(), new Constraints(new Leading(147, 344, 6, 6), new Leading(209, 6, 6)));
		add(getOpenVoicemailFileButton(), new Constraints(new Leading(496, 70, 6, 6), new Leading(209, 6, 6)));
		setSize(570, 450);
	}

	private JButton getOpenVoicemailFileButton() {
		if (openVoicemailFileButton == null) {
			openVoicemailFileButton = new JButton();
			openVoicemailFileButton.setText("open");
		}
		return openVoicemailFileButton;
	}

	private JTextField getVoicemailFileText() {
		if (voicemailFileText == null) {
			voicemailFileText = new JTextField();
			voicemailFileText.addFocusListener(new FocusAdapter() {

				public void focusGained(FocusEvent event) {
					voicemailFileTextFocusFocusGained(event);
				}
			});
		}
		return voicemailFileText;
	}

	private JButton getGetVoicemailFileButton() {
		if (getVoicemailFileButton == null) {
			getVoicemailFileButton = new JButton();
			getVoicemailFileButton.setText("getVoicemailFile");
		}
		return getVoicemailFileButton;
	}

	private JButton getBrowseVoicemailFileUrlButton() {
		if (browseVoicemailFileUrlButton == null) {
			browseVoicemailFileUrlButton = new JButton();
			browseVoicemailFileUrlButton.setText("browse");
		}
		return browseVoicemailFileUrlButton;
	}

	private JButton getMoveVoicemailListEntriesButton() {
		if (moveVoicemailListEntriesButton == null) {
			moveVoicemailListEntriesButton = new JButton();
			moveVoicemailListEntriesButton.setText("moveVoicemailListEntries");
		}
		return moveVoicemailListEntriesButton;
	}

	private JButton getDeleteVoicemailListEntriesButton() {
		if (deleteVoicemailListEntriesButton == null) {
			deleteVoicemailListEntriesButton = new JButton();
			deleteVoicemailListEntriesButton.setText("deleteVoicemailListEntries");
		}
		return deleteVoicemailListEntriesButton;
	}

	private JSeparator getJSeparator0() {
		if (jSeparator0 == null) {
			jSeparator0 = new JSeparator();
		}
		return jSeparator0;
	}

	private JButton getCallMailboxWithPhoneButton() {
		if (callMailboxWithPhoneButton == null) {
			callMailboxWithPhoneButton = new JButton();
			callMailboxWithPhoneButton.setText("callMailboxWithPhone");
		}
		return callMailboxWithPhoneButton;
	}

	private JButton getCallVoicemailWithNumberButton() {
		if (callVoicemailWithNumberButton == null) {
			callVoicemailWithNumberButton = new JButton();
			callVoicemailWithNumberButton.setText("callVoicemailWithNumber");
		}
		return callVoicemailWithNumberButton;
	}

	private JButton getCallVoicemailWithPhoneButton() {
		if (callVoicemailWithPhoneButton == null) {
			callVoicemailWithPhoneButton = new JButton();
			callVoicemailWithPhoneButton.setText("callVoicemailWithPhone");
		}
		return callVoicemailWithPhoneButton;
	}

	private JButton getCallMailboxWithNumberButton() {
		if (callMailboxWithNumberButton == null) {
			callMailboxWithNumberButton = new JButton();
			callMailboxWithNumberButton.setText("callMailboxWithNumber");
		}
		return callMailboxWithNumberButton;
	}

	private JTextField getPhoneNumberText() {
		if (phoneNumberText == null) {
			phoneNumberText = new JTextField();
			phoneNumberText.setText(Config.getValue("phoneNumber"));
			phoneNumberText.addFocusListener(new FocusAdapter() {

				public void focusGained(FocusEvent event) {
					phoneNumberTextFocusFocusGained(event);
				}
			});
		}
		return phoneNumberText;
	}

	private JLabel getPhoneNumberLabel() {
		if (phoneNumberLabel == null) {
			phoneNumberLabel = new JLabel();
			phoneNumberLabel.setText("phoneNumber");
		}
		return phoneNumberLabel;
	}

	private JLabel getMailbox() {
		if (mailbox == null) {
			mailbox = new JLabel();
			mailbox.setText("mailbox");
		}
		return mailbox;
	}

	private JComboBox getMailboxCombo() {
		if (mailboxCombo == null) {
			mailboxCombo = new JComboBox();
			mailboxCombo.setModel(new DefaultComboBoxModel(new Object[0]));
			mailboxCombo.setRenderer(new MailboxesListCellRenderer());
		}
		return mailboxCombo;
	}

	private JButton getGetMailboxesButton() {
		if (getMailboxesButton == null) {
			getMailboxesButton = new JButton();
			getMailboxesButton.setText("getMailboxes");
		}
		return getMailboxesButton;
	}

	private JComboBox getMailboxRestrictionCombo() {
		if (mailboxRestrictionCombo == null) {
			mailboxRestrictionCombo = new JComboBox();
			mailboxRestrictionCombo.setModel(new DefaultComboBoxModel(new Object[]{null}));
			mailboxRestrictionCombo.setRenderer(new MailboxesListCellRenderer());
		}
		return mailboxRestrictionCombo;
	}

	private JLabel getMailboxRestrictionLabel() {
		if (mailboxRestrictionLabel == null) {
			mailboxRestrictionLabel = new JLabel();
			mailboxRestrictionLabel.setText("mailboxRestriction");
		}
		return mailboxRestrictionLabel;
	}

	private JTextField getVoicemailFileUrlText() {
		if (voicemailFileUrlText == null) {
			voicemailFileUrlText = new JTextField();
			voicemailFileUrlText.setEditable(false);
			voicemailFileUrlText.addFocusListener(new FocusAdapter() {

				public void focusGained(FocusEvent event) {
					voicemailFileUrlTextFocusFocusGained(event);
				}
			});
		}
		return voicemailFileUrlText;
	}

	private JComboBox getDestinationFolderCombo() {
		if (destinationFolderCombo == null) {
			destinationFolderCombo = new JComboBox();
			destinationFolderCombo.setModel(new DefaultComboBoxModel(VoicemailListEntryFolder.values()));
		}
		return destinationFolderCombo;
	}

	private JLabel getDestinationFolderLabel() {
		if (destinationFolderLabel == null) {
			destinationFolderLabel = new JLabel();
			destinationFolderLabel.setText("destinationFolder");
		}
		return destinationFolderLabel;
	}

	private JButton getMoveVoicemailListEntryButton() {
		if (moveVoicemailListEntryButton == null) {
			moveVoicemailListEntryButton = new JButton();
			moveVoicemailListEntryButton.setText("moveVoicemailListEntry");
		}
		return moveVoicemailListEntryButton;
	}

	private JButton getGetVoicemailFileUrlButton() {
		if (getVoicemailFileUrlButton == null) {
			getVoicemailFileUrlButton = new JButton();
			getVoicemailFileUrlButton.setText("getVoicemailFileUrl");
		}
		return getVoicemailFileUrlButton;
	}

	private JButton getDeleteVoicemailListEntryButton() {
		if (deleteVoicemailListEntryButton == null) {
			deleteVoicemailListEntryButton = new JButton();
			deleteVoicemailListEntryButton.setText("deleteVoicemailListEntry");
		}
		return deleteVoicemailListEntryButton;
	}

	private JButton getGetVoicemailListButton() {
		if (getVoicemailListButton == null) {
			getVoicemailListButton = new JButton();
			getVoicemailListButton.setText("getVoicemailList");
		}
		return getVoicemailListButton;
	}

	private JLabel getCountLimitLabel() {
		if (countLimitLabel == null) {
			countLimitLabel = new JLabel();
			countLimitLabel.setText("countLimit");
		}
		return countLimitLabel;
	}

	private JLabel getCountOffsetLabel() {
		if (countOffsetLabel == null) {
			countOffsetLabel = new JLabel();
			countOffsetLabel.setText("countOffset");
		}
		return countOffsetLabel;
	}

	private JLabel getOrderDirectionLabel() {
		if (orderDirectionLabel == null) {
			orderDirectionLabel = new JLabel();
			orderDirectionLabel.setText("orderDirection");
		}
		return orderDirectionLabel;
	}

	private JLabel getOrderPropertyLabel() {
		if (orderPropertyLabel == null) {
			orderPropertyLabel = new JLabel();
			orderPropertyLabel.setText("orderProperty");
		}
		return orderPropertyLabel;
	}

	private JComboBox getFolderRestrictionCombo() {
		if (folderRestrictionCombo == null) {
			folderRestrictionCombo = new JComboBox();
		}
		return folderRestrictionCombo;
	}

	private JLabel getFolderRestrictionLabel() {
		if (folderRestrictionLabel == null) {
			folderRestrictionLabel = new JLabel();
			folderRestrictionLabel.setText("folderRestriction");
		}
		return folderRestrictionLabel;
	}

	private JTextField getStartedBeforeText() {
		if (startedBeforeText == null) {
			startedBeforeText = new JFormattedTextField(new EmptyDateFormatter());
		}
		return startedBeforeText;
	}

	private JLabel getStartedAfterLabel() {
		if (startedAfterLabel == null) {
			startedAfterLabel = new JLabel();
			startedAfterLabel.setText("startAfter");
		}
		return startedAfterLabel;
	}

	private JLabel getStartedBeforeLabel() {
		if (startedBeforeLabel == null) {
			startedBeforeLabel = new JLabel();
			startedBeforeLabel.setText("startBefore");
		}
		return startedBeforeLabel;
	}

	private JTextField getCountLimitText() {
		if (countLimitText == null) {
			countLimitText = new JFormattedTextField();
		}
		return countLimitText;
	}

	private JTextField getCountOffsetText() {
		if (countOffsetText == null) {
			countOffsetText = new JFormattedTextField();
		}
		return countOffsetText;
	}

	private JTextField getStartedAfterText() {
		if (startedAfterText == null) {
			startedAfterText = new JFormattedTextField(new EmptyDateFormatter());
		}
		return startedAfterText;
	}

	private JComboBox getOrderDirectionCombo() {
		if (orderDirectionCombo == null) {
			orderDirectionCombo = new JComboBox();
		}
		return orderDirectionCombo;
	}

	private JComboBox getOrderPropertyCombo() {
		if (orderPropertyCombo == null) {
			orderPropertyCombo = new JComboBox();
		}
		return orderPropertyCombo;
	}

	private JScrollPane getTableScrollPane() {
		if (tableScrollPane == null) {
			tableScrollPane = new JScrollPane();
			tableScrollPane.setViewportView(getTable());
		}
		return tableScrollPane;
	}

	private JTable getTable() {
		if (table == null) {
			table = new JTable();
			table.setModel(new DefaultTableModel(new Object[][]{}, new String[]{}) {
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[]{};

				public Class<?> getColumnClass(int columnIndex) {
					return types[columnIndex];
				}
			});
			table.setRowSelectionAllowed(true);
			table.setColumnSelectionAllowed(false);
			table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			table.setDefaultRenderer(Date.class, new DateCellRenderer());
		}
		return table;
	}

	private void phoneNumberTextFocusFocusGained(FocusEvent event) {
		phoneNumberText.selectAll();
	}

	private void voicemailFileUrlTextFocusFocusGained(FocusEvent event) {
		voicemailFileUrlText.selectAll();
	}

	private void voicemailFileTextFocusFocusGained(FocusEvent event) {
		voicemailFileText.selectAll();
	}

	public void setViewModel(VoicemailListViewModel viewModel) {
		initBindings(viewModel);
	}

	private void initBindings(final VoicemailListViewModel viewModel) {
		getVoicemailListButton.addActionListener(viewModel.getGetVoicemailListActionListener());
		deleteVoicemailListEntryButton.addActionListener(viewModel.getDeleteVoicemailListEntryActionListener());
		deleteVoicemailListEntriesButton.addActionListener(viewModel.getDeleteVoicemailListEntriesActionListener());
		moveVoicemailListEntryButton.addActionListener(viewModel.getMoveVoicemailListEntryActionListener());
		moveVoicemailListEntriesButton.addActionListener(viewModel.getMoveVoicemailListEntriesActionListener());
		getVoicemailFileUrlButton.addActionListener(viewModel.getGetVoicemailFileUrlActionListener());
		getVoicemailFileButton.addActionListener(viewModel.getGetVoicemailFileActionListener());
		browseVoicemailFileUrlButton.addActionListener(viewModel.getBrowseVoicemailFileUrlActionListener());
		openVoicemailFileButton.addActionListener(viewModel.getOpenVoicemailFileActionListener());
		getMailboxesButton.addActionListener(viewModel.getGetGetMailboxesActionListener());
		callMailboxWithNumberButton.addActionListener(viewModel.getCallMailboxWithNumberActionListener());
		callMailboxWithPhoneButton.addActionListener(viewModel.getCallMailboxWithPhoneActionListener());
		callVoicemailWithPhoneButton.addActionListener(viewModel.getCallVoicemailWithPhoneActionListener());
		callVoicemailWithNumberButton.addActionListener(viewModel.getCallVoicemailWithNumberActionListener());

		BeanAdapter<VoicemailListViewModel> beanAdapter = new BeanAdapter<VoicemailListViewModel>(viewModel, true);

		Bindings.bind(
			orderDirectionCombo,
			new SelectionInList<OrderDirection>(viewModel.getOrderDirectionValues(),
				beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_ORDER_DIRECTION)));
		Bindings.bind(
			orderPropertyCombo,
			new SelectionInList<VoicemailListEntryProperties>(viewModel.getOrderPropertyValues(),
				beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_ORDER_PROPERTY)));
		Bindings.bind(
			folderRestrictionCombo,
			new SelectionInList<VoicemailListEntryFolder>(viewModel.getFolderRestrictionValues(),
				beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_FOLDER_RESTRICTION)));
		Bindings.bind(
			mailboxRestrictionCombo,
			new SelectionInList<Mailbox>(
				beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_MAILBOX_RESTRICTION_VALUES),
				beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_MAILBOX_RESTRICTION)));

		Bindings.bind(
			mailboxCombo,
			new SelectionInList<Mailbox>(
				beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_MAILBOX_VALUES),
				beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_SELECTED_MAILBOX)));

		Bindings.bind(
			destinationFolderCombo,
			new SelectionInList<VoicemailListEntryFolder>(viewModel.getDestinationFolderValues(),
				beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_DESTINATION_FOLDER)));

		Bindings.bind(startedAfterText, beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_STARTED_AFTER));
		Bindings.bind(startedBeforeText, beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_STARTED_BEFORE));
		Bindings.bind(countLimitText, beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_COUNT_LIMIT));
		Bindings.bind(countOffsetText, beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_COUNT_OFFSET));
		Bindings.bind(voicemailFileUrlText,
			beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_VOICEMAIL_FILE_URL));
		Bindings.bind(voicemailFileText, beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_VOICEMAIL_FILE));
		Bindings.bind(phoneNumberText,
			beanAdapter.getValueModel(VoicemailListViewModel.PROPERTY_ORIGINATING_NUMBER));

		table.setModel(viewModel.getVoicemailListTableModel());
		table.setSelectionModel(new MultiListSelectionAdapter(viewModel.getSelectedRows()));
	}

}
