/*
 * $Id$
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.CallList;
import de.starface.integration.uci.java.v30.types.CallListEntry;
import de.starface.integration.uci.java.v30.values.CallListEntryProperties;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the call list
 *
 * @author tilmann
 */
public class CallListTableModel implements TableModel {
	private static final CallListEntryProperties[] columnOrder = new CallListEntryProperties[CallListEntryProperties.values().length];

	static {
		columnOrder[0] = CallListEntryProperties.id;
		columnOrder[1] = CallListEntryProperties.startTime;
		columnOrder[2] = CallListEntryProperties.callerNumber;
		columnOrder[3] = CallListEntryProperties.calledNumber;
		columnOrder[4] = CallListEntryProperties.direction;
		columnOrder[5] = CallListEntryProperties.result;
		columnOrder[6] = CallListEntryProperties.duration;
		columnOrder[7] = CallListEntryProperties.callDescription;
		columnOrder[8] = CallListEntryProperties.voicemailId;
		columnOrder[9] = CallListEntryProperties.groupId;
		columnOrder[10] = CallListEntryProperties.callerJabberId;
	}

	private CallList callList;

	public CallListTableModel() {
		callList = new CallList();
		callList.setEntries(new ArrayList<CallListEntry>());
	}

	private List<TableModelListener> listeners = new ArrayList<TableModelListener>();

	public void addTableModelListener(TableModelListener listener) {
		listeners.add(listener);
	}

	public Class<?> getColumnClass(int columnIndex) {
		return columnOrder[columnIndex].getType();
	}

	public int getColumnCount() {
		return columnOrder.length;
	}

	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	public int getRowCount() {
		return callList.getEntries().size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		return Utils.getProperty(columnOrder[columnIndex], callList.getEntries().get(rowIndex));
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

	public void removeTableModelListener(TableModelListener listener) {
		listeners.remove(listener);
	}

	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		// Not implemented
	}

	public CallListEntry getEntry(int rowIndex) {
		return callList.getEntries().get(rowIndex);
	}

	public void removeEntry(int rowIndex) {
		callList.getEntries().remove(rowIndex);
		fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
			TableModelEvent.DELETE));
	}
	public void addEntry(CallListEntry callListEntry) {
		callList.getEntries().add(callListEntry);
		fireTableChanged(new TableModelEvent(this));
	}

	public CallList getCallList() {
		return callList;
	}

	public void setCallList(CallList callList) {
		this.callList = callList;
		fireTableChanged(new TableModelEvent(this));
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

	public void removeEntries(List<Integer> rows) {
		List<Integer> myRows = new ArrayList<Integer>(rows);
		Collections.sort(myRows);
		Collections.reverse(myRows);
		for (int row : myRows) {
			removeEntry(row);
		}
	}
}
