package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.ConferenceCall;
import de.starface.integration.uci.java.v30.values.ConferenceCallProperties;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

public class ConferenceCallTableModel implements TableModel {

	private static final ConferenceCallProperties[] columnOrder = new ConferenceCallProperties[ConferenceCallProperties.values().length];
	private List<TableModelListener> listeners = new ArrayList<TableModelListener>();

	private ArrayList<ConferenceCall> conferenceCalls;

	static {
		columnOrder[0] = ConferenceCallProperties.id;
		columnOrder[1] = ConferenceCallProperties.number;
		columnOrder[2] = ConferenceCallProperties.name;
		columnOrder[3] = ConferenceCallProperties.listening;
		columnOrder[4] = ConferenceCallProperties.muted;
		columnOrder[5] = ConferenceCallProperties.talking;
		columnOrder[6] = ConferenceCallProperties.wantToSpeak;
		columnOrder[7] = ConferenceCallProperties.admin;
		columnOrder[8] = ConferenceCallProperties.state;
		columnOrder[9] = ConferenceCallProperties.roomId;
		columnOrder[10] = ConferenceCallProperties.enteredTimestamp;
		columnOrder[11] = ConferenceCallProperties.duration;
		columnOrder[12] = ConferenceCallProperties.jabberId;
		columnOrder[13] = ConferenceCallProperties.avatarHash;
	}

	public ConferenceCallTableModel() {
		conferenceCalls = new ArrayList<ConferenceCall>();
	}

	@Override
	public int getRowCount() {
		return conferenceCalls.size();
	}

	@Override
	public int getColumnCount() {
		return columnOrder.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		return columnOrder[columnIndex].getType();
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		return Utils.getProperty(columnOrder[columnIndex], conferenceCalls.get(rowIndex));
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		// TODO Auto-generated method stub

	}

	@Override
	public void addTableModelListener(TableModelListener listener) {
		listeners.add(listener);

	}

	@Override
	public void removeTableModelListener(TableModelListener listener) {
		listeners.remove(listener);

	}

	private void fireTableChanged(TableModelEvent event) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(event);
			}
		}
	}

	public void updateConferenceCall(ConferenceCall call) {
		final String callId = call.getId();
		int rowIndex = 0;
		for (ListIterator<ConferenceCall> listIterator = conferenceCalls.listIterator(); listIterator.hasNext(); rowIndex = listIterator.nextIndex()) {
			if (listIterator.next().getId().equals(callId)) {
				listIterator.set(call);
				fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
					TableModelEvent.UPDATE));
				break;
			}
		}
		if (rowIndex == conferenceCalls.size()) {
			conferenceCalls.add(call);
			fireTableChanged(new TableModelEvent(this, conferenceCalls.size(), conferenceCalls.size(),
				TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
		}
	}

	public void removeConferenceCall(String callId) {
		int rowIndex = 0;
		for (ListIterator<ConferenceCall> listIterator = conferenceCalls.listIterator(); listIterator.hasNext(); rowIndex = listIterator.nextIndex()) {
			if (listIterator.next().getId().equals(callId)) {
				listIterator.remove();
				fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
					TableModelEvent.DELETE));
			}
		}
	}

	public ConferenceCall getConferenceCall(int rowIndex) {
		return conferenceCalls.get(rowIndex);
	}

	public void setConferenceCalls(List<ConferenceCall> conferenceCalls) {
		this.conferenceCalls = (ArrayList<ConferenceCall>) conferenceCalls;
		fireTableChanged(new TableModelEvent(this));

	}
}
