package de.starface.examples.uci30.swing.viewmodel;

import de.starface.examples.uci30.swing.model.ConnectionManager;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.requests.UciCallRequests;
import de.starface.integration.uci.v30.client.UciConnectionEvents;
import de.starface.integration.uci.v30.client.UciConnectionRequests;
import de.starface.integration.uci.v30.client.UciProxy;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.UcpConnectionFailedException;

public class DemoClientViewModel {

	public interface Events {
		public void connectedChanged();
	}

	private UciCallRequests callRequests;
	private final ConnectionManager connectionManager;
	private Events eventListener;
	private boolean connected = false;
	private final LoginViewModel loginViewModel;
	private final PhoneViewModel phoneViewModel;
	private final GroupViewModel groupViewModel = new GroupViewModel();
	private final CallListViewModel callListViewModel;
	private final ConferenceCallViewModel conferenceCallViewModel;
	private final CallViewModel callViewModel;
	private final CcbsCallViewModel ccbsCallViewModel = new CcbsCallViewModel();
	private final MonitorCallViewModel monitorCallViewModel = new MonitorCallViewModel();
	private final FaxListViewModel faxListViewModel = new FaxListViewModel(groupViewModel);
	private final FunctionKeyViewModel functionKeyViewModel;
	private final VoicemailListViewModel voicemailListViewModel;
	private final UserStateViewModel userStateViewModel;
	private final FaxViewModel faxViewModel = new FaxViewModel();
	private final RedirectViewModel redirectViewModel = new RedirectViewModel(groupViewModel);
	private final QueueViewModel queueViewModel = new QueueViewModel();
	private final PermissionsViewModel permissionsViewModel = new PermissionsViewModel();
	private final FmcViewModel fmcViewModel = new FmcViewModel();

	private final SystemSettingViewModel systemSettingViewModel;

	public DemoClientViewModel(ConnectionManager connectionManager, GeneralViewTasks generalViewTasks) {
		this.connectionManager = connectionManager;
		loginViewModel = new LoginViewModel(connectionManager);
		userStateViewModel = new UserStateViewModel(generalViewTasks);
		phoneViewModel = new PhoneViewModel(generalViewTasks);
		systemSettingViewModel = new SystemSettingViewModel(generalViewTasks);

		functionKeyViewModel = new FunctionKeyViewModel(phoneViewModel, generalViewTasks);
		callListViewModel = new CallListViewModel(groupViewModel, phoneViewModel);
		conferenceCallViewModel = new ConferenceCallViewModel(phoneViewModel);
		callViewModel = new CallViewModel(phoneViewModel, conferenceCallViewModel);
		voicemailListViewModel = new VoicemailListViewModel(phoneViewModel, callViewModel);
		subscribeEvents();
	}

	private void subscribeEvents() {
		connectionManager.setEventListener(new ConnectionManager.Events() {
			public void newUciProxy(UciProxy proxy) {
				setUciServiceRequests(proxy);
				setUciConnectionRequsts(proxy);
			}

		});
	}

	private void setUciServiceRequests(UciServiceRequests uciServiceRequests) {
		try {
			callListViewModel.setUciServiceRequests(uciServiceRequests);
			callViewModel.setUciServiceRequests(uciServiceRequests);
			ccbsCallViewModel.setUciServiceRequests(uciServiceRequests);
			monitorCallViewModel.setUciServiceRequests(uciServiceRequests);
			conferenceCallViewModel.setUciServiceRequests(uciServiceRequests);
			faxListViewModel.setUciServiceRequests(uciServiceRequests);
			functionKeyViewModel.setUciServiceRequests(uciServiceRequests);
			voicemailListViewModel.setUciServiceRequests(uciServiceRequests);
			phoneViewModel.setUciServiceRequests(uciServiceRequests);
			userStateViewModel.setUciServiceRequests(uciServiceRequests);
			faxViewModel.setUciServiceRequests(uciServiceRequests);
			groupViewModel.setUciServiceRequests(uciServiceRequests);
			queueViewModel.setUciServiceRequests(uciServiceRequests);
			redirectViewModel.setUciServiceRequests(uciServiceRequests);
			permissionsViewModel.setUciServiceRequests(uciServiceRequests);
			callRequests = uciServiceRequests.getRequests(UciCallRequests.class);
			systemSettingViewModel.setUciServiceRequests(uciServiceRequests);
			fmcViewModel.setUciServiceRequests(uciServiceRequests);
		} catch (UciException e) {
			e.printStackTrace();
		}
	}

	private void setUciConnectionRequsts(UciConnectionRequests uciConnectionRequests) {
		uciConnectionRequests.addConnectionEventListener(new UciConnectionEvents() {
			public void receiveConnectionState(boolean connectionAvailable) {
				setConnected(connectionAvailable);
			}

			public void receiveConnectionError(UcpConnectionFailedException e) {
				e.printStackTrace();
				receiveConnectionState(false);
			}
		});
		setConnected(uciConnectionRequests.getConnectionState());
	}

	public void disconnect() {
		connectionManager.disconnect();
	}

	public void setEventListener(Events eventListener) {
		this.eventListener = eventListener;
	}

	public boolean isConnected() {
		return connected;
	}

	private void setConnected(boolean connected) {
		this.connected = connected;
		if (eventListener != null) {
			eventListener.connectedChanged();
			if (connected) {
				try {
					callRequests.getCurrentMessageWaitingState();
				} catch (UciException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	public LoginViewModel getLoginViewModel() {
		return loginViewModel;
	}

	public CallListViewModel getCallListViewModel() {
		return callListViewModel;
	}

	public CallViewModel getCallViewModel() {
		return callViewModel;
	}

	public CcbsCallViewModel getCcbsCallViewModel() {
		return ccbsCallViewModel;
	}

	public ConferenceCallViewModel getConferenceCallViewModel() {
		return conferenceCallViewModel;
	}

	public FaxListViewModel getFaxListViewModel() {
		return faxListViewModel;
	}

	public FunctionKeyViewModel getFunctionKeyViewModel() {
		return functionKeyViewModel;
	}

	public VoicemailListViewModel getVoicemailListViewModel() {
		return voicemailListViewModel;
	}

	public PhoneViewModel getPhoneViewModel() {
		return phoneViewModel;
	}

	public UserStateViewModel getUserStateViewModel() {
		return userStateViewModel;
	}

	public FaxViewModel getFaxViewModel() {
		return faxViewModel;
	}

	public GroupViewModel getGroupViewModel() {
		return groupViewModel;
	}

	public RedirectViewModel getRedirectViewModel() {
		return redirectViewModel;
	}

	/**
	 * @return the monitorCallViewModel
	 */
	public MonitorCallViewModel getMonitorCallViewModel() {
		return monitorCallViewModel;
	}

	public QueueViewModel getQueueViewModel() {
		return queueViewModel;
	}

	public PermissionsViewModel getPermissionsVierModel() {
		return permissionsViewModel;
	}

	public SystemSettingViewModel getSystemSettingViewModel() {
		return systemSettingViewModel;
	}

	public FmcViewModel getFmcViewModel() {
		return fmcViewModel;
	}
}
