/*
 * $Id$
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.FaxList;
import de.starface.integration.uci.java.v30.types.FaxListEntry;
import de.starface.integration.uci.java.v30.types.FaxListEntryMovedData;
import de.starface.integration.uci.java.v30.values.FaxListEntryProperties;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the fax list
 *
 * @author tilmann
 *
 */
public class FaxListTableModel implements TableModel {
	private static final FaxListEntryProperties[] columnOrder = new FaxListEntryProperties[FaxListEntryProperties.values().length];

	private List<TableModelListener> listeners = new ArrayList<TableModelListener>();

	static {
		columnOrder[0] = FaxListEntryProperties.id;
		columnOrder[1] = FaxListEntryProperties.receptionTime;
		columnOrder[2] = FaxListEntryProperties.callerNumber;
		columnOrder[3] = FaxListEntryProperties.calledNumber;
		columnOrder[4] = FaxListEntryProperties.folder;
		columnOrder[5] = FaxListEntryProperties.partial;
		columnOrder[6] = FaxListEntryProperties.callDescription;
		columnOrder[7] = FaxListEntryProperties.groupId;
		columnOrder[8] = FaxListEntryProperties.callerJabberId;
	}

	private FaxList faxList;

	public FaxListTableModel() {
		faxList = new FaxList();
		faxList.setEntries(new ArrayList<FaxListEntry>());
	}

	public void addTableModelListener(TableModelListener listener) {
		listeners.add(listener);
	}

	public Class<?> getColumnClass(int columnIndex) {
		return columnOrder[columnIndex].getType();
	}

	public int getColumnCount() {
		return columnOrder.length;
	}

	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	public int getRowCount() {
		return faxList.getEntries().size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		return Utils.getProperty(columnOrder[columnIndex], faxList.getEntries().get(rowIndex));
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

	public void removeTableModelListener(TableModelListener listener) {
		listeners.remove(listener);
	}

	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		// Not implemented	
	}

	public FaxListEntry getEntry(int rowIndex) {
		return faxList.getEntries().get(rowIndex);
	}

	public void addEntry(FaxListEntry faxListEntry) {
		int rowIndex = faxList.getEntries().size();
		faxList.getEntries().add(faxListEntry);
		fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
			TableModelEvent.INSERT));
	}

	public void removeEntry(int rowIndex) {
		faxList.getEntries().remove(rowIndex);
		fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
			TableModelEvent.DELETE));
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

	public FaxList getFaxList() {
		return faxList;
	}

	public void setFaxList(FaxList faxList) {
		this.faxList = faxList;
		fireTableChanged(new TableModelEvent(this));
	}

	public void removeEntries(List<Integer> rows) {
		List<Integer> myRows = new ArrayList<Integer>(rows);
		Collections.sort(myRows);
		Collections.reverse(myRows);
		for (int row : myRows) {
			removeEntry(row);
		}
	}
	
	public void moveEntries(List<FaxListEntryMovedData> movedFaxCdrEntries) {
		for (int i = 0; i < getRowCount(); i++) {
			FaxListEntry entry = getEntry(i);
			for (FaxListEntryMovedData movedData : movedFaxCdrEntries) {
				if (entry != null && movedData != null && movedData.getFaxListEntry() != null
						&& movedData.getFaxListEntry().getId().equalsIgnoreCase(entry.getId())) {
					entry.setFolder(movedData.getFaxListEntry().getFolder());
				}
			}
		}
		fireTableChanged(new TableModelEvent(this));
	}

}
