/*
 * Copyright (c) 2016 by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *  
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE SOFTWARE. UNAUTHORIZED ACCESS, USE,
 *  REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */

package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciFmcPhoneEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciFmcPhoneRequests;
import de.starface.integration.uci.java.v30.types.FmcPhone;
import de.starface.integration.uci.java.v30.types.FmcScheduleEntry;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.utils.EventsProvider;
import de.starface.integration.uci.v30.client.utils.EventsProviderSupport;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import javax.swing.table.TableModel;

public class FmcViewModel implements UciFmcPhoneEvents, EventsProvider<UciFmcPhoneEvents> {
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	private EventsProviderSupport<UciFmcPhoneEvents> eventsSupport = new EventsProviderSupport<UciFmcPhoneEvents>(
		UciFmcPhoneEvents.class);

	private UciFmcPhoneRequests fmcPhoneRequests;

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.fmcPhoneRequests = uciServiceRequests.getRequests(UciFmcPhoneRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciFmcPhoneEvents.class);
	}

	@Override
	public void fmcPhonesChangedEvent(final List<FmcPhone> phones) {
		fmcPhonesTableModel.setFmcPhones(phones);
		eventsSupport.getEventsDistributor().fmcPhonesChangedEvent(phones);
	}

	@Override
	public void subscribeEvents(UciFmcPhoneEvents listener) {
		eventsSupport.subscribeEvents(listener);
	}

	@Override
	public void unsubscribeEvents(UciFmcPhoneEvents listener) {
		eventsSupport.unsubscribeEvents(listener);
	}

	/// View Actions
	////////////////////////////////////////////////////////////////

	private void doGetFmcPhones() {
		try {
			List<FmcPhone> fmcPhones = fmcPhoneRequests.getFmcPhones();
			fmcPhonesTableModel.setFmcPhones(fmcPhones);
			eventsSupport.getEventsDistributor().fmcPhonesChangedEvent(fmcPhones);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doCreateFmcPhone() {
		List<FmcPhone> phones = fmcPhonesTableModel.getFmcPhones();
		FmcPhone newPhone = new FmcPhone();
		newPhone.setFmcId(-1);
		newPhone.setTelephoneId(-1);
		ArrayList<FmcScheduleEntry> schedules = new ArrayList<FmcScheduleEntry>();
		FmcScheduleEntry scheduleEntry = new FmcScheduleEntry();
		scheduleEntry.setMonday(true);
		scheduleEntry.setTuesday(true);
		scheduleEntry.setWednesday(true);
		scheduleEntry.setThursday(true);
		scheduleEntry.setFriday(true);
		scheduleEntry.setSaturday(true);
		scheduleEntry.setSunday(true);
		scheduleEntry.setTimeBegin("00:00");
		scheduleEntry.setTimeEnd("23:59");
		schedules.add(scheduleEntry);
		newPhone.setSchedules(schedules);
		phones.add(newPhone);
		fmcPhonesTableModel.setFmcPhones(phones);
		eventsSupport.getEventsDistributor().fmcPhonesChangedEvent(phones);
	}

	private void doSaveFmcPhone() {
		if (selectedRow >= 0) {
			FmcPhone fmcPhone = fmcPhonesTableModel.getFmcPhone(selectedRow);
			try {
				fmcPhoneRequests.saveFmcPhone(fmcPhone);
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doDeleteFmcPhones() {
		if (selectedRow >= 0) {
			FmcPhone fmcPhone = fmcPhonesTableModel.getFmcPhone(selectedRow);
			try {
				fmcPhoneRequests.deleteFmcPhone(fmcPhone);
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private final ActionListener getFmcPhonesActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetFmcPhones();
		}
	};

	private final ActionListener createFmcPhoneActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doCreateFmcPhone();
		}
	};

	private final ActionListener saveFmcPhoneActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSaveFmcPhone();
		}
	};

	private final ActionListener deleteFmcPhoneActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doDeleteFmcPhones();
			doGetFmcPhones();
		}
	};

	/// View Data
	////////////////////////////////////////////////////////////////

	private final FmcPhonesTableModel fmcPhonesTableModel = new FmcPhonesTableModel();

	private int selectedRow = -1;

	private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public int getSelectedRow() {
		return selectedRow;
	}

	public static final String PROPERTY_SELECTED_ROW = "selectedRow";

	public void setSelectedRow(int selectedRow) {
		int old = this.selectedRow;
		this.selectedRow = selectedRow;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_ROW, old, selectedRow);
	}

	public TableModel getFmcPhonesTableModel() {
		return fmcPhonesTableModel;
	}

	public ActionListener getSaveFmcPhoneActionListener() {
		return saveFmcPhoneActionListener;
	}

	public ActionListener getDeleteFmcPhoneActionListener() {
		return deleteFmcPhoneActionListener;
	}

	public ActionListener getGetFmcPhonesActionListener() {
		return getFmcPhonesActionListener;
	}

	public ActionListener getCreateFmcPhoneActionListener() {
		return createFmcPhoneActionListener;
	}
}
