/*
 * $Id: RedirectSettingsTableModel.java 515 2011-03-11 19:14:15Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2011-03-11 20:14:15 +0100 (Fr, 11 Mrz 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 515 $
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.GroupSetting;
import de.starface.integration.uci.java.v30.types.PhoneNumber;
import de.starface.integration.uci.java.v30.values.GroupSettingProperties;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the redirect settings
 *
 * @author tilmann
 */
public class GroupSettingsTableModel implements TableModel {
	private static final GroupSettingProperties[] columnOrder;

	static {
		columnOrder = new GroupSettingProperties[4];
		columnOrder[0] = GroupSettingProperties.name;
		columnOrder[1] = GroupSettingProperties.logonId;
		columnOrder[2] = GroupSettingProperties.phoneNumbers;
		columnOrder[3] = GroupSettingProperties.loggedOn;
	}

	private List<TableModelListener> listeners = new ArrayList<TableModelListener>(1);

	private List<GroupSetting> groupSettings = new ArrayList<GroupSetting>();

	public void addTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.add(l);
		}
	}

	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
			case 2:
				return String.class;
			default:
				return columnOrder[columnIndex].getType();
		}
	}

	public int getColumnCount() {
		return columnOrder.length;
	}

	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	public int getRowCount() {
		return groupSettings.size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 2:
				return phoneNumberListToString(groupSettings.get(rowIndex).getPhoneNumbers());
			default:
				return Utils.getProperty(columnOrder[columnIndex], groupSettings.get(rowIndex));
		}
	}

	private String phoneNumberListToString(List<PhoneNumber> phoneNumbers) {
		StringBuilder sb = new StringBuilder();
		if (phoneNumbers.size() > 0) {
			sb.append(phoneNumbers.get(0).getNumber());
		}
		for (int i = 1; i < phoneNumbers.size(); i++) {
			sb.append(", ");
			sb.append(phoneNumbers.get(i).getNumber());
		}
		return sb.toString();
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 3:
				return true;
			default:
				return false;
		}
	}

	public List<GroupSetting> getGroupSettings() {
		return groupSettings;
	}

	public void setGroupSettings(List<GroupSetting> groupSettings) {
		this.groupSettings = groupSettings;
		fireTableChanged(new TableModelEvent(this));
	}

	public GroupSetting getGroupSetting(int rowIndex) {
		return groupSettings.get(rowIndex);
	}

	public void removeTableModelListener(TableModelListener l) {
		synchronized (listeners) {
			listeners.remove(l);
		}
	}

	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 3:
				Utils.setProperty(columnOrder[columnIndex], groupSettings.get(rowIndex), value);
			default:
		}
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

}
