package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciQueueEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciQueueRequests;
import de.starface.integration.uci.java.v30.types.QueueAgentState;
import de.starface.integration.uci.java.v30.types.QueueSetting;
import de.starface.integration.uci.java.v30.types.UciQueueCall;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.utils.EventsProvider;
import de.starface.integration.uci.v30.client.utils.EventsProviderSupport;

import org.mortbay.log.Log;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Collections;
import java.util.List;

import javax.swing.table.TableModel;

public class QueueViewModel implements UciQueueEvents, EventsProvider<UciQueueEvents> {

	private final QueueSettingsTableModel queueSettingsTableModel = new QueueSettingsTableModel();
	private final QueueAgentTableModel queueAgentTableModel = new QueueAgentTableModel();
	private final QueueCallTableModel queueCallTableModel = new QueueCallTableModel(this);

	private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	private int selectedRow = -1;

	public int getSelectedRow() {
		return selectedRow;
	}

	public static final String PROPERTY_SELECTED_ROW = "selectedRow";

	public void setSelectedRow(int selectedRow) {
		int old = this.selectedRow;
		this.selectedRow = selectedRow;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_ROW, old, selectedRow);
		if (selectedRow != -1) {

			Integer queueId = (Integer) queueSettingsTableModel.getValueAt(selectedRow, 2);
			this.queueAgentTableModel.setCurrentlyDisplayedGroupAccountId(queueId);
			this.doGetQueueAgentsSettings();
			this.queueCallTableModel.setSelectedQueueId(queueId);
		} else {
			this.queueAgentTableModel.setCurrentlyDisplayedGroupAccountId(-1);
			this.queueCallTableModel.setSelectedQueueId(-1);
		}
	}

	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	public QueueViewModel() {
	}

	private EventsProviderSupport<UciQueueEvents> eventsSupport = new EventsProviderSupport<UciQueueEvents>(
		UciQueueEvents.class);

	private UciQueueRequests queueRequests;

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.queueRequests = uciServiceRequests.getRequests(UciQueueRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciQueueEvents.class);
		this.doGetQueueCalls();
	}

	@Override
	public void queueSettingsChanged(List<QueueSetting> queueSettings) {
		queueSettingsTableModel.setQueueSettings(queueSettings);
		eventsSupport.getEventsDistributor().queueSettingsChanged(queueSettings);

	}

	@Override
	public void queueSettingChanged(Integer queueAccountId) {
		QueueSetting possiblyNull;
		try {
			possiblyNull = this.queueRequests.getQueueSetting(queueAccountId);
		} catch (UciException e) {
			Log.warn("Could not fetch settings for group " + queueAccountId);
			possiblyNull = null;
		}
		queueSettingsTableModel.updateQueueSetting(queueAccountId, possiblyNull);
		eventsSupport.getEventsDistributor().queueSettingChanged(queueAccountId);
	}

	@Override
	public void queueAgentStateChanged(QueueAgentState agentState) {
		queueAgentTableModel.updateAgentState(agentState);
	}

	@Override
	public void subscribeEvents(UciQueueEvents listener) {
		eventsSupport.subscribeEvents(listener);
	}

	@Override
	public void unsubscribeEvents(UciQueueEvents listener) {
		eventsSupport.unsubscribeEvents(listener);
	}

	/// View Actions
	////////////////////////////////////////////////////////////////
	private void doGetQueueSettings() {
		try {
			// Retrieve the queue settings sending the getQueueSettings request
			List<QueueSetting> queueSettings = queueRequests.getQueueSettings();
			queueSettingsTableModel.setQueueSettings(queueSettings);
			eventsSupport.getEventsDistributor().queueSettingsChanged(queueSettings);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}

	}

	private void doGetQueueCalls() {
		List<UciQueueCall> calls;
		try {
			calls = queueRequests.getQueueCalls();
			queueCallTableModel.init(calls);
		} catch (UciException e) {
			e.printStackTrace();
		}
	}

	private void doGetQueueAgentsSettings() {
		try {
			List<QueueAgentState> agentStates = queueRequests.getQueueAgents(queueAgentTableModel.getCurrentlyDisplayedGroupAccountId());
			queueAgentTableModel.setQueueAgents(agentStates,
				queueAgentTableModel.getCurrentlyDisplayedGroupAccountId());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private final ActionListener getQueueSettingsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetQueueSettings();
		}
	};

	private final ActionListener getQueueAgentsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetQueueAgentsSettings();
		}
	};

	private final ActionListener getGetQueueCallsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetQueueCalls();
		}
	};

	/// View Data
	////////////////////////////////////////////////////////////////

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public TableModel getQueueSettingsTableModel() {
		return queueSettingsTableModel;
	}

	public TableModel getQueueAgentTableModel() {
		return queueAgentTableModel;
	}

	public TableModel getQueueCallTableModel() {
		return queueCallTableModel;
	}

	public ActionListener getGetQueueSettingsActionListener() {
		return getQueueSettingsActionListener;
	}

	/**
	 * @return Returns the getQueueAgentsActionListener.
	 */
	public ActionListener getGetQueueAgentsActionListener() {
		return getQueueAgentsActionListener;
	}

	public ActionListener getGetQueueCallsActionListener() {
		return getGetQueueCallsActionListener;
	}

	@Override
	public void queueAgentChangedEvent(Integer groupAccountId) {
		if (queueAgentTableModel.getCurrentlyDisplayedGroupAccountId() == groupAccountId) {
			try {
				Log.info("The number of agents in queue " + groupAccountId + " changed.");
				queueAgentTableModel.setQueueAgents(
					queueRequests.getQueueAgents(queueAgentTableModel.getCurrentlyDisplayedGroupAccountId()),
					groupAccountId);
			} catch (UciException e) {
				Log.warn("Could not fetch Agent settings ", e);
			}
		}
		//just in case we got removed or added to a group
		doGetQueueSettings();
		doGetQueueCalls();
	}

	@Override
	public void queueGroupChangedEvent(boolean newGroup, Integer groupAccountId) {
		if (newGroup) {
			Log.info("New Queue (" + groupAccountId + ") was created.");
		} else {
			if (queueAgentTableModel.getCurrentlyDisplayedGroupAccountId() == groupAccountId) {
				queueAgentTableModel.setQueueAgents(Collections.EMPTY_LIST, groupAccountId);
			}
			Log.info("Queue (" + groupAccountId + ") was deleted.");
		}

	}

	@Override
	public void queueCallEvent(UciQueueCall call) {
		queueCallTableModel.updateCall(call);
	}

	@Override
	public void queueCallChangedIdEvent(String oldId, String newId) {
		queueCallTableModel.updateCallId(oldId, newId);

	}

	public String getQueueName(int AccountId) {
		return queueSettingsTableModel.getQueueName(AccountId);
	}

	public String getAgentName(int AccountId) {
		return queueAgentTableModel.getAgentName(AccountId);
	}

}
