package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciGroupEvents;
import de.starface.integration.uci.java.v30.messages.events.UciRedirectEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciRedirectRequests;
import de.starface.integration.uci.java.v30.types.GroupSetting;
import de.starface.integration.uci.java.v30.types.RedirectSetting;
import de.starface.integration.uci.java.v30.values.GroupRestriction;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.java.v30.values.RedirectSettingProperties;
import de.starface.integration.uci.java.v30.values.RedirectSettingType;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.utils.EventsProvider;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.swing.table.TableModel;

public class RedirectViewModel implements UciRedirectEvents, UciGroupEvents {
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	private UciRedirectRequests redirectRequests;

	public RedirectViewModel(EventsProvider<UciGroupEvents> groupEventsProvider) {
		groupEventsProvider.subscribeEvents(this);
	}

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.redirectRequests = uciServiceRequests.getRequests(UciRedirectRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciRedirectEvents.class);
	}

	@Override
	public void groupSettingsChanged(List<GroupSetting> groupSettings) {
		setGroupRestrictionValues(groupSettings);
	}

	@Override
	public void redirectSettingsChanged(List<RedirectSetting> redirectSettings) {
		if (typeRestriction != null) {
			Iterator<RedirectSetting> iter = redirectSettings.iterator();
			while (iter.hasNext() && iter.next().getType() != typeRestriction) {
				iter.remove();
			}
		}
		redirectSettingsTableModel.updateRedirectSettings(redirectSettings);
	}

	/// View Actions
	////////////////////////////////////////////////////////////////

	private void doGetRedirectSettings() {
		try {
			String groupRestriction = this.groupRestriction instanceof GroupSetting ? ((GroupSetting) this.groupRestriction).getId()
				: (String) this.groupRestriction;

			// Retrieve the redirect settings sending the getRedirectSettings request
			List<RedirectSetting> redirectSettings = redirectRequests.getRedirectSettings(typeRestriction,
				groupRestriction, orderProperty, orderDirection);
			redirectSettingsTableModel.setRedirectSettings(redirectSettings);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doSetRedirectSetting() {
		if (selectedRow >= 0) {
			RedirectSetting redirectSetting = redirectSettingsTableModel.getRedirectSetting(selectedRow);
			try {
				redirectRequests.setRedirectSetting(redirectSetting);
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doSetRedirectSettings() {
		try {
			redirectRequests.setRedirectSettings(redirectSettingsTableModel.getRedirectSettings());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private final ActionListener getRedirectSettingsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetRedirectSettings();
		}
	};

	private final ActionListener setRedirectSettingActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSetRedirectSetting();
		}
	};

	private final ActionListener setRedirectSettingsActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSetRedirectSettings();
		}
	};

	/// View Data
	////////////////////////////////////////////////////////////////

	private final RedirectSettingsTableModel redirectSettingsTableModel = new RedirectSettingsTableModel();

	private int selectedRow = -1;

	private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	private RedirectSettingType typeRestriction;

	private RedirectSettingProperties orderProperty;

	private OrderDirection orderDirection;

	private Object groupRestriction = "";

	private List<Object> groupRestrictionValues;

	{
		setGroupRestrictionValues(Collections.<GroupSetting>emptyList());
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public int getSelectedRow() {
		return selectedRow;
	}

	public static final String PROPERTY_SELECTED_ROW = "selectedRow";

	public void setSelectedRow(int selectedRow) {
		int old = this.selectedRow;
		this.selectedRow = selectedRow;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_ROW, old, selectedRow);
	}

	public TableModel getRedirectSettingsTableModel() {
		return redirectSettingsTableModel;
	}

	public RedirectSettingType getTypeRestriction() {
		return typeRestriction;
	}

	public static final String PROPERTY_TYPE_RESTRICTION = "typeRestriction";

	public void setTypeRestriction(RedirectSettingType typeRestriction) {
		RedirectSettingType old = this.typeRestriction;
		this.typeRestriction = typeRestriction;
		changeSupport.firePropertyChange(PROPERTY_TYPE_RESTRICTION, old, typeRestriction);
	}

	public RedirectSettingProperties getOrderProperty() {
		return orderProperty;
	}

	public static final String PROPERTY_ORDER_PROPERTY = "orderProperty";

	public void setOrderProperty(RedirectSettingProperties orderProperty) {
		RedirectSettingProperties old = this.orderProperty;
		this.orderProperty = orderProperty;
		changeSupport.firePropertyChange(PROPERTY_ORDER_PROPERTY, old, orderProperty);
	}

	public OrderDirection getOrderDirection() {
		return orderDirection;
	}

	public static final String PROPERTY_ORDER_DIRECTION = "orderDirection";

	public void setOrderDirection(OrderDirection orderDirection) {
		OrderDirection old = this.orderDirection;
		this.orderDirection = orderDirection;
		changeSupport.firePropertyChange(PROPERTY_ORDER_DIRECTION, old, orderDirection);
	}

	public static final String PROPERTY_GROUP_RESTRICTION = "groupRestriction";

	public Object getGroupRestriction() {
		return groupRestriction;
	}

	public void setGroupRestriction(Object groupRestriction) {
		Object old = this.groupRestriction;
		this.groupRestriction = groupRestriction;
		changeSupport.firePropertyChange(PROPERTY_GROUP_RESTRICTION, old, groupRestriction);
	}

	public static final String PROPERTY_GROUP_RESTRICTION_VALUES = "groupRestrictionValues";

	public List<Object> getGroupRestrictionValues() {
		return groupRestrictionValues;
	}

	private void setGroupRestrictionValues(List<GroupSetting> groupSettings) {
		List<Object> old = groupRestrictionValues;
		groupRestrictionValues = new ArrayList<Object>();
		groupRestrictionValues.add("");
		groupRestrictionValues.add(GroupRestriction.NON_GROUP);
		groupRestrictionValues.addAll(groupSettings);
		if (changeSupport != null) {
			changeSupport.firePropertyChange(PROPERTY_GROUP_RESTRICTION_VALUES, old, groupRestrictionValues);
		}
	}

	public ActionListener getSetRedirectSettingActionListener() {
		return setRedirectSettingActionListener;
	}

	public ActionListener getSetRedirectSettingsActionListener() {
		return setRedirectSettingsActionListener;
	}

	public ActionListener getGetRedirectSettingsActionListener() {
		return getRedirectSettingsActionListener;
	}

	public RedirectSettingType[] getTypeRestrictionValues() {
		return Utils.createArrayWithLeadingNull(RedirectSettingType.values());
	}

	public OrderDirection[] getOrderDirectionValues() {
		return Utils.createArrayWithLeadingNull(OrderDirection.values());
	}

	public RedirectSettingProperties[] getOrderPropertyValues() {
		return Utils.createArrayWithLeadingNull(RedirectSettingProperties.values());
	}

}
