package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciSystemEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciSystemRequests;
import de.starface.integration.uci.java.v30.types.GlobalLineConfiguration;
import de.starface.integration.uci.java.v30.types.WebServerConfiguration;
import de.starface.integration.uci.v30.client.UciServiceRequests;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * Created by paul on 26.03.15.
 */
public class SystemSettingViewModel implements UciSystemEvents {
	private UciServiceRequests uciServiceRequests;
	private PlainDocument document = new PlainDocument();
	private final GeneralViewTasks generalViewTasks;

	public SystemSettingViewModel(GeneralViewTasks generalViewTasks) {
		this.generalViewTasks = generalViewTasks;
	}

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.uciServiceRequests = uciServiceRequests;
		uciServiceRequests.subscribeEventsAsync(this, UciSystemEvents.class);
	}

	private void displayInDocumentField(Object object) {
		try {
			document.replace(0, document.getLength(), object.toString(), null);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void receiveGlobalLineConfigChange(GlobalLineConfiguration config) {
		System.out.println("config = [" + config + "]");
		displayInDocumentField(config);
	}

	public PlainDocument getDocument() {
		return document;
	}

	public ActionListener getRequestSystemSettingsActionListener() {
		return new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					final GlobalLineConfiguration globalLineConfiguration = uciServiceRequests.getRequests(
						UciSystemRequests.class).getGlobalLineConfiguration();
					receiveGlobalLineConfigChange(globalLineConfiguration);
				} catch (UciException e1) {
					e1.printStackTrace();
				}
			}
		};
	}

	public ActionListener getRequestWebServerConfigurationActionListener() {
		return new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					final WebServerConfiguration webServerConfiguration = uciServiceRequests.getRequests(
						UciSystemRequests.class).getWebServerConfiguration();
					displayInDocumentField(webServerConfiguration);
				} catch (UciException e1) {
					e1.printStackTrace();
				}
			}
		};
	}

	public ActionListener getRequestSystemVersionActionListener() {
		return new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					final String version = uciServiceRequests.getRequests(UciSystemRequests.class).getServerVersion();
					generalViewTasks.displayInfo("Server Version", version);
				} catch (UciException e1) {
					e1.printStackTrace();
				}
			}
		};
	}

	public ActionListener getRequestSystemTimeZoneIdActionListener() {
		return new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					final String timeZoneId = uciServiceRequests.getRequests(UciSystemRequests.class).getTimeZoneId();
					generalViewTasks.displayInfo("Server TimeZoneId", timeZoneId);
				} catch (UciException e1) {
					e1.printStackTrace();
				}
			}
		};
	}
}
