package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciUserStateEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciAccountRequests;
import de.starface.integration.uci.java.v30.messages.requests.UciUserStateRequests;
import de.starface.integration.uci.java.v30.types.Image;
import de.starface.integration.uci.java.v30.types.UserState;
import de.starface.integration.uci.java.v30.values.ChatPresence;
import de.starface.integration.uci.java.v30.values.TelephonyState;
import de.starface.integration.uci.v30.client.UciServiceRequests;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLConnection;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.MemoryCacheImageInputStream;
import javax.swing.*;

public class UserStateViewModel implements UciUserStateEvents {
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	private UciUserStateRequests userStateRequests;

	private UciAccountRequests uciAccountRequests;

	public UserStateViewModel(GeneralViewTasks generalViewTasks) {
		this.generalViewTasks = generalViewTasks;
	}

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.userStateRequests = uciServiceRequests.getRequests(UciUserStateRequests.class);
		this.uciAccountRequests = uciServiceRequests.getRequests(UciAccountRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciUserStateEvents.class);
	}

	@Override
	public void userStateChanged(UserState state) {
		setChatPresence(state.getChatPresence());
		setChatPresenceMessage(state.getChatPresenceMessage());
		setAvatarHash(state.getAvatarHash());
		setRedirectAlwaysActive(state.isRedirectAlwaysActive());
		setTelephonyState(state.getTelephonyState());
		setDoNotDisturbSetting(state.isDoNotDisturbSetting());
	}

	/// View Actions
	////////////////////////////////////////////////////////////////

	private void doGetUserState() {
		try {
			userStateChanged(userStateRequests.getUserState());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doGetAvatarHash() {
		try {
			String avatarImageHashByLogin = uciAccountRequests.getAvatarImageHashByLogin(loginInput);
			generalViewTasks.displayInfo("AvatarHash", avatarImageHashByLogin);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doGetPasswordChangeRequired() {
		try {
			boolean tt = uciAccountRequests.getPasswordChangeRequired();
			generalViewTasks.displayInfo("PasswordChangeRequired", Boolean.toString(tt));
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doSetChatPresenceMessage() {
		try {
			userStateRequests.setChatPresenceMessage(chatPresenceMessage);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doSetDoNotDisturbSetting() {
		try {
			userStateRequests.setDoNotDisturbSetting(doNotDisturbSetting);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doGetAvatarImage() {
		try {
			Image image = userStateRequests.getAvatarImage();
			if (image == null) {
				setAvatar(null);
			} else {
				Icon icon = scaleImageToIcon(image, 64);
				if (icon != null) {
					setAvatar(icon);
				}
			}
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doGetUserName() {
		try {
			setUserName(userStateRequests.getUserName());
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	public static Icon scaleImageToIcon(Image image, int size) {
		try {
			ImageReader reader = getImageReaderForMimeType(image.getMimeType());
			if (reader != null) {
				reader.setInput(new MemoryCacheImageInputStream(new ByteArrayInputStream(image.getData())));
				BufferedImage bufferedImage = reader.read(0);
				return new ImageIcon(bufferedImage.getScaledInstance(size, size, java.awt.Image.SCALE_SMOOTH));
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	private void doSetAvatarImage() {
		try {
			File file = generalViewTasks.selectFile(new ImageFileFilter());
			if (file != null) {
				//Image image = readImageFromFile(file);
				//userStateRequests.setAvatarImage(image);
				userStateRequests.setAvatarImageFile(file);
			}
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
		//		catch (IOException e)
		//		{
		//			// TODO Auto-generated catch block
		//			e.printStackTrace();
		//		}
	}

	private Image readImageFromFile(File file) throws IOException {
		byte[] imageData = new byte[new Long(file.length()).intValue()];
		InputStream inputStream = new FileInputStream(file);
		inputStream.read(imageData);
		inputStream.close();

		Image image = new Image();
		image.setMimeType(URLConnection.guessContentTypeFromName(file.getAbsolutePath()));
		image.setData(imageData);
		image.setHash("");

		return image;
	}

	private static ImageReader getImageReaderForMimeType(String mimeType) {
		Iterator<ImageReader> readers = ImageIO.getImageReadersByMIMEType(mimeType);
		if (readers.hasNext()) {
			return readers.next();
		} else {
			return null;
		}
	}

	private final ActionListener getAvatarImageActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetAvatarImage();
		}
	};

	private final ActionListener setAvatarImageActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSetAvatarImage();
		}
	};

	private final ActionListener getUserStateActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetUserState();
		}
	};

	private final ActionListener setChatPresenceMessageActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSetChatPresenceMessage();
		}
	};

	private final ActionListener setDoNotDisturbSettingActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doSetDoNotDisturbSetting();
		}
	};

	private final ActionListener userNameActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetUserName();
		}
	};

	private final ActionListener getAvatarHashByLoginActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetAvatarHash();
		}
	};

	private final ActionListener getPasswordChangeRequiredActionListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			doGetPasswordChangeRequired();
		}
	};

	/// View Data
	////////////////////////////////////////////////////////////////

	private final GeneralViewTasks generalViewTasks;

	private boolean doNotDisturbSetting;

	private boolean redirectAlwaysActive;

	private ChatPresence chatPresence;

	private String chatPresenceMessage = "";

	private String loginInput = "";

	private String avatarHash;

	private TelephonyState telephonyState;

	private Icon avatar;

	private String userName;

	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public TelephonyState[] getTelephonyStateValues() {
		return TelephonyState.values();
	}

	public ChatPresence[] getChatPresenceValues() {
		return ChatPresence.values();
	}

	public ActionListener getGetUserStateActionListener() {
		return getUserStateActionListener;
	}

	public ActionListener getSetChatPresenceMessageActionListener() {
		return setChatPresenceMessageActionListener;
	}

	public ActionListener getSetDoNotDisturbSettingActionListener() {
		return setDoNotDisturbSettingActionListener;
	}

	public ActionListener getGetAvatarImageActionListener() {
		return getAvatarImageActionListener;
	}

	public ActionListener getUserNameActionListener() {
		return userNameActionListener;
	}

	public ActionListener getGetAvatarHashByLoginActionListener() {
		return getAvatarHashByLoginActionListener;
	}

	public ActionListener getSetAvatarImageActionListener() {
		return setAvatarImageActionListener;
	}

	public boolean getDoNotDisturbSetting() {
		return doNotDisturbSetting;
	}

	public static final String PROPERTY_DO_NOT_DISTURB_SETTING = "doNotDisturbSetting";

	public void setDoNotDisturbSetting(boolean doNotDisturbSetting) {
		boolean old = this.doNotDisturbSetting;
		this.doNotDisturbSetting = doNotDisturbSetting;
		changeSupport.firePropertyChange(PROPERTY_DO_NOT_DISTURB_SETTING, old, doNotDisturbSetting);
	}

	public boolean isRedirectAlwaysActive() {
		return redirectAlwaysActive;
	}

	public static final String PROPERTY_REDIRECT_ALWAYS_ACTIVE = "redirectAlwaysActive";

	public void setRedirectAlwaysActive(boolean redirectAlwaysActive) {
		boolean old = this.redirectAlwaysActive;
		this.redirectAlwaysActive = redirectAlwaysActive;
		changeSupport.firePropertyChange(PROPERTY_REDIRECT_ALWAYS_ACTIVE, old, redirectAlwaysActive);
	}

	public ChatPresence getChatPresence() {
		return chatPresence;
	}

	public static final String PROPERTY_CHAT_PRESENCE = "chatPresence";

	public void setChatPresence(ChatPresence chatPresence) {
		ChatPresence old = this.chatPresence;
		this.chatPresence = chatPresence;
		changeSupport.firePropertyChange(PROPERTY_CHAT_PRESENCE, old, chatPresence);
	}

	public String getChatPresenceMessage() {
		return chatPresenceMessage;
	}

	public static final String PROPERTY_CHAT_PRESENCE_MESSAGE = "chatPresenceMessage";

	public void setChatPresenceMessage(String chatPresenceMessage) {
		String old = this.chatPresenceMessage;
		this.chatPresenceMessage = chatPresenceMessage;
		changeSupport.firePropertyChange(PROPERTY_CHAT_PRESENCE_MESSAGE, old, chatPresenceMessage);
	}

	public static final String PROPERTY_LOGIN_INPUT = "loginInput";

	public void setLoginInput(String loginInput) {
		String old = this.loginInput;
		this.loginInput = loginInput;
		changeSupport.firePropertyChange(PROPERTY_LOGIN_INPUT, old, loginInput);
	}

	public String getLoginInput() {
		return loginInput;
	}

	public String getAvatarHash() {
		return avatarHash;
	}

	public static final String PROPERTY_AVATAR_HASH = "avatarHash";

	public void setAvatarHash(String avatarHash) {
		String old = this.avatarHash;
		this.avatarHash = avatarHash;
		changeSupport.firePropertyChange(PROPERTY_AVATAR_HASH, old, avatarHash);
	}

	public TelephonyState getTelephonyState() {
		return telephonyState;
	}

	public static final String PROPERTY_TELEPHONY_STATE = "telephonyState";

	public void setTelephonyState(TelephonyState telephonyState) {
		TelephonyState old = this.telephonyState;
		this.telephonyState = telephonyState;
		changeSupport.firePropertyChange(PROPERTY_TELEPHONY_STATE, old, telephonyState);
	}

	public static final String PROPERTY_AVATAR = "avatar";

	public Icon getAvatar() {
		return avatar;
	}

	public void setAvatar(Icon avatar) {
		Icon old = this.avatar;
		this.avatar = avatar;
		changeSupport.firePropertyChange(PROPERTY_AVATAR, old, avatar);
	}

	public static final String PROPERTY_USER_NAME = "userName";

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		String old = this.userName;
		this.userName = userName;
		changeSupport.firePropertyChange(PROPERTY_USER_NAME, old, userName);
	}

	public ActionListener getGetPasswordChangeRequiredActionListener() {
		return getPasswordChangeRequiredActionListener;
	}

}
