/*
 * $Id$
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.integration.uci.java.v30.types.VoicemailList;
import de.starface.integration.uci.java.v30.types.VoicemailListEntry;
import de.starface.integration.uci.java.v30.types.VoicemailListEntryMovedData;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryProperties;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A swing table model for the voicemail list
 *
 * @author tilmann
 */
public class VoicemailListTableModel implements TableModel {
	private static final VoicemailListEntryProperties[] columnOrder = new VoicemailListEntryProperties[VoicemailListEntryProperties.values().length];

	private List<TableModelListener> listeners = new ArrayList<TableModelListener>();

	static {
		columnOrder[0] = VoicemailListEntryProperties.id;
		columnOrder[1] = VoicemailListEntryProperties.startTime;
		columnOrder[2] = VoicemailListEntryProperties.duration;
		columnOrder[3] = VoicemailListEntryProperties.callerNumber;
		columnOrder[4] = VoicemailListEntryProperties.calledNumber;
		columnOrder[5] = VoicemailListEntryProperties.folder;
		columnOrder[6] = VoicemailListEntryProperties.mailboxId;
		columnOrder[7] = VoicemailListEntryProperties.mailboxName;
		columnOrder[8] = VoicemailListEntryProperties.callDescription;
		columnOrder[9] = VoicemailListEntryProperties.groupId;
		columnOrder[10] = VoicemailListEntryProperties.callerJabberId;
	}

	private VoicemailList voicemailList;

	public VoicemailListTableModel() {
		voicemailList = new VoicemailList();
		voicemailList.setEntries(new ArrayList<VoicemailListEntry>());
	}

	public void addTableModelListener(TableModelListener listener) {
		listeners.add(listener);
	}

	public Class<?> getColumnClass(int columnIndex) {
		return columnOrder[columnIndex].getType();
	}

	public int getColumnCount() {
		return columnOrder.length;
	}

	public String getColumnName(int columnIndex) {
		return columnOrder[columnIndex].name();
	}

	public int getRowCount() {
		return voicemailList.getEntries().size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		return Utils.getProperty(columnOrder[columnIndex], voicemailList.getEntries().get(rowIndex));
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

	public void removeTableModelListener(TableModelListener listener) {
		listeners.remove(listener);
	}

	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		// Not implemented	
	}

	public VoicemailListEntry getEntry(int rowIndex) {
		return voicemailList.getEntries().get(rowIndex);
	}

	public void addEntry(VoicemailListEntry voicemailListEntry) {
		int rowIndex = voicemailList.getEntries().size();
		voicemailList.getEntries().add(voicemailListEntry);
		fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
			TableModelEvent.INSERT));
	}

	public void removeEntry(int rowIndex) {
		voicemailList.getEntries().remove(rowIndex);
		fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
			TableModelEvent.DELETE));
	}

	private void fireTableChanged(TableModelEvent e) {
		synchronized (listeners) {
			for (TableModelListener l : listeners) {
				l.tableChanged(e);
			}
		}
	}

	public void setVoicemailList(VoicemailList voicemailList) {
		this.voicemailList = voicemailList;
		fireTableChanged(new TableModelEvent(this));
	}

	public void removeEntries(List<Integer> rows) {
		List<Integer> myRows = new ArrayList<Integer>(rows);
		Collections.sort(myRows);
		Collections.reverse(myRows);
		for (int row : myRows) {
			removeEntry(row);
		}
	}
	
	public void moveEntries(List<VoicemailListEntryMovedData> movedVoicemailCdrEntries) {
		for (int i = 0; i < getRowCount(); i++) {
			VoicemailListEntry entry = getEntry(i);
			for (VoicemailListEntryMovedData movedData : movedVoicemailCdrEntries) {
				if (entry != null && movedData != null
                        && movedData.getVoicemailListEntry() != null
                        && movedData.getOldVoicemailId() != null
						&& movedData.getOldVoicemailId().equalsIgnoreCase(entry.getId())) {
					entry.setFolder(movedData.getVoicemailListEntry().getFolder());
					entry.setId(movedData.getVoicemailListEntry().getId());
				}
			}
		}
		fireTableChanged(new TableModelEvent(this));
	}
}
