/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client;

/**
 * Provides methods used to signal connection state changes to clients of the {@link UciProxy}
 *
 * @author tilmann
 */
public interface UciConnectionEvents {
	/**
	 * The {@link UciProxy} sends this event if the state of its connection to the UCP server
	 * changes.
	 *
	 * @param connectionAvailable
	 *        <code>true</code> if the connection to the UCP server is available, <code>false</code>
	 *        otherwise
	 */
	void receiveConnectionState(boolean connectionAvailable);

	/**
	 * In addition to the receiveConnectionState exceptions that occur while connecting to the UCP
	 * server will be reported.
	 *
	 * @param e
	 *        the {@link UcpConnectionFailedException} holding details about the problem
	 */
	void receiveConnectionError(UcpConnectionFailedException e);
}
