/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client.transport;

import de.starface.com.rpc.client.OutgoingRequest;
import de.starface.com.rpc.client.RpcExceptionConverter;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.integration.uci.java.v22.exceptions.UciException;
import de.starface.integration.uci.java.v22.ucp.exceptions.UcpCommunicationException;
import de.starface.integration.uci.v22.client.utils.EventsProvider;
import de.starface.integration.uci.v22.client.utils.EventsProviderSupport;

/**
 * This {@link RpcExceptionConverter} makes sure that only {@link UciException}s are thrown to the
 *
 * @author tilmann
 */
public class ExceptionConverter implements RpcExceptionConverter<RpcTransportToken>,
	EventsProvider<UcpTransportEvents> {

	private final EventsProviderSupport<UcpTransportEvents> eventsSupport = new EventsProviderSupport<>(
		UcpTransportEvents.class);

	/**
	 * @see de.starface.integration.uci.v22.client.utils.EventsProvider#subscribeEvents(java.lang.Object)
	 */
	public void subscribeEvents(UcpTransportEvents listener) {
		eventsSupport.subscribeEvents(listener);
	}

	/**
	 * @see de.starface.integration.uci.v22.client.utils.EventsProvider#unsubscribeEvents(java.lang.Object)
	 */
	public void unsubscribeEvents(UcpTransportEvents listener) {
		eventsSupport.unsubscribeEvents(listener);
	}

	/**
	 * @see de.starface.com.rpc.client.RpcExceptionConverter#convertExceptionToBeThrownOnProxy(java.lang.Throwable,
	 *      de.starface.com.rpc.client.OutgoingRequest)
	 */
	public Throwable convertExceptionToBeThrownOnProxy(Throwable e, OutgoingRequest<RpcTransportToken> request) {
		informListenersOfException(e, request);
		return convertException(e, request);
	}

	private void informListenersOfException(Throwable t, OutgoingRequest<RpcTransportToken> request) {
		eventsSupport.getEventsDistributor().ucpRequestFailed(t, request.getRpcInterface(), request.getRpcMethod());
	}

	private Throwable convertException(Throwable t, OutgoingRequest<RpcTransportToken> request) {
		if (!(t instanceof UciException)) {
			return new UcpCommunicationException("An error occured during an RPC call.", t);
		} else {
			return t;
		}
	}
}
