/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client.transport.http;

import de.starface.com.rpc.common.authentication.RpcAuth;
import de.starface.integration.uci.java.v22.exceptions.UciException;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * This class is used as a proxy to receive the events from every HTTP UCP connection and distribute
 * them according to the auth tokens to the correct UCP clients.
 *
 * @author tilmann
 *
 */
class HttpUcpTransportEventsReceiver<Events> implements InvocationHandler {
	private ConcurrentMap<LoginUrlAuthToken, Events> listeners = new ConcurrentHashMap<>(16, 0.75f, 1);

	private final Events proxy;

	private final Class<Events> eventsInterface;

	@SuppressWarnings("unchecked")
	HttpUcpTransportEventsReceiver(Class<Events> eventsInterface) {
		proxy = (Events) Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(),
			new Class<?>[]{eventsInterface}, this);
		this.eventsInterface = eventsInterface;
	}

	boolean isEmpty() {
		return listeners.isEmpty();
	}

	void addListener(LoginUrlAuthToken authToken, Events listener) {
		if (listener != null) {
			listeners.put(authToken, listener);
		}
	}

	void removeListener(LoginUrlAuthToken authToken) {
		listeners.remove(authToken);
	}

	/**
	 * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object, java.lang.reflect.Method,
	 *      java.lang.Object[])
	 */
	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		LoginUrlAuthToken currentAuthToken = RpcAuth.getCurrentAuthToken();
		Events listener = listeners.get(currentAuthToken);
		if (listener != null) {
			return method.invoke(listener, args);
		} else {
			throw new UciException("Client for loginId " + currentAuthToken.getLoginId() + " unknown.");
		}
	}

	Events getEventsDistributor() {
		return proxy;
	}

	Class<Events> getEventsInterface() {
		return eventsInterface;
	}
}
