/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client.transport.http;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * This container is used to create and store the the event receiver instances for the different UCP
 * services interface types.
 *
 * @author tilmann
 */
class HttpUcpTransportEventsReceiverRegistry {
	private Map<Class<?>, HttpUcpTransportEventsReceiver<?>> receivers = new HashMap<>();

	@SuppressWarnings("unchecked")
	private <Events> HttpUcpTransportEventsReceiver<Events> getEventsReceiver(Class<Events> eventsInterface) {
		return (HttpUcpTransportEventsReceiver<Events>) receivers.get(eventsInterface);
	}

	private <Events> HttpUcpTransportEventsReceiver<Events> createEventsReceiver(Class<Events> eventsInterface) {
		HttpUcpTransportEventsReceiver<Events> receiver = new HttpUcpTransportEventsReceiver<>(
			eventsInterface);
		receivers.put(eventsInterface, receiver);
		return receiver;
	}

	synchronized <Events> Events getEventsDistributor(Class<Events> eventsInterface) {
		HttpUcpTransportEventsReceiver<Events> receiver = getEventsReceiver(eventsInterface);
		if (receiver != null) {
			return receiver.getEventsDistributor();
		}
		return null;
	}

	private <Events> void removeEventsReceiver(Class<Events> eventsInterface) {
		receivers.remove(eventsInterface);
	}

	synchronized <Events> void addEventsListener(LoginUrlAuthToken authToken, Events listener,
												 Class<Events> eventsInterface) {
		HttpUcpTransportEventsReceiver<Events> eventsReceiver = getEventsReceiver(eventsInterface);
		if (eventsReceiver == null) {
			eventsReceiver = createEventsReceiver(eventsInterface);
		}

		eventsReceiver.addListener(authToken, listener);
	}

	synchronized <Events> void removeEventsListener(LoginUrlAuthToken authToken, Class<Events> eventsInterface) {
		HttpUcpTransportEventsReceiver<Events> eventsReceiver = getEventsReceiver(eventsInterface);
		if (eventsReceiver != null) {
			eventsReceiver.removeListener(authToken);

			if (eventsReceiver.isEmpty()) {
				removeEventsReceiver(eventsInterface);
			}
		}
	}

	synchronized Collection<Class<?>> removeEventsListeners(LoginUrlAuthToken authToken) {
		List<Class<?>> emptyReceiversInterfaces = new LinkedList<>();
		for (HttpUcpTransportEventsReceiver<?> receiver : receivers.values()) {
			receiver.removeListener(authToken);
			if (receiver.isEmpty()) {
				emptyReceiversInterfaces.add(receiver.getEventsInterface());
			}
		}
		return emptyReceiversInterfaces;
	}
}
