/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v22.client.utils;

/**
 * An events provider can be used to register event listeners subscribing to the providers events.
 *
 * @author tilmann
 *
 * @param <EventsInterface>
 *        The interface of the event listeners
 *
 */
public interface EventsProvider<EventsInterface> {
	/**
	 * Subscribe the given listener to the events of this provider
	 *
	 * @param listener
	 *        the listener to be subscribed
	 */
	public void subscribeEvents(EventsInterface listener);

	/**
	 * Unsubscribe the given listener from the events of this provider
	 *
	 * @param listener
	 *        the listener to be unsubscribed
	 */
	public void unsubscribeEvents(EventsInterface listener);
}
