/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client;

import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;

/**
 * This exception is thrown when the {@link UciProxy} cannot be opened due to a UCP connection
 * problem.
 *
 * @author tilmann
 *
 */
public class UcpConnectionFailedException extends UcpNotConnectedException {
	private static final long serialVersionUID = 1L;

	/**
	 * This static class contains constants for the codes specifying the reason for the exception.
	 *
	 * @author tilmann
	 */
	public static class Codes {
		/** A failure in the underlying XML-RPC transport has occurred */
		public static final int TRANSPORT_FAILURE = 1;

		/** The host to connect to cannot be found */
		public static final int UNKNOWN_HOST = 2;

		/** The STARFACE user authentication has failed */
		public static final int AUTHENTICATION_FAILED = 3;

		/** The communication end point on the server side is not an UCP server */
		public static final int NO_UCP_SERVER = 4;

		/** The login at the UCP server failed */
		public static final int LOGIN_FAILED = 5;

		/** Events could not be subscribed at the UCP server */
		public static final int SUBSCRIBE_EVENTS_FAILED = 6;
	}

	private int code;

	/**
	 * Creates a new {@link UcpConnectionFailedException} with the given reason code
	 *
	 * @param code
	 *        the code identifying the reason for the exception defined in {@link Codes}
	 */
	public UcpConnectionFailedException(int code) {
		super();
		this.code = code;
	}

	/**
	 * Creates a new {@link UcpConnectionFailedException} with the given reason code and message
	 *
	 * @param code
	 *        the code identifying the reason for the exception defined in {@link Codes}
	 * @param message
	 *        a details message
	 */
	public UcpConnectionFailedException(int code, String message) {
		super(message);
		this.code = code;
	}

	/**
	 * Creates a new {@link UcpConnectionFailedException} with the given reason code, message and
	 * cause
	 *
	 * @param code
	 *        the code identifying the reason for the exception defined in {@link Codes}
	 * @param message
	 *        a details message
	 * @param cause
	 *        a throwable that caused this exception to be thrown
	 */
	public UcpConnectionFailedException(int code, String message, Throwable cause) {
		super(message, cause);
		this.code = code;
	}

	/**
	 * Retrieve the reason code specifying why this exception has been thrown
	 *
	 * @return one of the constants in {@link Codes}
	 */
	public int getCode() {
		return code;
	}
}
