/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.http;

import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;
import de.starface.com.rpc.xmlrpc.http.URL;

import org.apache.commons.lang3.Validate;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * An {@link RpcAuthTokenConverter} that converts between {@link URL} and {@link LoginUrlAuthToken}.
 *
 * @author tilmann
 */
class LoginUrlAuthTokenConverter implements RpcAuthTokenConverter<URL, LoginUrlAuthToken> {

	private ConcurrentMap<String, LoginUrlAuthToken> cache = new ConcurrentHashMap<>(16, 0.75f, 1);

	LoginUrlAuthTokenConverter() {
		// Just reducing visibility
	}

	/**
	 * Clear all entries from the cache.
	 */
	synchronized void clearCache() {
		cache.clear();
	}

	/**
	 * Clear the entry for the given AccountAuthToken from the cache
	 *
	 * @param accountAuthToken
	 *        the AccountAuthToken to be removed from the cache
	 */
	synchronized void clearFromCache(LoginUrlAuthToken authToken) {
		cache.remove(authToken.getLoginId());
	}

	/**
	 * @see de.vertico.com.xmlrpc.XmlRpcAuthTokenConverter#convert(de.vertico.com.xmlrpc.XmlRpcTransportToken)
	 */
	public LoginUrlAuthToken convert(URL url) {
		final String loginId = url.getUser();
		if (loginId != null) {
			return cache.get(loginId);
		} else {
			return null;
		}
	}

	/**
	 * @see de.vertico.com.xmlrpc.XmlRpcAuthTokenConverter#convert(de.vertico.com.xmlrpc.XmlRpcAuthToken)
	 */
	public URL convert(LoginUrlAuthToken authToken) {
		URL url = authToken.getUrl();
		Validate.notNull(url, "authToken.url");
		cache.put(authToken.getLoginId(), authToken);
		return url;
	}

	/**
	 * @see de.vertico.com.xmlrpc.XmlRpcAuthTokenConverter#getAuthType()
	 */
	public Class<LoginUrlAuthToken> getAuthType() {
		return LoginUrlAuthToken.class;
	}

	/**
	 * @see de.vertico.com.xmlrpc.XmlRpcAuthTokenConverter#getTransportType()
	 */
	public Class<URL> getTransportType() {
		return URL.class;
	}
}
