/*
 * Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.logging;

import de.starface.com.rpc.common.LoggingUtils;
import de.starface.integration.uci.v30.client.UcpConnectionFailedException;
import de.starface.integration.uci.v30.client.transport.UcpTransport;
import de.starface.integration.uci.v30.client.transport.UcpTransportEvents;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

/**
 * This is a dummy implementation of {@link UcpTransport} that exposes a dummy proxies that simply
 * logs all incoming calls to a commons logging log.
 *
 * @author tilmann
 */
public class LoggingDummyUcpTransport implements UcpTransport {
	private final Log log;

	LoggingDummyUcpTransport(String loginId) {
		log = LogFactory.getLog(LoggingDummyUcpTransport.class.getName() + "." + loginId);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#close()
	 */
	public void close() {
		// Nothing to do
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#open()
	 */
	public void open() throws UcpConnectionFailedException {
		// Nothing to do
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#subscribeUcpTransportEvents(de.starface.integration.uci.v30.client.transport.UcpTransportEvents)
	 */
	public void subscribeUcpTransportEvents(UcpTransportEvents listener) {
		log.info(".subscribeUcpTransportEvents(" + listener + ")");
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#unsubscribeUcpTransportEvents(de.starface.integration.uci.v30.client.transport.UcpTransportEvents)
	 */
	public void unsubscribeUcpTransportEvents(UcpTransportEvents listener) {
		log.info(".unsubscribeUcpTransportEvents(" + listener + ")");
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#registerUcpEventListener(java.lang.Object,
	 *      java.lang.Class)
	 */
	public <Events> void registerUcpEventListener(Events listener, Class<Events> eventsInterface)
		throws UcpConnectionFailedException {
		log.info(".registerUcpEventListener(" + listener.toString() + ", " + eventsInterface.getName() + ")");
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#unregisterUcpEventListener(java.lang.Class)
	 */
	public <Events> void unregisterUcpEventListener(Class<Events> eventsInterface) {
		log.info(".unregisterUcpEventListener(" + eventsInterface.getName() + ")");
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#getUcpRequests(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	public <Requests> Requests getUcpRequests(final Class<Requests> requestsInterface)
		throws UcpConnectionFailedException {
		return (Requests) Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(),
			new Class<?>[]{requestsInterface}, new InvocationHandler() {
				public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
					log.info("." + requestsInterface.getName() + "." + method.getName() + "("
						+ LoggingUtils.rpcParametersToString(args) + ")");
					return null;
				}
			});
	}
}
