/*
 * $Id: UciClient.java 296 2010-01-28 18:00:12Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2010-01-28 19:00:12 +0100 (Do, 28 Jan 2010) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 296 $
 */
package de.starface.integration.uci.java.v22.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.UciServices;
import de.starface.integration.uci.java.v22.types.Fax;
import de.starface.integration.uci.java.v22.ucp.messages.events.UcpFaxEvents;
import de.starface.integration.uci.java.v22.values.FaxProperties;

/**
 * This interface defines communication fax events that can be received by a UCI client
 * implementation in order to handle outgoing faxes.
 *
 * @author tilmann
 *
 */
@UciService(UcpFaxEvents.SERVICE_NAME)
@RpcInterface(UcpFaxEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciFaxEvents {
	/**
	 * Much the same as ReceiveCallState the ReceiveFaxState event notifies the UCI client of state
	 * changes to an outgoing fax request by providing several properties about the fax request
	 * including the id of the fax request, the new state of the fax request and the timestamp when
	 * the request did change its state. The state Waiting is never signaled by the server. It is
	 * assumed after a successful call to SendFax.
	 *
	 * @param faxProperties
	 *        the different properties of the fax in a map. The constants of {@link FaxProperties}
	 *        are used as keys in the map.
	 */
	@RpcAsynchronous
	public void receiveFaxState(Fax faxProperties);
}
