/*
 * $Id: UcpFunctionKeyEvents.java 485 2011-01-31 14:52:54Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/client/UcpClientCommunicationCall.java $
 * 
 * $LastChangedDate: 2011-01-31 15:52:54 +0100 (Mo, 31 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 485 $
 */
package de.starface.integration.uci.java.v22.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.UciServices;
import de.starface.integration.uci.java.v22.types.FunctionKey;
import de.starface.integration.uci.java.v22.ucp.messages.events.UcpFunctionKeyEvents;

import java.util.List;

/**
 * This interface defines the function key events that can be received by the UCI client
 * implementation in order to handle function key changes entries.
 *
 * @author tilmann
 */
@UciService(UcpFunctionKeyEvents.SERVICE_NAME)
@RpcInterface(UcpFunctionKeyEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciFunctionKeyEvents {

	/**
	 * The ReceiveFunctionKeyState event provides a number of properties about the function key the
	 * state is changing of.
	 *
	 * @param functionKeyProperties
	 *        the different properties of the function key
	 */
	@RpcAsynchronous
	public void receiveFunctionKeyState(FunctionKey functionKeyProperties);

	/**
	 * This event allows the UCI client to get informed when the configuration of the function keys
	 * of the user changed.
	 *
	 * @param functionKeys
	 *        the new function keys the user has
	 */
	@RpcAsynchronous
	public void functionKeysChanged(List<FunctionKey> functionKeys);
}
