package de.starface.integration.uci.java.v22.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v22.messages.events.UciCallEvents;
import de.starface.integration.uci.java.v22.values.CallState;

import java.io.Serializable;
import java.util.Date;

/**
 * This object holds the properties of the call as returned by
 * {@link UciCallEvents#receiveCallState(Call)}.
 *
 * @author tilmann
 */
@RpcValueObject
public class Call implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The id of the call in form of a UUID as {@link String} */
	@RpcValue
	protected String id;

	/** The state of the call */
	@RpcValue
	protected CallState state;

	/** The point of time when the call reached the state */
	@RpcValue
	protected Date timestamp;

	/** The phone number that has been called */
	@RpcValue
	protected String calledNumber = "";

	/** The name of the person being called */
	@RpcValue
	protected String calledName = "";

	/** The number of the caller */
	@RpcValue
	protected String callerNumber = "";

	/** The name of the person calling */
	@RpcValue
	protected String callerName = "";

	/**
	 * The id of the group if this call was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 22)
	protected String groupId = "";

	@Override
	public String toString() {
		return "Call [id=" + id + ", state=" + state + ", timestamp=" + timestamp
			+ ", calledNumber=" + calledNumber + ", calledName=" + calledName + ", callerNumber="
			+ callerNumber + ", callerName=" + callerName + ", groupId=" + groupId + "]";
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public CallState getState() {
		return state;
	}

	public void setState(CallState state) {
		this.state = state;
	}

	public Date getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Date timestamp) {
		this.timestamp = timestamp;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public String getCalledName() {
		return calledName;
	}

	public void setCalledName(String calledName) {
		this.calledName = calledName;
	}

	public String getCallerNumber() {
		return callerNumber;
	}

	public void setCallerNumber(String callerNumber) {
		this.callerNumber = callerNumber;
	}

	public String getCallerName() {
		return callerName;
	}

	public void setCallerName(String callerName) {
		this.callerName = callerName;
	}

	public String getGroupId() {
		return groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public boolean isGroupCall() {
		return groupId != null && !"".equals(groupId);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((calledName == null) ? 0 : calledName.hashCode());
		result = prime * result + ((calledNumber == null) ? 0 : calledNumber.hashCode());
		result = prime * result + ((callerName == null) ? 0 : callerName.hashCode());
		result = prime * result + ((callerNumber == null) ? 0 : callerNumber.hashCode());
		result = prime * result + ((groupId == null) ? 0 : groupId.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((state == null) ? 0 : state.hashCode());
		result = prime * result + ((timestamp == null) ? 0 : timestamp.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Call other = (Call) obj;
		if (calledName == null) {
			if (other.calledName != null) {
				return false;
			}
		} else if (!calledName.equals(other.calledName)) {
			return false;
		}
		if (calledNumber == null) {
			if (other.calledNumber != null) {
				return false;
			}
		} else if (!calledNumber.equals(other.calledNumber)) {
			return false;
		}
		if (callerName == null) {
			if (other.callerName != null) {
				return false;
			}
		} else if (!callerName.equals(other.callerName)) {
			return false;
		}
		if (callerNumber == null) {
			if (other.callerNumber != null) {
				return false;
			}
		} else if (!callerNumber.equals(other.callerNumber)) {
			return false;
		}
		if (groupId == null) {
			if (other.groupId != null) {
				return false;
			}
		} else if (!groupId.equals(other.groupId)) {
			return false;
		}
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		if (state != other.state) {
			return false;
		}
		if (timestamp == null) {
			if (other.timestamp != null) {
				return false;
			}
		} else if (!timestamp.equals(other.timestamp)) {
			return false;
		}
		return true;
	}

}
