package de.starface.integration.uci.java.v22.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v22.messages.requests.UciUserStateRequests;
import de.starface.integration.uci.java.v22.values.ChatPresence;
import de.starface.integration.uci.java.v22.values.TelephonyState;

import java.io.Serializable;

/**
 * This object holds the properties of the user state as returned by
 * {@link UciUserStateRequests#getUserState()}
 *
 * @author tilmann
 */
@RpcValueObject
public class UserState implements Serializable {
	private static final long serialVersionUID = 1L;

	/** A flag indicating whether the user has DND set to active */
	@RpcValue
	private boolean doNotDisturbSetting;

	/** A flag indicating whether the user has any redirect always settings enabled */
	@RpcValue
	private boolean redirectAlwaysActive;

	/** The current chat presence of the user */
	@RpcValue
	private ChatPresence chatPresence = ChatPresence.UNAVAILABLE;

	/** The current chat presence message of the user */
	@RpcValue
	private String chatPresenceMessage = "";

	/**
	 * The SHA-1 hash of the users avatar image if one is set. The image itself can be queried with
	 * {@link UciUserStateRequests#getAvatarImage()}.
	 */
	@RpcValue
	private String avatarHash = "";

	/** The current telephony state of the user */
	@RpcValue
	private TelephonyState telephonyState;

	public boolean getDoNotDisturbSetting() {
		return doNotDisturbSetting;
	}

	public void setDoNotDisturbSetting(boolean doNotDisturbSetting) {
		this.doNotDisturbSetting = doNotDisturbSetting;
	}

	public boolean isRedirectAlwaysActive() {
		return redirectAlwaysActive;
	}

	public void setRedirectAlwaysActive(boolean redirectAlwaysActive) {
		this.redirectAlwaysActive = redirectAlwaysActive;
	}

	public ChatPresence getChatPresence() {
		return chatPresence;
	}

	public void setChatPresence(ChatPresence chatPresence) {
		this.chatPresence = chatPresence;
	}

	public String getChatPresenceMessage() {
		return chatPresenceMessage;
	}

	public void setChatPresenceMessage(String chatPresenceMessage) {
		this.chatPresenceMessage = chatPresenceMessage;
	}

	public String getAvatarHash() {
		return avatarHash;
	}

	public void setAvatarHash(String hash) {
		this.avatarHash = hash;
	}

	public TelephonyState getTelephonyState() {
		return telephonyState;
	}

	public void setTelephonyState(TelephonyState telephonyState) {
		this.telephonyState = telephonyState;
	}

	@Override
	public String toString() {
		return "UserState [doNotDisturbSetting=" + doNotDisturbSetting + ", redirectAlwaysActive="
			+ redirectAlwaysActive + ", chatPresence=" + chatPresence + ", chatPresenceMessage="
			+ chatPresenceMessage + ", avatarImageId=" + avatarHash + ", telephonyState=" + telephonyState + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chatPresence == null) ? 0 : chatPresence.hashCode());
		result = prime * result + ((chatPresenceMessage == null) ? 0 : chatPresenceMessage.hashCode());
		result = prime * result + (doNotDisturbSetting ? 1231 : 1237);
		result = prime * result + ((avatarHash == null) ? 0 : avatarHash.hashCode());
		result = prime * result + (redirectAlwaysActive ? 1231 : 1237);
		result = prime * result + ((telephonyState == null) ? 0 : telephonyState.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		UserState other = (UserState) obj;
		if (chatPresence != other.chatPresence) {
			return false;
		}
		if (chatPresenceMessage == null) {
			if (other.chatPresenceMessage != null) {
				return false;
			}
		} else if (!chatPresenceMessage.equals(other.chatPresenceMessage)) {
			return false;
		}
		if (doNotDisturbSetting != other.doNotDisturbSetting) {
			return false;
		}
		if (avatarHash == null) {
			if (other.avatarHash != null) {
				return false;
			}
		} else if (!avatarHash.equals(other.avatarHash)) {
			return false;
		}
		if (redirectAlwaysActive != other.redirectAlwaysActive) {
			return false;
		}
		if (telephonyState != other.telephonyState) {
			return false;
		}
		return true;
	}

}
