/*
 * $Id: UcpFunctionKeyEvents.java 642 2011-10-19 16:29:23Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/client/UcpClientCommunicationCall.java $
 * 
 * $LastChangedDate: 2011-10-19 18:29:23 +0200 (Wed, 19 Oct 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 642 $
 */
package de.starface.integration.uci.java.v22.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.values.FunctionKeyProperties;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the function key events that can be received by the UCI client
 * implementation in order to handle function key changes entries.
 *
 * @author tilmann
 */
@RpcInterface(UcpFunctionKeyEvents.SERVICE_NAME)
@UciService(UcpFunctionKeyEvents.SERVICE_NAME)
public interface UcpFunctionKeyEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v22.events.functionKey";

	/**
	 * The ReceiveFunctionKeyState event provides a number of properties about the function key the
	 * state is changing of. The information provided in the map is defined by its properties as
	 * seen in {@link FunctionKeyProperties}.
	 *
	 * @param functionKeyProperties
	 *        the different properties of the function key in a {@link Map}. The constants of
	 *        {@link FunctionKeyProperties} are used as keys in the map.
	 */
	@RpcAsynchronous
	public void receiveFunctionKeyState(Map<String, Object> functionKeyProperties);

	/**
	 * This event allows the UCI client to get informed when the configuration of the function keys
	 * of the user changed.
	 *
	 * @param functionKeys
	 *        the new function keys the user has as {@link List} of {@link Map} containing values as
	 *        defined by {@link FunctionKeyProperties}.
	 */
	@RpcAsynchronous
	public void functionKeysChanged(List<Map<String, Object>> functionKeys);
}
