/*
 * $Id: UcpConnectionRequests.java 526 2011-03-18 11:26:51Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerConnection.java $
 * 
 * $LastChangedDate: 2011-03-18 12:26:51 +0100 (Fri, 18 Mar 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 526 $
 */
package de.starface.integration.uci.java.v22.ucp.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcLogging;
import de.starface.com.rpc.annotation.RpcLoggingLevel;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v22.ucp.exceptions.UcpCommunicationException;

/**
 * This interface defines the UCP connection requests provided by the UCI server. This is a UCP
 * addition to the standard UCI requests used for UCP connection handling.
 *
 * @author tilmann
 */
@RpcInterface(UcpConnectionRequests.SERVICE_NAME)
@UciService(UcpConnectionRequests.SERVICE_NAME)
public interface UcpConnectionRequests {
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v22.requests.connection";

	/**
	 * The login request is sent by the client to connect to the UCP server. To stay connected to
	 * the server the client is required to send continuous {@link #keepAlive()} requests!
	 *
	 * @return The return value of the message indicates if the login was successful.
	 * @throws UciIllegalAccessException
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}).
	 */
	public boolean login() throws UciIllegalAccessException, UcpCommunicationException;

	/**
	 * The client uses the logout request to indicate its intention to close the UCP connection. The
	 * response of the server is always {@code true} and may therefore be ignored.
	 */
	public void logout() throws UcpCommunicationException;

	/**
	 * The keepAlive request is used to check if the server is still available, the connection has
	 * not been lost and tell the server that the client is still available.
	 *
	 * @return A return value of {@code true} denotes that the client connection to the server is
	 *         still available and {@code false} tells the client that it is not logged in from the
	 *         servers point of view. In the case of the {@code false} return value the client must
	 *         login again and discard all information about states (call, fax, ...) it had
	 *         previously received from the server.
	 */
	@RpcLogging(RpcLoggingLevel.TRACE)
	public boolean keepAlive() throws UcpCommunicationException;

}
