package de.starface.integration.uci.java.v22.values;

import de.starface.integration.uci.java.v22.ucp.messages.events.UcpCallEvents;

import java.util.Date;

/**
 * This enum holds constants with the property names of the call as returned by
 * {@link UcpCallEvents#receiveCallState(java.util.Map)}.
 *
 * @author tilmann
 */
public enum CallProperties {
	/** The id of the call in form of a UUID as {@link String} */
	id(String.class),

	/** The state of the call as {@link String} by name defined in {@link CallState} */
	state(String.class),

	/** The point of time when the call reached the state as {@link Date} */
	timestamp(Date.class),

	/** The phone number that has been called as {@link String} */
	calledNumber(String.class),

	/** The name of the person being called as {@link String} */
	calledName(String.class),

	/** The number of the caller as {@link String} */
	callerNumber(String.class),

	/** The name of the person calling as {@link String} */
	callerName(String.class),

	/**
	 * The id of the group if this call was received by a group as {@link String} (the empty string
	 * otherwise)
	 */
	groupId(String.class);

	private Class<?> type;

	private CallProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}
}
