package de.starface.integration.uci.java.v22.values;

/**
 * This enum defines the different states a call can take. The constants are used for the call
 * property {@link CallProperties#state}.
 *
 * @author tilmann
 */
public enum CallState {
	/**
	 * A new outgoing call is being requested. it is not known so far if the call will eventually be
	 * dialled.
	 */
	REQUESTED,

	/**
	 * A new call is incoming for the account and has not been further handeled. Especially no
	 * phones are ringing for the call so far.
	 */
	INCOMING,

	/**
	 * The the identified account is calling someone. Dialing has finished and the call is now
	 * proceeding through the telephony subsystem.
	 */
	PROCEEDING,

	/**
	 * The identified account is called by someone an it's telephone is ringing.
	 */
	RINGING,

	/**
	 * The the identified account is calling someone whose telephone is ringing.
	 */
	RINGBACK,

	/**
	 * This state indicates that the two parties of the call have been connected and the
	 * corresponding call is established.
	 */
	CONNECTED,

	/**
	 * When a call reaches this state it is finished independent from its previous state.
	 */
	HANGUP
}
