package de.starface.integration.uci.java.v22.values;

import java.util.Date;

/**
 * This enum contains the keys for properties of a fax transmitted in fax state changes.
 *
 * @author tilmann
 */
public enum FaxProperties {
	/** The id of the fax in form of a UUID as {@link String} */
	id(String.class),

	/** The state of the fax as {@link String} by name defined in {@link FaxState} */
	state(String.class),

	/** The point of time when the event occurred as {@link Date} */
	timestamp(Date.class),

	/** The number that the fax is being send to as {@link String} */
	calledNumber(String.class),

	/** The number of pages already sent as {@link Integer} */
	sentPages(Integer.class),

	/** The number of pages total if known as {@link Integer} (zero otherwise) */
	totalPages(Integer.class);

	private Class<?> type;

	private FaxProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}
}
