package de.starface.integration.uci.java.v22.values;

/**
 * This enum defines the different states a funktion key can take. The constants are used for the
 * funktion key property {@link FunctionKeyProperties#state}. The meaning of the different states
 * depends on the {@link FunctionKeyType} of the function key and is documented per type.
 *
 * @author tilmann
 */
public enum FunctionKeyState {
	/**
	 * This function key is no longer assigned to a function due to a configuration change. This
	 * means that the function key is permanently unusable
	 */
	INVALID, // corresponds to NOTASSIGNED on the server

	/**
	 * This function key's function is currently unavailable. This means that the function key is
	 * temporarily unusable
	 */
	UNAVAILABLE, // corresponds mostly to CONNECTED on the server

	/**
	 * This function key is currently in an inactive state.
	 */
	INACTIVE, // corresponds mostly to OFF on the server

	/**
	 * This function key is currently in a flashy state.
	 */
	FLASHY, // corresponds to BLINKING on the server

	/**
	 * This function key is currently in an active state.
	 */
	ACTIVE; // corresponds to ON on the server

	/////////////
	// ATTENTION: PLEASE ONLY ADD CONSTANTS AT THE END AND DO NOT REARRANGE THE ENUM
	/////////////
}
