/*
 * $Id: CallListEntryProperties.java 466 2011-01-18 18:25:29Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * v21/CallListEntryProperties.java $
 * 
 * $LastChangedDate: 2011-01-18 19:25:29 +0100 (Di, 18 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 466 $
 */
package de.starface.integration.uci.java.v22.values;

import de.starface.integration.uci.java.v22.ucp.messages.requests.UcpVoicemailListRequests;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * This enum holds constants with the property names of the voicemail list as returned by
 * {@link UcpVoicemailListRequests#getVoicemailList(Date, Date, String, String, String, String, int, int)}
 *
 * @author tilmann
 */
public enum VoicemailListProperties {
	/** The entries start before this point of time given as {@link Date} */
	startBefore(Date.class),

	/** The entries start after this point of time given as {@link Date} */
	startAfter(Date.class),

	/**
	 * The entries are restricted to voicemails in the mailbox with the given id as {@link String}.
	 * If an empty String is given no restriction is used.
	 */
	mailboxRestriction(String.class),

	/**
	 * The entries are restricted to voicemails in the given folder defined in
	 * {@link VoicemailListEntryFolder} as {@link String}. If an empty String is given no
	 * restriction is used.
	 */
	folderRestriction(String.class),

	/**
	 * The entries are ordered by the given property. This may be one of the properties in
	 * {@link VoicemailListEntryProperties} as {@link String}
	 */
	orderProperty(String.class),

	/**
	 * The entries are ordered in the given direction. This may be one of the values in
	 * {@link OrderDirection} as {@link String}
	 */
	orderDirection(String.class),

	/**
	 * The offset used for the retrieval of this list regarding the given time constraints and
	 * restrictions as {@link Integer}
	 */
	countOffset(Integer.class),

	/**
	 * The limit used for the retrieval of this list regarding the given time constraints and
	 * restrictions as {@link Integer}
	 */
	countLimit(Integer.class),

	/**
	 * The total count of available entries on the server regarding the given time constraints and
	 * restrictions as {@link Integer}.
	 */
	totalCount(Integer.class),

	/**
	 * The call list entries in this list as {@link List} of {@link Map} with properties as defined
	 * in {@link VoicemailListEntryProperties}
	 */
	entries(List.class);

	private Class<?> type;

	private VoicemailListProperties(Class<?> type) {
		this.type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return type;
	}

}
