/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/starface/integration/uci/ucp/v21/
 * UcpServices.java $
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.integration.uci.java.v30;

import java.util.ArrayList;
import java.util.List;

/**
 * This class contains some utility methods used to convert interfaces to UCI service names and the
 * other way round.
 *
 * @author tilmann
 *
 */
public class UciServices {
	/**
	 * The UCI version of the current set of UCI interfaces
	 */
	public static final int UCI_VERSION = 30;

	private UciServices() {
		// No instances will be made of this
	}

	/**
	 * Retrieves the Name of the UCI service the given interface is tagged with
	 *
	 * @param serviceInterface
	 *        a interface that is tagged with {@link UciService}
	 * @return the value of the {@link UciService} annotation
	 * @throws IllegalArgumentException
	 *         if the given class is not an interface or it is not annotated with {@link UciService}
	 */
	public static String getServiceName(Class<?> serviceInterface) throws IllegalArgumentException {
		if (serviceInterface == null) {
			throw new IllegalArgumentException("serviceInterface=null");
		}
		if (serviceInterface.isInterface()) {
			UciService serviceAnnotation = serviceInterface.getAnnotation(UciService.class);
			if (serviceAnnotation != null) {
				return serviceAnnotation.value();
			}
		}
		throw new IllegalArgumentException("The given type " + serviceInterface.getName()
			+ " is not a valid interface annotated as UciService.");
	}

	/**
	 * Compute a list of string containing the names of all UCI services provided by the given
	 * interfaces and all of their super interfaces.
	 *
	 * @param serviceInterfaces
	 *        any number of interfaces to be scanned for provided services
	 * @return a list containing the service names retrieved from the {@link UciService} annotations
	 *         present in the interfaces
	 */
	public static List<String> getServiceNames(Class<?>... serviceInterfaces) {
		final List<String> serviceNames = new ArrayList<>();
		for (Class<?> serviceClass : serviceInterfaces) {
			if (serviceClass.isInterface()) {
				UciService serviceAnnotation = serviceClass.getAnnotation(UciService.class);
				if (serviceAnnotation != null) {
					serviceNames.add(serviceAnnotation.value());
				}
			}
			Class<?>[] superInterfaces = serviceClass.getInterfaces();
			if (!(superInterfaces.length == 0)) {
				serviceNames.addAll(getServiceNames(superInterfaces));
			}
		}
		return serviceNames;
	}
}
