/*
 * $Id: UcpCallEvents.java 485 2011-01-31 14:52:54Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/client/UcpClientCommunicationCall.java $
 * 
 * $LastChangedDate: 2011-01-31 15:52:54 +0100 (Mo, 31 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 485 $
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.CcbsAccount;
import de.starface.integration.uci.java.v30.types.CcbsCall;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpCcbsEvents;
import de.starface.integration.uci.java.v30.values.CcbsAccountState;

/**
 * This interface defines the conference call events that can be received by a UCI client
 * implementation in order to handle phone conference calls.
 *
 * @author dirk
 */
@UciService(UcpCcbsEvents.SERVICE_NAME)
@RpcInterface(UcpCcbsEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciCcbsCallEvents {

	/**
	 * The receiveCcbsCallState event provides a number of properties about the call the state is
	 * changing of. The information provided in the map is defined by its properties as seen in
	 * {@link CcbsCall}. If a name or number is not known to STARFACE an empty string is committed
	 * for that property. It is possible that information about the caller or the called person gets
	 * available in the middle of a series of events so that the first call state events don’t
	 * provide any details but later events contain full participant information. The sequential
	 * change of the {@link CcbsCall#ccbsCallState} property adheres to the state diagram given in
	 * the UCI specification.
	 *
	 * @param ccbsCall
	 *        the different properties of the ccbsCall call.
	 */
	@RpcAsynchronous
	public void receiveCcbsCallState(CcbsCall ccbsCall);

	/**
	 * The receiveCcbsAccountState event provides a number of properties about the account the state
	 * is changing of. The information provided in the map is defined by its properties as seen in
	 * {@link CcbsAccountState}. If a name or number is not known to STARFACE an empty string is
	 * committed for that property. It is possible that information about the caller or the called
	 * person gets available in the middle of a series of events so that the first call state events
	 * don’t provide any details but later events contain full participant information. The
	 * sequential change of the {@link CcbsAccount#ccbsAccountState} property adheres to the state
	 * diagram given in the UCI specification.
	 *
	 * @param ccbsAccountState
	 *        the different properties of the ccbsAccountState call.
	 */
	@RpcAsynchronous
	public void receiveCcbsAccountState(CcbsAccountState ccbsAccountState);
}
