package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciCallEvents;
import de.starface.integration.uci.java.v30.values.CallState;
import de.starface.integration.uci.java.v30.values.ForwardType;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * This object holds the properties of the call as returned by
 * {@link UciCallEvents#receiveCallState(Call)}.
 *
 * @author tilmann
 */
@RpcValueObject
public class Call implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The id of the call in form of a UUID as {@link String} */
	@RpcValue
	protected String id;

	/** The state of the call */
	@RpcValue
	protected CallState state;

	/** The point of time when the call reached the state */
	@RpcValue
	protected Date timestamp;

	/** The phone number that has been called */
	@RpcValue
	protected String calledNumber = "";

	/** The name of the person being called */
	@RpcValue
	protected String calledName = "";

	/** The number of the caller */
	@RpcValue
	protected String callerNumber = "";

	/** The name of the person calling */
	@RpcValue
	protected String callerName = "";

	/**
	 * The id of the group if this call was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 22)
	protected String groupId = "";

	/** Name of the device **/
	@RpcValue(minVersion = 30)
	protected List<String> peerNames;

	/** The channel name **/
	@RpcValue(minVersion = 30)
	protected List<String> channelNames;

	/** The sip callId **/
	@RpcValue(minVersion = 30)
	protected List<String> sipCallIds;

	/** Is caller **/
	@RpcValue(minVersion = 30)
	protected Boolean caller = Boolean.FALSE;

	/** The point of time when the call is connected */
	@RpcValue(minVersion = 30)
	protected Date connectedTimestamp;

	/** Is in the current call recording activated */
	@RpcValue(minVersion = 30)
	protected Boolean recorded = Boolean.FALSE;

	/** Is in the current call recorded by the user */
	@RpcValue(minVersion = 30)
	protected Boolean recorder = Boolean.FALSE;

	/** Is the user a monitor owner */
	@RpcValue(minVersion = 30)
	protected Boolean recordOwner = Boolean.FALSE;

	/** AvatarHash of Caller */
	@RpcValue(minVersion = 30)
	protected String avatarHash = "";

	/** Doorline Cam of Caller */
	@RpcValue(minVersion = 30)
	protected String doorlineCamUrl = "";

	/** Doorline dtm code */
	@RpcValue(minVersion = 30)
	protected String doorlineDtmfCode = "";

	/** Doorline image provider id */
	@RpcValue(minVersion = 30)
	protected Integer doorlineImageProviderId;

	/** Forward type */
	@RpcValue(minVersion = 30)
	protected ForwardType forwardType = ForwardType.NONE;

	/** Forwarder callerIdName of farwarder */
	@RpcValue(minVersion = 30)
	protected String forwarderCallerIdName = "";

	/** Forwarder callerIdNumber of farwarder */
	@RpcValue(minVersion = 30)
	protected String forwarderCallerIdNumber = "";

	/** The conference room identifikation */
	@RpcValue(minVersion = 30)
	protected String conferenceRoomId = "";

	/** The current call is an conference admin */
	@RpcValue(minVersion = 30)
	protected boolean conferenceAdmin = false;

	/** The reference of the outher side of consultation */
	@RpcValue(minVersion = 30)
	protected String referenceOfConsultation = "";

	/** The time between now and the connected timestamp/timestamp */
	@RpcValue(minVersion = 30)
	protected long duration = 0;

	/** Is this call an internal call */
	@RpcValue(minVersion = 30)
	protected boolean internalCall = false;

	/** Is this call a consultationCall */
	@RpcValue(minVersion = 30)
	protected boolean consultationCall = false;

	/** The jabber id */
	@RpcValue(minVersion = 30)
	protected String jabberId = "";

	/** The account id */
	@RpcValue(minVersion = 30)
	protected String accountId = "";

	public Date getConnectedTimestamp() {
		return connectedTimestamp;
	}

	public void setConnectedTimestamp(Date connectedTimestamp) {
		this.connectedTimestamp = connectedTimestamp;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public CallState getState() {
		return state;
	}

	public void setState(CallState state) {
		this.state = state;
	}

	public Date getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Date timestamp) {
		this.timestamp = timestamp;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public String getCalledName() {
		return calledName;
	}

	public void setCalledName(String calledName) {
		this.calledName = calledName;
	}

	public String getCallerNumber() {
		return callerNumber;
	}

	public void setCallerNumber(String callerNumber) {
		this.callerNumber = callerNumber;
	}

	public String getCallerName() {
		return callerName;
	}

	public void setCallerName(String callerName) {
		this.callerName = callerName;
	}

	public List<String> getChannelNames() {
		return channelNames;
	}

	public void setChannelNames(List<String> channelNames) {
		this.channelNames = channelNames;
	}

	public String getGroupId() {
		return groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public List<String> getPeerNames() {
		return peerNames;
	}

	public void setPeerNames(List<String> peerNames) {
		this.peerNames = peerNames;
	}

	public boolean isGroupCall() {
		return groupId != null && !"".equals(groupId);
	}

	public void setRecorder(Boolean recorder) {
		this.recorder = recorder;
	}

	public void setRecorded(Boolean recorded) {
		this.recorded = recorded;
	}

	public void setRecordOwner(Boolean recordOwner) {
		this.recordOwner = recordOwner;
	}

	public String getAvatarHash() {
		return avatarHash;
	}

	public void setAvatarHash(String avatarHash) {
		this.avatarHash = avatarHash;
	}

	public String getDoorlineCamUrl() {
		return doorlineCamUrl;
	}

	public void setDoorlineCamUrl(String doorlineCamUrl) {
		this.doorlineCamUrl = doorlineCamUrl;
	}

	public String getDoorlineDtmfCode() {
		return doorlineDtmfCode;
	}

	public void setDoorlineDtmfCode(String doorlineDtmfCode) {
		this.doorlineDtmfCode = doorlineDtmfCode;
	}

	public Integer getDoorlineImageProviderId() {
		return doorlineImageProviderId;
	}

	public void setDoorlineImageProviderId(Integer doorlineImageProviderId) {
		this.doorlineImageProviderId = doorlineImageProviderId;
	}

	public List<String> getSipCallIds() {
		return sipCallIds;
	}

	public void setSipCallIds(List<String> sipCallIds) {
		this.sipCallIds = sipCallIds;
	}

	public Boolean isCaller() {
		return caller;
	}

	public void setCaller(Boolean caller) {
		this.caller = caller;
	}

	public ForwardType getForwardType() {
		return forwardType;
	}

	public void setForwardType(ForwardType forwardType) {
		this.forwardType = forwardType;
	}

	public String getForwarderCallerIdName() {
		return forwarderCallerIdName;
	}

	public void setForwarderCallerIdName(String forwarderCallerIdName) {
		this.forwarderCallerIdName = forwarderCallerIdName;
	}

	public String getForwarderCallerIdNumber() {
		return forwarderCallerIdNumber;
	}

	public void setForwarderCallerIdNumber(String forwarderCallerIdNumber) {
		this.forwarderCallerIdNumber = forwarderCallerIdNumber;
	}

	public String getConferenceRoomId() {
		return conferenceRoomId;
	}

	public void setConferenceRoomId(String conferenceRoomId) {
		this.conferenceRoomId = conferenceRoomId;
	}

	public Boolean isRecorder() {
		return recorder;
	}

	public Boolean isRecorded() {
		return recorded;
	}

	public Boolean isRecordOwner() {
		return recordOwner;
	}

	public boolean isConferenceAdmin() {
		return conferenceAdmin;
	}

	public void setConferenceAdmin(boolean conferenceAdmin) {
		this.conferenceAdmin = conferenceAdmin;
	}

	public long getDuration() {
		return duration;
	}

	public void setDuration(long duration) {
		this.duration = duration;
	}

	public boolean isConsultationCall() {
		return consultationCall;
	}

	public void setConsultationCall(boolean consultationCall) {
		this.consultationCall = consultationCall;
	}

	public String getReferenceOfConsultation() {
		return referenceOfConsultation;
	}

	public void setReferenceOfConsultation(String referenceOfConsultation) {
		this.referenceOfConsultation = referenceOfConsultation;
	}

	/**
	 * @return the jabberId
	 */
	public String getJabberId() {
		return jabberId;
	}

	/**
	 * @param jabberId
	 *        the jabberId to set
	 */
	public void setJabberId(String jabberId) {
		this.jabberId = jabberId;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @return Returns the internalCall.
	 */
	public boolean isInternalCall() {
		return internalCall;
	}

	/**
	 * @param internalCall The internalCall to set.
	 */
	public void setInternalCall(boolean internalCall) {
		this.internalCall = internalCall;
	}

	@Override
	public boolean equals(final Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof Call)) {
			return false;
		}

		final Call call = (Call) o;

		if (conferenceAdmin != call.conferenceAdmin) {
			return false;
		}
		if (duration != call.duration) {
			return false;
		}
		if (internalCall != call.internalCall) {
			return false;
		}
		if (consultationCall != call.consultationCall) {
			return false;
		}
		if (id != null ? !id.equals(call.id) : call.id != null) {
			return false;
		}
		if (state != call.state) {
			return false;
		}
		if (timestamp != null ? !timestamp.equals(call.timestamp) : call.timestamp != null) {
			return false;
		}
		if (calledNumber != null ? !calledNumber.equals(call.calledNumber) : call.calledNumber != null) {
			return false;
		}
		if (calledName != null ? !calledName.equals(call.calledName) : call.calledName != null) {
			return false;
		}
		if (callerNumber != null ? !callerNumber.equals(call.callerNumber) : call.callerNumber != null) {
			return false;
		}
		if (callerName != null ? !callerName.equals(call.callerName) : call.callerName != null) {
			return false;
		}
		if (groupId != null ? !groupId.equals(call.groupId) : call.groupId != null) {
			return false;
		}
		if (peerNames != null ? !peerNames.equals(call.peerNames) : call.peerNames != null) {
			return false;
		}
		if (channelNames != null ? !channelNames.equals(call.channelNames) : call.channelNames != null) {
			return false;
		}
		if (sipCallIds != null ? !sipCallIds.equals(call.sipCallIds) : call.sipCallIds != null) {
			return false;
		}
		if (caller != null ? !caller.equals(call.caller) : call.caller != null) {
			return false;
		}
		if (connectedTimestamp != null ? !connectedTimestamp.equals(call.connectedTimestamp) : call.connectedTimestamp != null) {
			return false;
		}
		if (recorded != null ? !recorded.equals(call.recorded) : call.recorded != null) {
			return false;
		}
		if (recorder != null ? !recorder.equals(call.recorder) : call.recorder != null) {
			return false;
		}
		if (recordOwner != null ? !recordOwner.equals(call.recordOwner) : call.recordOwner != null) {
			return false;
		}
		if (avatarHash != null ? !avatarHash.equals(call.avatarHash) : call.avatarHash != null) {
			return false;
		}
		if (doorlineCamUrl != null ? !doorlineCamUrl.equals(call.doorlineCamUrl) : call.doorlineCamUrl != null) {
			return false;
		}
		if (doorlineDtmfCode != null ? !doorlineDtmfCode.equals(call.doorlineDtmfCode) : call.doorlineDtmfCode != null) {
			return false;
		}
		if (doorlineImageProviderId != null ? !doorlineImageProviderId.equals(call.doorlineImageProviderId) : call.doorlineImageProviderId != null) {
			return false;
		}
		if (forwardType != call.forwardType) {
			return false;
		}
		if (forwarderCallerIdName != null ? !forwarderCallerIdName.equals(call.forwarderCallerIdName) : call.forwarderCallerIdName != null) {
			return false;
		}
		if (forwarderCallerIdNumber != null ? !forwarderCallerIdNumber.equals(call.forwarderCallerIdNumber) : call.forwarderCallerIdNumber != null) {
			return false;
		}
		if (conferenceRoomId != null ? !conferenceRoomId.equals(call.conferenceRoomId) : call.conferenceRoomId != null) {
			return false;
		}
		if (referenceOfConsultation != null ? !referenceOfConsultation.equals(call.referenceOfConsultation) : call.referenceOfConsultation != null) {
			return false;
		}
		if (jabberId != null ? !jabberId.equals(call.jabberId) : call.jabberId != null) {
			return false;
		}
		return accountId != null ? accountId.equals(call.accountId) : call.accountId == null;

	}

	@Override
	public int hashCode() {
		int result = id != null ? id.hashCode() : 0;
		result = 31 * result + (state != null ? state.hashCode() : 0);
		result = 31 * result + (timestamp != null ? timestamp.hashCode() : 0);
		result = 31 * result + (calledNumber != null ? calledNumber.hashCode() : 0);
		result = 31 * result + (calledName != null ? calledName.hashCode() : 0);
		result = 31 * result + (callerNumber != null ? callerNumber.hashCode() : 0);
		result = 31 * result + (callerName != null ? callerName.hashCode() : 0);
		result = 31 * result + (groupId != null ? groupId.hashCode() : 0);
		result = 31 * result + (peerNames != null ? peerNames.hashCode() : 0);
		result = 31 * result + (channelNames != null ? channelNames.hashCode() : 0);
		result = 31 * result + (sipCallIds != null ? sipCallIds.hashCode() : 0);
		result = 31 * result + (caller != null ? caller.hashCode() : 0);
		result = 31 * result + (connectedTimestamp != null ? connectedTimestamp.hashCode() : 0);
		result = 31 * result + (recorded != null ? recorded.hashCode() : 0);
		result = 31 * result + (recorder != null ? recorder.hashCode() : 0);
		result = 31 * result + (recordOwner != null ? recordOwner.hashCode() : 0);
		result = 31 * result + (avatarHash != null ? avatarHash.hashCode() : 0);
		result = 31 * result + (doorlineCamUrl != null ? doorlineCamUrl.hashCode() : 0);
		result = 31 * result + (doorlineDtmfCode != null ? doorlineDtmfCode.hashCode() : 0);
		result = 31 * result + (doorlineImageProviderId != null ? doorlineImageProviderId.hashCode() : 0);
		result = 31 * result + (forwardType != null ? forwardType.hashCode() : 0);
		result = 31 * result + (forwarderCallerIdName != null ? forwarderCallerIdName.hashCode() : 0);
		result = 31 * result + (forwarderCallerIdNumber != null ? forwarderCallerIdNumber.hashCode() : 0);
		result = 31 * result + (conferenceRoomId != null ? conferenceRoomId.hashCode() : 0);
		result = 31 * result + (conferenceAdmin ? 1 : 0);
		result = 31 * result + (referenceOfConsultation != null ? referenceOfConsultation.hashCode() : 0);
		result = 31 * result + (int) (duration ^ (duration >>> 32));
		result = 31 * result + (internalCall ? 1 : 0);
		result = 31 * result + (consultationCall ? 1 : 0);
		result = 31 * result + (jabberId != null ? jabberId.hashCode() : 0);
		result = 31 * result + (accountId != null ? accountId.hashCode() : 0);
		return result;
	}

	/**
	 * @see Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder("Call [");
		sb.append("id=").append(id);
		sb.append(", state=").append(state);
		sb.append(", timestamp=").append(timestamp);
		sb.append(", calledNumber=").append(calledNumber);
		sb.append(", calledName=").append(calledName);
		sb.append(", callerNumber=").append(callerNumber);
		sb.append(", callerName=").append(callerName);
		sb.append(", groupId=").append(groupId);
		sb.append(", peerNames=").append(peerNames);
		sb.append(", channelNames=").append(channelNames);
		sb.append(", sipCallIds=").append(sipCallIds);
		sb.append(", caller=").append(caller);
		sb.append(", connectedTimestamp=").append(connectedTimestamp);
		sb.append(", recorded=").append(recorded);
		sb.append(", recorder=").append(recorder);
		sb.append(", recordOwner=").append(recordOwner);
		sb.append(", avatarHash=").append(avatarHash);
		sb.append(", doorlineCamUrl=").append(doorlineCamUrl);
		sb.append(", doorlineDtmfCode=").append(doorlineDtmfCode);
		sb.append(", doorlineImageProviderId=").append(doorlineImageProviderId);
		sb.append(", forwardType=").append(forwardType);
		sb.append(", forwarderCallerIdName=").append(forwarderCallerIdName);
		sb.append(", forwarderCallerIdNumber=").append(forwarderCallerIdNumber);
		sb.append(", conferenceRoomId=").append(conferenceRoomId);
		sb.append(", conferenceAdmin=").append(conferenceAdmin);
		sb.append(", referenceOfConsultation=").append(referenceOfConsultation);
		sb.append(", duration=").append(duration);
		sb.append(", internalCall=").append(internalCall);
		sb.append(", consultationCall=").append(consultationCall);
		sb.append(", jabberId=").append(jabberId);
		sb.append(", accountId=").append(accountId);
		sb.append(']');
		return sb.toString();
	}

	public String getAccountId() {
		return accountId;
	}

	public void setAccountId(final String accountId) {
		this.accountId = accountId;
	}
}
