package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.requests.UciFunctionKeyRequests;
import de.starface.integration.uci.java.v30.values.CallInfoState;

import java.io.Serializable;

/**
 * This object holds the properties of an ongoing call as returned by the request
 * {@link UciFunctionKeyRequests#getCallInfoForKey(String)}
 *
 * @author tilmann
 */
@RpcValueObject
public class CallInfo implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The state of the call */
	@RpcValue
	private CallInfoState state;

	/** The duration the call is going on in seconds */
	@RpcValue
	private int duration;

	/** The number of the other call participant */
	@RpcValue
	private String number = "";

	/** A description of the call (usually the CLIP), if known */
	@RpcValue
	private String description = "";

	/** The accountId of the other call participant */
	@RpcValue
	private Integer accountId = null;

	@Override
	public String toString() {
		return "CallInfo [state=" + state + ", duration=" + duration + ", number=" + number + ", description="
			+ description + ", accountId=" + accountId + "]";
	}

	public CallInfoState getState() {
		return state;
	}

	public void setState(CallInfoState state) {
		this.state = state;
	}

	public int getDuration() {
		return duration;
	}

	public void setDuration(int duration) {
		this.duration = duration;
	}

	public String getNumber() {
		return number;
	}

	public void setNumber(String number) {
		this.number = number;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Integer getAccountId() {
		return accountId;
	}

	public void setAccountId(Integer accountId) {
		this.accountId = accountId;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((accountId == null) ? 0 : accountId.hashCode());
		result = prime * result + ((description == null) ? 0 : description.hashCode());
		result = prime * result + duration;
		result = prime * result + ((number == null) ? 0 : number.hashCode());
		result = prime * result + ((state == null) ? 0 : state.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		CallInfo other = (CallInfo) obj;
		if (accountId == null) {
			if (other.accountId != null) return false;
		} else if (!accountId.equals(other.accountId)) return false;
		if (description == null) {
			if (other.description != null) return false;
		} else if (!description.equals(other.description)) return false;
		if (duration != other.duration) return false;
		if (number == null) {
			if (other.number != null) return false;
		} else if (!number.equals(other.number)) return false;
		if (state != other.state) return false;
		return true;
	}
}
