/*
 * $Id: CallListEntryProperties.java 482 2011-01-25 15:21:54Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * v21/CallListEntryProperties.java $
 * 
 * $LastChangedDate: 2011-01-25 16:21:54 +0100 (Di, 25 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 482 $
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciCallEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciCallListRequests;
import de.starface.integration.uci.java.v30.messages.requests.UciCallRequests;
import de.starface.integration.uci.java.v30.values.CallListEntryDirection;
import de.starface.integration.uci.java.v30.values.CallListEntryResult;

import java.io.Serializable;
import java.util.Date;

/**
 * This object holds the properteis of the call list entries as returned by
 * {@link UciCallListRequests#getCallList(Date, Date, CallListEntryDirection, CallListEntryResult, String, de.starface.integration.uci.java.v30.values.CallListEntryProperties, de.starface.integration.uci.java.v30.values.OrderDirection, int, int)}
 *
 * @author tilmann
 */
@RpcValueObject
public class CallListEntry implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * The id of the call list entry
	 *
	 * ATTENTION: This is not the same id as used in the call functions of {@link UciCallRequests}
	 * and {@link UciCallEvents}
	 */
	@RpcValue
	private String id;

	/** The number that was called, if known */
	@RpcValue
	private String calledNumber = "";

	/** The number of the caller, if known */
	@RpcValue
	private String callerNumber = "";

	/** A description of the call (usually the CLIP), if known */
	@RpcValue
	private String callDescription = "";

	/**
	 * The start time of the call. Per default dates will be ordered newest first
	 */
	@RpcValue
	private Date startTime;

	/** The duration of the call in seconds */
	@RpcValue
	private int duration;

	/**
	 * The direction of the call
	 */
	@RpcValue
	private CallListEntryDirection direction;

	/**
	 * The result of the call
	 */
	@RpcValue
	private CallListEntryResult result;

	/**
	 * The id of a voicemail that was recorded for the call if existing (the empty string otherwise)
	 */
	@RpcValue
	private String voicemailId = "";

	/**
	 * The name of the group if this call was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 22)
	private String groupId = "";

	/**
	 * The Jabber-ID of the caller if the caller is known, empty String otherwise
	 */
	@RpcValue
	private String callerJabberId = "";

	/**
	 * The Account-ID of the call partner if the partner is known, empty String otherwise
	 */
	@RpcValue
	private String callPartnerAccountId = "";

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public String getCallerNumber() {
		return callerNumber;
	}

	public void setCallerNumber(String callerNumber) {
		this.callerNumber = callerNumber;
	}

	public String getCallDescription() {
		return callDescription;
	}

	public void setCallDescription(String callDescription) {
		this.callDescription = callDescription;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public int getDuration() {
		return duration;
	}

	public void setDuration(int duration) {
		this.duration = duration;
	}

	public CallListEntryDirection getDirection() {
		return direction;
	}

	public void setDirection(CallListEntryDirection direction) {
		this.direction = direction;
	}

	public CallListEntryResult getResult() {
		return result;
	}

	public void setResult(CallListEntryResult result) {
		this.result = result;
	}

	public String getVoicemailId() {
		return voicemailId;
	}

	public void setVoicemailId(String voicemailId) {
		this.voicemailId = voicemailId;
	}

	public String getGroupId() {
		return groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public boolean isGroupCall() {
		return groupId != null && !"".equals(groupId);
	}

	public String getCallerJabberId() {
		return callerJabberId;
	}

	public void setCallerJabberId(final String callerJabberId) {
		this.callerJabberId = callerJabberId;
	}

	@Override
	public boolean equals(final Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof CallListEntry)) {
			return false;
		}

		final CallListEntry that = (CallListEntry) o;

		if (duration != that.duration) {
			return false;
		}
		if (id != null ? !id.equals(that.id) : that.id != null) {
			return false;
		}
		if (calledNumber != null ? !calledNumber.equals(that.calledNumber) : that.calledNumber != null) {
			return false;
		}
		if (callerNumber != null ? !callerNumber.equals(that.callerNumber) : that.callerNumber != null) {
			return false;
		}
		if (callDescription != null ? !callDescription.equals(that.callDescription) : that.callDescription != null) {
			return false;
		}
		if (startTime != null ? !startTime.equals(that.startTime) : that.startTime != null) {
			return false;
		}
		if (direction != that.direction) {
			return false;
		}
		if (result != that.result) {
			return false;
		}
		if (voicemailId != null ? !voicemailId.equals(that.voicemailId) : that.voicemailId != null) {
			return false;
		}
		if (groupId != null ? !groupId.equals(that.groupId) : that.groupId != null) {
			return false;
		}
		if (callerJabberId != null ? !callerJabberId.equals(that.callerJabberId) : that.callerJabberId != null) {
			return false;
		}
		return callPartnerAccountId != null ? callPartnerAccountId.equals(that.callPartnerAccountId) : that.callPartnerAccountId == null;

	}

	@Override
	public int hashCode() {
		int result1 = id != null ? id.hashCode() : 0;
		result1 = 31 * result1 + (calledNumber != null ? calledNumber.hashCode() : 0);
		result1 = 31 * result1 + (callerNumber != null ? callerNumber.hashCode() : 0);
		result1 = 31 * result1 + (callDescription != null ? callDescription.hashCode() : 0);
		result1 = 31 * result1 + (startTime != null ? startTime.hashCode() : 0);
		result1 = 31 * result1 + duration;
		result1 = 31 * result1 + (direction != null ? direction.hashCode() : 0);
		result1 = 31 * result1 + (result != null ? result.hashCode() : 0);
		result1 = 31 * result1 + (voicemailId != null ? voicemailId.hashCode() : 0);
		result1 = 31 * result1 + (groupId != null ? groupId.hashCode() : 0);
		result1 = 31 * result1 + (callerJabberId != null ? callerJabberId.hashCode() : 0);
		result1 = 31 * result1 + (callPartnerAccountId != null ? callPartnerAccountId.hashCode() : 0);
		return result1;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder("CallListEntry [");
		sb.append("id=").append(id);
		sb.append(", calledNumber=").append(calledNumber);
		sb.append(", callerNumber=").append(callerNumber);
		sb.append(", callDescription=").append(callDescription);
		sb.append(", startTime=").append(startTime);
		sb.append(", duration=").append(duration);
		sb.append(", direction=").append(direction);
		sb.append(", result=").append(result);
		sb.append(", voicemailId=").append(voicemailId);
		sb.append(", groupId=").append(groupId);
		sb.append(", callerJabberId=").append(callerJabberId);
		sb.append(", callPartnerAccountId=").append(callPartnerAccountId);
		sb.append(']');
		return sb.toString();
	}

	public String getCallPartnerAccountId() {
		return callPartnerAccountId;
	}

	public void setCallPartnerAccountId(final String callPartnerAccountId) {
		this.callPartnerAccountId = callPartnerAccountId;
	}
}
