package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciFaxEvents;
import de.starface.integration.uci.java.v30.values.FaxState;

import java.io.Serializable;
import java.util.Date;

/**
 * This object holds the properties of a fax transmitted in fax state changes by
 * {@link UciFaxEvents#receiveFaxState(Fax)}
 *
 * @author tilmann
 */
@RpcValueObject
public class Fax implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The id of the fax in form of a UUID as {@link String} */
	@RpcValue
	private String id;

	/** The state of the fax */
	@RpcValue
	private FaxState state;

	/** The point of time when the event occurred */
	@RpcValue
	private Date timestamp;

	/** The number that the fax is being send to */
	@RpcValue(minVersion = 22)
	private String calledNumber = "";

	/** The number of pages already sent */
	@RpcValue(minVersion = 22)
	private int sentPages;

	/** The number of pages total if known (zero otherwise) */
	@RpcValue(minVersion = 22)
	private int totalPages;

	@Override
	public String toString() {
		return "Fax [id=" + id + ", state=" + state + ", timestamp=" + timestamp
			+ ", calledNumber=" + calledNumber + ", sentPages=" + sentPages + ", totalPages="
			+ totalPages + "]";
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public FaxState getState() {
		return state;
	}

	public void setState(FaxState state) {
		this.state = state;
	}

	public Date getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Date timestamp) {
		this.timestamp = timestamp;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public int getSentPages() {
		return sentPages;
	}

	public void setSentPages(int sentPages) {
		this.sentPages = sentPages;
	}

	public int getTotalPages() {
		return totalPages;
	}

	public void setTotalPages(int totalPages) {
		this.totalPages = totalPages;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((calledNumber == null) ? 0 : calledNumber.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + sentPages;
		result = prime * result + ((state == null) ? 0 : state.hashCode());
		result = prime * result + ((timestamp == null) ? 0 : timestamp.hashCode());
		result = prime * result + totalPages;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Fax other = (Fax) obj;
		if (calledNumber == null) {
			if (other.calledNumber != null) {
				return false;
			}
		} else if (!calledNumber.equals(other.calledNumber)) {
			return false;
		}
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		if (sentPages != other.sentPages) {
			return false;
		}
		if (state != other.state) {
			return false;
		}
		if (timestamp == null) {
			if (other.timestamp != null) {
				return false;
			}
		} else if (!timestamp.equals(other.timestamp)) {
			return false;
		}
		if (totalPages != other.totalPages) {
			return false;
		}
		return true;
	}
}
