package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.requests.UciFunctionKeyRequests;
import de.starface.integration.uci.java.v30.values.FunctionKeyState;
import de.starface.integration.uci.java.v30.values.FunctionKeyType;

import java.io.Serializable;

/**
 * This object holds the properties of the function keys as returned by
 * {@link UciFunctionKeyRequests#getFunctionKeys()}.
 *
 * @author tilmann
 */
@RpcValueObject
public class FunctionKey implements Positionable, Serializable {
	private static final long serialVersionUID = 1L;

	/** The id of the function key */
	@RpcValue
	private String id;

	/** The display name of the function key */
	@RpcValue
	private String name;

	/** The state of the function key with one of the constants of {@link FunctionKeyState} */
	@RpcValue
	private FunctionKeyState state;

	/** The type of the function key with one of the constants of {@link FunctionKeyType} */
	@RpcValue
	private FunctionKeyType type;

	/**
	 * The Xmpp chat id of the corresponding user.
	 */
	@RpcValue(minVersion = 30)
	private String chatId;

	/**
	 * The email address of the corresponding account.
	 */
	@RpcValue(minVersion = 30)
	private String email;

	/**
	 * The phone number of the corresponding account.
	 */
	@RpcValue(minVersion = 30)
	private String phoneNumber;

	/**
	 * The second phone number of the corresponding account.
	 */
	@RpcValue(minVersion = 30)
	private String phoneNumber2;

	/**
	 * The mobile phone number of the corresponding account.
	 */
	@RpcValue(minVersion = 30)
	private String mobilePhoneNumber;

	/**
	 * The home phone number of the corresponding account.
	 */
	@RpcValue(minVersion = 30)
	private String homePhoneNumber;

	/**
	 * The internal number of the corresponding account.
	 */
	@RpcValue(minVersion = 30)
	private String internalPhoneNumber;

	/**
	 * The fax number of the corresponding account.
	 */
	@RpcValue(minVersion = 30)
	private String fax;

	/**
	 * The user state of the function key if the key is of type
	 * {@link FunctionKeyType#BUSY_LAMP_FIELD}
	 */
	@RpcValue
	private UserState userState;

	/**
	 * The hash of the key's image if an image is assigned to the key the image itself can then be
	 * requested using {@link UciFunctionKeyRequests#getImageForKey(String)}
	 */
	@RpcValue
	private String imageHash = "";

	/** The position of the key in the users function key settings */
	@RpcValue
	private int position;

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public FunctionKeyState getState() {
		return state;
	}

	public void setState(FunctionKeyState state) {
		this.state = state;
	}

	public FunctionKeyType getType() {
		return type;
	}

	public void setType(FunctionKeyType type) {
		this.type = type;
	}

	public UserState getUserState() {
		return userState;
	}

	public void setUserState(UserState userState) {
		this.userState = userState;
	}

	public String getImageHash() {
		return imageHash;
	}

	public void setImageHash(String imageHash) {
		this.imageHash = imageHash;
	}

	public int getPosition() {
		return position;
	}

	public void setPosition(int position) {
		this.position = position;
	}

	/**
	 * @return the chatId
	 */
	public String getChatId() {
		return chatId;
	}

	/**
	 * @param chatId
	 *        the chatId to set
	 */
	public void setChatId(String chatId) {
		this.chatId = chatId;
	}

	/**
	 * @return the email
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * @param email
	 *        the email to set
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * @return the phoneNumber
	 */
	public String getPhoneNumber() {
		return phoneNumber;
	}

	/**
	 * @param phoneNumber
	 *        the phoneNumber to set
	 */
	public void setPhoneNumber(String phoneNumber) {
		this.phoneNumber = phoneNumber;
	}

	/**
	 * @return the phoneNumber2
	 */
	public String getPhoneNumber2() {
		return phoneNumber2;
	}

	/**
	 * @param phoneNumber2
	 *        the phoneNumber2 to set
	 */
	public void setPhoneNumber2(String phoneNumber2) {
		this.phoneNumber2 = phoneNumber2;
	}

	/**
	 * @return the mobilePhoneNumber
	 */
	public String getMobilePhoneNumber() {
		return mobilePhoneNumber;
	}

	/**
	 * @param mobilePhoneNumber
	 *        the mobilePhoneNumber to set
	 */
	public void setMobilePhoneNumber(String mobilePhoneNumber) {
		this.mobilePhoneNumber = mobilePhoneNumber;
	}

	/**
	 * @return the homePhoneNumber
	 */
	public String getHomePhoneNumber() {
		return homePhoneNumber;
	}

	/**
	 * @param homePhoneNumber
	 *        the homePhoneNumber to set
	 */
	public void setHomePhoneNumber(String homePhoneNumber) {
		this.homePhoneNumber = homePhoneNumber;
	}

	/**
	 * @return the internalPhoneNumber
	 */
	public String getInternalPhoneNumber() {
		return internalPhoneNumber;
	}

	/**
	 * @param internalPhoneNumber
	 *        the internalPhoneNumber to set
	 */
	public void setInternalPhoneNumber(String internalPhoneNumber) {
		this.internalPhoneNumber = internalPhoneNumber;
	}

	/**
	 * @return the fax
	 */
	public String getFax() {
		return fax;
	}

	/**
	 * @param fax
	 *        the fax to set
	 */
	public void setFax(String fax) {
		this.fax = fax;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chatId == null) ? 0 : chatId.hashCode());
		result = prime * result + ((email == null) ? 0 : email.hashCode());
		result = prime * result + ((fax == null) ? 0 : fax.hashCode());
		result = prime * result + ((homePhoneNumber == null) ? 0 : homePhoneNumber.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((imageHash == null) ? 0 : imageHash.hashCode());
		result = prime * result + ((internalPhoneNumber == null) ? 0 : internalPhoneNumber.hashCode());
		result = prime * result + ((mobilePhoneNumber == null) ? 0 : mobilePhoneNumber.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((phoneNumber == null) ? 0 : phoneNumber.hashCode());
		result = prime * result + ((phoneNumber2 == null) ? 0 : phoneNumber2.hashCode());
		result = prime * result + position;
		result = prime * result + ((state == null) ? 0 : state.hashCode());
		result = prime * result + ((type == null) ? 0 : type.hashCode());
		result = prime * result + ((userState == null) ? 0 : userState.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		FunctionKey other = (FunctionKey) obj;
		if (chatId == null) {
			if (other.chatId != null) {
				return false;
			}
		} else if (!chatId.equals(other.chatId)) {
			return false;
		}
		if (email == null) {
			if (other.email != null) {
				return false;
			}
		} else if (!email.equals(other.email)) {
			return false;
		}
		if (fax == null) {
			if (other.fax != null) {
				return false;
			}
		} else if (!fax.equals(other.fax)) {
			return false;
		}
		if (homePhoneNumber == null) {
			if (other.homePhoneNumber != null) {
				return false;
			}
		} else if (!homePhoneNumber.equals(other.homePhoneNumber)) {
			return false;
		}
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		if (imageHash == null) {
			if (other.imageHash != null) {
				return false;
			}
		} else if (!imageHash.equals(other.imageHash)) {
			return false;
		}
		if (internalPhoneNumber == null) {
			if (other.internalPhoneNumber != null) {
				return false;
			}
		} else if (!internalPhoneNumber.equals(other.internalPhoneNumber)) {
			return false;
		}
		if (mobilePhoneNumber == null) {
			if (other.mobilePhoneNumber != null) {
				return false;
			}
		} else if (!mobilePhoneNumber.equals(other.mobilePhoneNumber)) {
			return false;
		}
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		if (phoneNumber == null) {
			if (other.phoneNumber != null) {
				return false;
			}
		} else if (!phoneNumber.equals(other.phoneNumber)) {
			return false;
		}
		if (phoneNumber2 == null) {
			if (other.phoneNumber2 != null) {
				return false;
			}
		} else if (!phoneNumber2.equals(other.phoneNumber2)) {
			return false;
		}
		if (position != other.position) {
			return false;
		}
		if (state != other.state) {
			return false;
		}
		if (type != other.type) {
			return false;
		}
		if (userState == null) {
			if (other.userState != null) {
				return false;
			}
		} else if (!userState.equals(other.userState)) {
			return false;
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "FunctionKey [id=" + id + ", name=" + name + ", state=" + state + ", type=" + type + ", chatId="
			+ chatId + ", email=" + email + ", phoneNumber=" + phoneNumber + ", phoneNumber2=" + phoneNumber2
			+ ", mobilePhoneNumber=" + mobilePhoneNumber + ", homePhoneNumber=" + homePhoneNumber
			+ ", internalPhoneNumber=" + internalPhoneNumber + ", fax=" + fax + ", userState=" + userState
			+ ", imageHash=" + imageHash + ", position=" + position + "]";
	}
}
