/*
 * $Id: MailboxProperties.java 477 2011-01-24 14:32:18Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * v21/MailboxProperties.java $
 * 
 * $LastChangedDate: 2011-01-24 15:32:18 +0100 (Mo, 24 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 477 $
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.requests.UciGroupRequests;

import java.io.Serializable;
import java.util.List;

/**
 * This object holds the properties of the group settings as returned by
 * {@link UciGroupRequests#getGroupSettings()}
 *
 * @author tilmann
 */
@RpcValueObject
public class GroupSetting implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The id of the group */
	@RpcValue
	private String id;

	/** The display name of the group for the current user */
	@RpcValue
	private String name;

	/** A flag indicating if the user is currently logged on to the group or not */
	@RpcValue
	private boolean loggedOn;

	/** The Id that can be used to log on to the group using the telephone */
	@RpcValue
	private String logonId;

	/**
	 * The phone numbers allocated to the group
	 */
	@RpcValue
	private List<PhoneNumber> phoneNumbers;

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public List<PhoneNumber> getPhoneNumbers() {
		return phoneNumbers;
	}

	public void setPhoneNumbers(List<PhoneNumber> phoneNumbers) {
		this.phoneNumbers = phoneNumbers;
	}

	@Override
	public String toString() {
		return "GroupSetting [id=" + id + ", name=" + name + ", loggedOn=" + loggedOn
			+ ", logonId=" + logonId + ", phoneNumbers=" + phoneNumbers + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + (loggedOn ? 1231 : 1237);
		result = prime * result + ((logonId == null) ? 0 : logonId.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((phoneNumbers == null) ? 0 : phoneNumbers.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		GroupSetting other = (GroupSetting) obj;
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		if (loggedOn != other.loggedOn) {
			return false;
		}
		if (logonId == null) {
			if (other.logonId != null) {
				return false;
			}
		} else if (!logonId.equals(other.logonId)) {
			return false;
		}
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		if (phoneNumbers == null) {
			if (other.phoneNumbers != null) {
				return false;
			}
		} else if (!phoneNumbers.equals(other.phoneNumbers)) {
			return false;
		}
		return true;
	}

	public boolean isLoggedOn() {
		return loggedOn;
	}

	public void setLoggedOn(boolean loggedOn) {
		this.loggedOn = loggedOn;
	}

	public String getLogonId() {
		return logonId;
	}

	public void setLogonId(String logonId) {
		this.logonId = logonId;
	}

}
