/*
 * $Id: CallListEntryProperties.java 466 2011-01-18 18:25:29Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * v21/CallListEntryProperties.java $
 * 
 * $LastChangedDate: 2011-01-18 19:25:29 +0100 (Di, 18 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 466 $
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.requests.UciMonitorListRequests;
import de.starface.integration.uci.java.v30.values.MonitorListEntryProperties;
import de.starface.integration.uci.java.v30.values.OrderDirection;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * This object holds the properties of the monitor list as returned by
 * {@link UciMonitorListRequests#getMonitorList(Date, Date, MonitorListEntryProperties, OrderDirection, int, int)}
 *
 * @author tilmann
 */
@RpcValueObject
public class MonitorList implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The entries start before this point of time */
	@RpcValue
	private Date startBefore;

	/** The entries start after this point of time */
	@RpcValue
	private Date startAfter;

	/**
	 * The entries are ordered by the given property. This may be one of the properties in
	 * {@link MonitorListEntryProperties}
	 */
	@RpcValue
	private MonitorListEntryProperties orderProperty;

	/**
	 * The entries are ordered in the given direction. This may be one of the values in
	 * {@link OrderDirection}
	 */
	@RpcValue
	private OrderDirection orderDirection;

	/**
	 * The offset used for the retrieval of this list regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int countOffset;

	/**
	 * The limit used for the retrieval of this list regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int countLimit;

	/**
	 * The total count of available entries on the server regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int totalCount;

	/**
	 * The call list entries in this list
	 */
	@RpcValue
	private List<MonitorListEntry> entries;

	/**
	 * @return the startBefore
	 */
	public Date getStartBefore() {
		return startBefore;
	}

	/**
	 * @param startBefore
	 *        the startBefore to set
	 */
	public void setStartBefore(Date startBefore) {
		this.startBefore = startBefore;
	}

	/**
	 * @return the startAfter
	 */
	public Date getStartAfter() {
		return startAfter;
	}

	/**
	 * @param startAfter
	 *        the startAfter to set
	 */
	public void setStartAfter(Date startAfter) {
		this.startAfter = startAfter;
	}

	/**
	 * @return the orderProperty
	 */
	public MonitorListEntryProperties getOrderProperty() {
		return orderProperty;
	}

	/**
	 * @param orderProperty
	 *        the orderProperty to set
	 */
	public void setOrderProperty(MonitorListEntryProperties orderProperty) {
		this.orderProperty = orderProperty;
	}

	/**
	 * @return the orderDirection
	 */
	public OrderDirection getOrderDirection() {
		return orderDirection;
	}

	/**
	 * @param orderDirection
	 *        the orderDirection to set
	 */
	public void setOrderDirection(OrderDirection orderDirection) {
		this.orderDirection = orderDirection;
	}

	/**
	 * @return the countOffset
	 */
	public int getCountOffset() {
		return countOffset;
	}

	/**
	 * @param countOffset
	 *        the countOffset to set
	 */
	public void setCountOffset(int countOffset) {
		this.countOffset = countOffset;
	}

	/**
	 * @return the countLimit
	 */
	public int getCountLimit() {
		return countLimit;
	}

	/**
	 * @param countLimit
	 *        the countLimit to set
	 */
	public void setCountLimit(int countLimit) {
		this.countLimit = countLimit;
	}

	/**
	 * @return the totalCount
	 */
	public int getTotalCount() {
		return totalCount;
	}

	/**
	 * @param totalCount
	 *        the totalCount to set
	 */
	public void setTotalCount(int totalCount) {
		this.totalCount = totalCount;
	}

	/**
	 * @return the entries
	 */
	public List<MonitorListEntry> getEntries() {
		return entries;
	}

	/**
	 * @param entries
	 *        the entries to set
	 */
	public void setEntries(List<MonitorListEntry> entries) {
		this.entries = entries;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + countLimit;
		result = prime * result + countOffset;
		result = prime * result + ((entries == null) ? 0 : entries.hashCode());
		result = prime * result + ((orderDirection == null) ? 0 : orderDirection.hashCode());
		result = prime * result + ((orderProperty == null) ? 0 : orderProperty.hashCode());
		result = prime * result + ((startAfter == null) ? 0 : startAfter.hashCode());
		result = prime * result + ((startBefore == null) ? 0 : startBefore.hashCode());
		result = prime * result + totalCount;
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		MonitorList other = (MonitorList) obj;
		if (countLimit != other.countLimit) {
			return false;
		}
		if (countOffset != other.countOffset) {
			return false;
		}
		if (entries == null) {
			if (other.entries != null) {
				return false;
			}
		} else if (!entries.equals(other.entries)) {
			return false;
		}
		if (orderDirection != other.orderDirection) {
			return false;
		}
		if (orderProperty != other.orderProperty) {
			return false;
		}
		if (startAfter == null) {
			if (other.startAfter != null) {
				return false;
			}
		} else if (!startAfter.equals(other.startAfter)) {
			return false;
		}
		if (startBefore == null) {
			if (other.startBefore != null) {
				return false;
			}
		} else if (!startBefore.equals(other.startBefore)) {
			return false;
		}
		if (totalCount != other.totalCount) {
			return false;
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "MonitorList [startBefore=" + startBefore + ", startAfter=" + startAfter + ", orderProperty="
			+ orderProperty + ", orderDirection=" + orderDirection + ", countOffset=" + countOffset
			+ ", countLimit=" + countLimit + ", totalCount=" + totalCount + ", entries=" + entries + "]";
	}

}
