/*
 * $$Id$$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on ${date} by ${user}
 * 
 * $$HeadURL$$
 * 
 * $$LastChangedDate$$ $$LastChangedBy$$ $$LastChangedRevision$$
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.values.SoftPhoneNatType;

import java.io.Serializable;

/**
 * This object holds information to identify and register a SoftPhone
 *
 * @author tpfeifer
 *
 */
@RpcValueObject
public class SoftPhoneSetting implements Serializable {
	private static final long serialVersionUID = 1L;

	public static final String DEFAULT_CODECS = "alaw, ulaw";
	public static final SoftPhoneNatType DEFAULT_NATTYPE = SoftPhoneNatType.DEFAULT;

	@RpcValue(minVersion = 30)
	private String softPhoneId;

	@RpcValue(minVersion = 30)
	private String codecs;

	@RpcValue(minVersion = 30)
	private SoftPhoneNatType natType;

	public SoftPhoneSetting(String softPhoneId, String codecs, SoftPhoneNatType natType) {
		super();
		this.softPhoneId = softPhoneId;
		this.codecs = codecs;
		this.natType = natType;
	}

	public SoftPhoneSetting(String softPhoneId) {
		this(softPhoneId, DEFAULT_CODECS, DEFAULT_NATTYPE);
	}

	public SoftPhoneSetting() {
		this("");
	}

	/**
	 * @return Returns the softPhoneType.
	 */
	public String getSoftPhoneId() {
		return softPhoneId;
	}

	/**
	 * @param softPhoneId
	 *        The softPhoneType to set.
	 */
	public void setSoftPhoneId(String softPhoneId) {
		this.softPhoneId = softPhoneId;
	}

	/**
	 * @return Returns the codecs.
	 */
	public String getCodecs() {
		return codecs;
	}

	/**
	 * @param codecs
	 *        The codecs to set.
	 */
	public void setCodecs(String codecs) {
		this.codecs = codecs;
	}

	/**
	 * @return Returns the natType.
	 */
	public SoftPhoneNatType getNatType() {
		return natType;
	}

	/**
	 * @param natType
	 *        The natType to set.
	 */
	public void setNatType(SoftPhoneNatType natType) {
		this.natType = natType;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((codecs == null) ? 0 : codecs.hashCode());
		result = prime * result + ((natType == null) ? 0 : natType.hashCode());
		result = prime * result + ((softPhoneId == null) ? 0 : softPhoneId.hashCode());
		return result;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		SoftPhoneSetting other = (SoftPhoneSetting) obj;
		if (codecs == null) {
			if (other.codecs != null) {
				return false;
			}
		} else if (!codecs.equals(other.codecs)) {
			return false;
		}
		if (natType != other.natType) {
			return false;
		}
		if (softPhoneId == null) {
			if (other.softPhoneId != null) {
				return false;
			}
		} else if (!softPhoneId.equals(other.softPhoneId)) {
			return false;
		}
		return true;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "SoftPhoneSetting [softPhoneId=" + softPhoneId + ", codecs=" + codecs + ", natType=" + natType
			+ "]";
	}

}
