/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/client/UcpClientCommunicationCall.java $
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.types.CallIdChanges;
import de.starface.integration.uci.java.v30.types.MessageWaitingIndicationInfo;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpCallRequests;
import de.starface.integration.uci.java.v30.values.CallProperties;

import java.util.Map;

/**
 * This interface defines the call events that can be received by a UCI client implementation in
 * order to handle phone calls.
 *
 * @author tilmann
 */
@RpcInterface(UcpCallEvents.SERVICE_NAME)
@UciService(UcpCallEvents.SERVICE_NAME)
public interface UcpCallEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.call";

	/**
	 * The ReceiveCallState event provides a number of properties about the call the state is
	 * changing of. The information provided in the map is defined by its properties as seen in
	 * {@link CallProperties}. If a name or number is not known to STARFACE an empty string is
	 * committed for that property. It is possible that information about the caller or the called
	 * person gets available in the middle of a series of events so that the first call state events
	 * don’t provide any details but later events contain full participant information. The
	 * sequential change of the {@link CallProperties#state} property adheres to the state diagram
	 * given in the UCI specification. It is assumed after a successful return of the
	 * {@link UcpCallRequests#placeCallWithPhone(String, String, String)} request.
	 *
	 * @param callProperties
	 *        the different properties of the call in a {@link Map}. The constants of
	 *        {@link CallProperties} are used as keys in the map.
	 */
	@RpcAsynchronous
	public void receiveCallState(Map<String, Object> callProperties);

	/**
	 * This event provides Information about new VoicemailMessages to the UCI client
	 *
	 * @param messageWaitingIndicationInfo
	 *        the new messageWaitingIndication {@link MessageWaitingIndicationInfo}
	 */
	@RpcAsynchronous
	public void newMessageWaitingIndicationEvent(Map<String, Object> messageWaitingIndicationInfo);

	/**
	 * This event provides Information about the new callId of a call to the UCI client
	 *
	 * @param callIdChangesEvent
	 *        the callChangesIdEvent {@link CallIdChanges}
	 */
	@RpcAsynchronous
	public void callChangesIdEvent(Map<String, Object> callIdChangesEvent);
}
