/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/client/UcpClientCommunicationCall.java $
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.types.CallListEntry;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the call list events that can be received by a UCI client implementation
 * in order to handle call list entries.
 * 
 * @author dwendland
 */
@RpcInterface(UcpCallListEvents.SERVICE_NAME)
@UciService(UcpCallListEvents.SERVICE_NAME)
public interface UcpCallListEvents {
	
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.callList";
	
	/**
	 * This event allows the UCI client to get informed about new call list entries being available.
	 * 
	 * @param callListEntry the new call list entry with a {@link Map} containing the values defined
	 *        in {@link CallListEntry}
	 */
	@RpcAsynchronous
	public void newCallListEntry(Map<String, Object> callListEntry);
	
	/**
	 * This event allows the UCI client to get informed about deleted call list entries.
	 *
	 * @param deletedCallListEntryIds
	 *        a {@link List} containing the ids of deleted call list entries
	 */
	@RpcAsynchronous
	public void deletedCallListEntries(List<String> deletedCallListEntryIds);
}
