/*
 * $Id: UcpCallEvents.java 721 2013-04-30 12:36:07Z dirk $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/client/UcpClientCommunicationCall.java $
 * 
 * $LastChangedDate: 2013-04-30 14:36:07 +0200 (Di, 30. Apr 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 721 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpCallRequests;
import de.starface.integration.uci.java.v30.values.CallProperties;
import de.starface.integration.uci.java.v30.values.ConferenceCallProperties;

import java.util.Map;

/**
 * This interface defines the call events that can be received by a UCI client implementation in
 * order to handle phone calls.
 *
 * @author tilmann
 */
@RpcInterface(UcpConferenceCallEvents.SERVICE_NAME)
@UciService(UcpConferenceCallEvents.SERVICE_NAME)
public interface UcpConferenceCallEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.conferencecall";

	/**
	 * The ReceiveConfrenceCallState event provides a number of properties about the call the state
	 * is changing of. The information provided in the map is defined by its properties as seen in
	 * {@link ConferenceCallProperties}. If a name or number is not known to STARFACE an empty
	 * string is committed for that property. It is possible that information about the caller or
	 * the called person gets available in the middle of a series of events so that the first call
	 * state events don’t provide any details but later events contain full participant information.
	 * The sequential change of the {@link CallProperties#state} property adheres to the state
	 * diagram given in the UCI specification. It is assumed after a successful return of the
	 * {@link UcpCallRequests#placeCallWithPhone(String, String, String)} request.
	 *
	 * @param conferenceCallProperties
	 *        the different properties of the call in a {@link Map}. The constants of
	 *        {@link ConferenceCallProperties} are used as keys in the map.
	 */
	@RpcAsynchronous
	public void receiveConfrenceCallState(Map<String, Object> conferenceCallProperties);

}
