/*
 * $Id$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/client/UcpClientCommunicationFax.java $
 * 
 * $LastChangedDate$ $LastChangedBy$
 * $LastChangedRevision$
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpFaxRequests;
import de.starface.integration.uci.java.v30.values.FaxProperties;
import de.starface.integration.uci.java.v30.values.FaxState;

import java.util.Map;

/**
 * This interface defines fax events that can be received by a UCI client implementation in order to
 * handle outgoing faxes.
 *
 * @author tilmann
 */
@RpcInterface(UcpFaxEvents.SERVICE_NAME)
@UciService(UcpFaxEvents.SERVICE_NAME)
public interface UcpFaxEvents {
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.fax";

	/**
	 * Much the same as {@link UcpCallEvents#receiveCallState(Map)} the ReceiveFaxState event
	 * notifies the UCI client of state changes of an outgoing fax request by providing several
	 * properties about the fax request including the id of the fax request, the new state of the
	 * fax request and the timestamp when the request did change its state. The state
	 * {@link FaxState#WAITING} is never signaled by the server. It is assumed after a successful
	 * call to {@link UcpFaxRequests#sendFax(String, String, String, String)}.
	 *
	 * @param faxProperties
	 *        the different properties of the fax in a {@link Map}. The constants of
	 *        {@link FaxProperties} are used as keys in the map.
	 */
	@RpcAsynchronous
	public void receiveFaxState(Map<String, Object> faxProperties);
}
