/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE SOFTWARE. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 *  
 *
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.types.MonitorListEntry;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the monitor list events that can be received by a UCI client implementation in
 * order to handle monitor list entries.
 */
@RpcInterface(UcpMonitorListEvents.SERVICE_NAME)
@UciService(UcpMonitorListEvents.SERVICE_NAME)
public interface UcpMonitorListEvents
{
	
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.monitorList";
	
	/**
	 * This event allows the UCI client to get informed about new monitor list entries being available.
	 *
	 * @param monitorListEntry
	 *        the new monitor list entry with a {@link Map} containing the values defined in
	 *        {@link MonitorListEntry}
	 */
	@RpcAsynchronous
	public void newMonitorListEntry(MonitorListEntry monitorListEntry);
	
	/**
	 * This event allows the UCI client to get informed about deleted monitor list entries.
	 *
	 * @param deletedMonitorEntryIds
	 *        a {@link List} containing the ids of deleted monitor entries
	 */
	@RpcAsynchronous
	public void deletedMonitorListEntries(List<String> deletedMonitorEntryIds);
}
