/*
 * $$Id$$
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on ${date} by ${user}
 * 
 * $$HeadURL$$
 * 
 * $$LastChangedDate$$ $$LastChangedBy$$ $$LastChangedRevision$$
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.types.SoftPhoneRegistrationInfo;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the phone events that can be received by a UCI client implementation in
 * order to handle phone registration.
 *
 * @author tpfeifer
 */
@RpcInterface(UcpPhoneEvents.SERVICE_NAME)
@UciService(UcpPhoneEvents.SERVICE_NAME)
public interface UcpPhoneEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.phone";

	/**
	 * This event provides Information about new SoftPhone registrations thats SIP credentials has
	 * been requested by the UCI client. Another UCI client might be just about to use these
	 * credentials.
	 *
	 * @param softPhoneRegistrationInfo
	 *        the new softPhoneRegistration {@link SoftPhoneRegistrationInfo}
	 */
	@RpcAsynchronous
	public void newSoftPhoneRegisteredEvent(Map<String, Object> softPhoneRegistrationInfo);

	/**
	 * This event provides Information about changes of telephones of this user.
	 *
	 * @param phones
	 *        a list of all phones of this user.
	 */
	@RpcAsynchronous
	public void phonesChangedEvent(List<String> phones);

	/**
	 * This event provides Information about changes of the primary phone of this user.
	 *
	 * @param phone
	 *        primary phone
	 */
	@RpcAsynchronous
	public void primaryPhoneChangedEvent(String phone);
}
