/*
 * $Id: UcpQueueRequests.java 528 2011-03-18 12:02:13Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationCall.java $
 * 
 * $LastChangedDate: 2011-03-18 13:02:13 +0100 (Fr, 18 Mrz 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 528 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.types.UciQueueCall;
import de.starface.integration.uci.java.v30.types.UserState;
import de.starface.integration.uci.java.v30.values.QueueSettingProperties;

import java.util.List;
import java.util.Map;

/**
 * This interface defines the Queue events that can be received by a UCI client implementation in
 * order to handle Queue changes.
 *
 * @author tilmann
 */
@UciService(UcpQueueEvents.SERVICE_NAME)
@RpcInterface(UcpQueueEvents.SERVICE_NAME)
public interface UcpQueueEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.queue";

	/**
	 * This event allows the UCI client to get informed when the state or number of the queue
	 * setting changes.
	 *
	 * @param queueSetting
	 *        the new state of the Queue setting with values for the properties defined in
	 *        {@link QueueSettingProperties}.
	 */
	@RpcAsynchronous
	public void queueSettingChanged(Map<String, Object> queueSetting);

	/**
	 * One of the agents states has changed. This can be the {@link UserState} or the loginstate at
	 * the queue, as well as a name change.
	 *
	 * @param agentStates
	 */
	@RpcAsynchronous
	public void queueAgentStateChanged(List<Map<String, Object>> agentStates);

	/**
	 * The queue has a new agent or an agent was removed. Please reload the agent list for this
	 * queue.
	 *
	 * @param groupAccountId
	 */
	@RpcAsynchronous
	public void queueAgentChangedEvent(Integer groupAccountId);

	/**
	 * A new queue has been created (parameter newGroup = true) or a queue was deleted (parameter
	 * newGroup = false)
	 *
	 * @param newGroup
	 *        true if a queuegroup was created, false if a queuegroup was deleted
	 * @param groupAccountId
	 *        the accountid of the queuegroup
	 */
	@RpcAsynchronous
	public void queueGroupChangedEvent(boolean newGroup, Integer groupAccountId);

	/**
	 * Updates the UCI client about a new queue call or a statechange of a queue call.
	 *
	 * @param call
	 *        contains all the new/updated information about this queue call.
	 */
	@RpcAsynchronous
	public void queueCallEvent(UciQueueCall call);

	/**
	 * During a call, the id of the call can change (e.g. during a transfer). If this happens, the
	 * existing call must be updated. Ignoring this event may lead to "visually frozen" calls in
	 * your client, because all new events will reference the new id.
	 *
	 * @param oldId
	 *        the former id of the call
	 * @param newId
	 *        the new id of the call
	 */
	@RpcAsynchronous
	public void queueCallChangedIdEvent(String oldId, String newId);
}
