/*
 * $Id: UcpCallRequests.java 721 2013-04-30 12:36:07Z dirk $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationCall.java $
 * 
 * $LastChangedDate: 2013-04-30 14:36:07 +0200 (Di, 30. Apr 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 721 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.CcbsCall;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.values.CcbsAccountState;

import java.util.List;

/**
 * This interface defines the call requests provided by the UCI server.
 *
 * @author tilmann
 */
@UciService(UcpCcbsCallRequests.SERVICE_NAME)
@RpcInterface(UcpCcbsCallRequests.SERVICE_NAME)
public interface UcpCcbsCallRequests {
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.ccbscall";

	/**
	 * This request tells the UCI server to remove an available ccbs call by the given UUID.
	 *
	 * @param callId
	 *        the UUID of the call to be transfered.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void removeAvailableCcbs(String callId) throws UciException;

	/**
	 * This request tells the UCI server to start or quit ccbs to the given call identified by the
	 * given UUID.
	 *
	 * @param callId
	 *        the UUID of the call to be transfered.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public boolean ccbs(String callId) throws UciException;

	/**
	 * This request tells the UCI server to get the CcbsAccountState.
	 *
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public CcbsAccountState getCcbsAccountState() throws UciException;

	/**
	 * This request tells the UCI server to get all current ccbs calls.
	 *
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public List<CcbsCall> getAllCcbsCalls() throws UciException;
}
