/*
 * Copyright (C) by STARFACE Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on Oct 31, 2014 by pburlov
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;

import java.util.List;
import java.util.Map;

@UciService(UcpSystemRequests.SERVICE_NAME)
@RpcInterface(UcpSystemRequests.SERVICE_NAME)
public interface UcpSystemRequests {
	public static final String SERVICE_NAME = "ucp.v30.requests.system";

	/**
	 *
	 * @param parent NAme der Rechtegruppe. Leeren String für alle Rechte. Darf nicht null sein.
	 * @return Liste mit allen freigeschalteten Rechten (aus der Gruppe)
	 * @throws UciException
	 */
	List<String> getPermissions(String parent) throws UciException;

	/**
	 * Fragt den Server eine bestimmte lizenzierbare Funktionalitaet fuer den angemeldeten
	 * benutzer freizuschalten. Wenn es serverseitig ausreichende Lizenzen vorhanden sind dann wird
	 * die Funktion/Recht automatisch für den Benutzer aktiviert ohne dass ein Admin in dem Adminbereich
	 * es explizit freischalten muss.
	 * @param permission Rechtename, darf nicht null sein
	 * @return true wenn Funktionalitaet/Recht fuer den Benutzer jetzt freigeschaltet ist
	 * @throws UciException
	 */
	boolean requestPermission(String permission) throws UciException;

	Map<String, Object> getGlobalLineConfiguration() throws UciException;

	/**
	 * Liefert die Versionsnummer des verbundenen Servers
	 *
	 * @return Version des Servers
	 * @throws UciException
	 */
	public String getServerVersion() throws UciException;

	/**
	 * Liefert die Webserverkonfiguration des verbundenen Servers
	 * @return Webserverkonfiguration
	 * @throws UciException
	 */
	public Map<String, Object> getWebServerConfiguration() throws UciException;

	/**
	 * Retrieves the server timezone id as used by java.util.TimeZone. See the IANA time zone database for further
	 * information. All Date-Objects have to be adjusted to match this timezone since timezone information are not 
	 * supported by xml-rpc.
	 * For example "Europe/Berlin".
	 * @return Id of the servers TimeZone
	 * @throws UciException
	 */
	public String getTimeZoneId() throws UciException;
}
