package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpFunctionKeyRequests;

import java.util.Map;

/**
 * This enum holds constants with the property names of the function keys as returned by
 * {@link UcpFunctionKeyRequests#getFunctionKeys()}.
 *
 * @author tilmann
 */
public enum FunctionKeyProperties {
	/** The id of the function key as {@link String} */
	id(String.class),

	/** The display name of the function key as {@link String} */
	name(String.class),

	/**
	 * The state of the function key with one of the constants of {@link FunctionKeyState} as
	 * {@link String}
	 */
	state(String.class),

	/**
	 * The type of the function key with one of the constants of {@link FunctionKeyType} as
	 * {@link String}
	 */
	type(String.class),

	/**
	 * The Xmpp chat id of the corresponding user as {@link String}
	 */
	chatId(String.class),

	/**
	 * The user state of the function key if the key is of type
	 * {@link FunctionKeyType#BUSY_LAMP_FIELD} as a Map containing values for the keys defined in
	 * {@link UserStateProperties}
	 */
	userState(Map.class),

	/**
	 * The hash of the key's image if an image is assigned to the key the image itself can then be
	 * requested using {@link UcpFunctionKeyRequests#getImageForKey(String)}
	 */
	imageHash(String.class),

	/** The position of the key in the users function key settings */
	position(Integer.class),

	/**
	 * The email address of the corresponding account.
	 */
	email(Integer.class),

	/**
	 * The phone number of the corresponding account.
	 */
	phoneNumber(String.class),

	/**
	 * The second phone number of the corresponding account.
	 */
	phoneNumber2(String.class),

	/**
	 * The mobile phone number of the corresponding account.
	 */
	mobilePhoneNumber(String.class),

	/**
	 * The home phone number of the corresponding account.
	 */
	homePhoneNumber(String.class),

	/**
	 * The internal number of the corresponding account.
	 */
	internalPhoneNumber(String.class),

	/**
	 * The fax number of the corresponding account.
	 */
	fax(String.class);

	private Class<?> _type;

	private FunctionKeyProperties(Class<?> type) {
		this._type = type;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return _type;
	}
}
